/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.samba;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

/**
 * @author eriks
 *
 * This class is designed to read and write ini files (like smb.conf), while
 * attempting to preserve as much as possible. Modifying or removing a section will 
 * lose all comments within that section. Modified sections will show up in the
 * original section location.
 * 
 * ini files look something like this:
 * ; comments must begin the line according to smb.conf
 * # comment
 * # global section
 * [global]
 *     # key/value pairs under each section
 *     description = true
 *     bob = "foo"
 * 
 * # another section
 * [section3]
 *     betty = "boop"
 * 
 */

public class IniPreserve {
	private String fileName;
	private ArrayList fileLines;
	private ArrayList sections; // Mapping of section headers and the lines they're on
	private String tab="    "; // Use this as our tab for inserting keys
	
	// read the lines of the file into a local array, and initialize 'sections'
	public IniPreserve(String iniFileName) throws FileNotFoundException, IOException {
		fileName = iniFileName;
        try
        {
            BufferedReader br = new BufferedReader(
                                    new InputStreamReader(new FileInputStream(fileName), "UTF-8"));

            String line;
            fileLines = new ArrayList();
            while ((line = br.readLine()) != null) {
                fileLines.add(line);
            }
            br.close();
            findSections();

        } catch (UnsupportedEncodingException e) {
            throw new IOException("Unsupported encoding exception in IniPreserve");
	    }
        }

	// find all the sections in the array, and save for later
	private void findSections() {
		IniSection currentSection = null;
		
		sections = new ArrayList();
		ListIterator lineIterator = fileLines.listIterator();
		String line;
		while (lineIterator.hasNext()) {
			line = (String)lineIterator.next();
			
			// clean up the line
			line.trim();
			if (line.length() == 0 || // empty/whitespace line
				line.startsWith(";") || //comment line
				line.startsWith("#") // comment line
				) { continue; }
			
			if (line.startsWith("[")) { // section
				int endOfSectionName = line.indexOf("]");
				int startOfSectionName = 1;
				String sectionName = line.substring(startOfSectionName,endOfSectionName);
				currentSection = new IniSection(sectionName);
				currentSection.setStartLine(lineIterator.previousIndex());
				currentSection.setEndLine(lineIterator.previousIndex());
				sections.add(currentSection);
			} else { // must be a key value pair
				if (null == currentSection) { continue; } // line before section? 
				int equals;
				if ((equals = line.indexOf("=")) != -1) {
					int beginningOfLine = 0;
					String key = line.substring(beginningOfLine, equals);
					String value = line.substring(equals+1);
					key = key.trim();
					value = value.trim();
					currentSection.putKey(key,value);
					currentSection.setEndLine(lineIterator.previousIndex());
				} else {
					// bad line? skip it.
					continue;
				}
			}
		}
	}
	
	// add a new empty section at the end of the array
	public IniSection addSection(String sectionName) throws IniSectionAlreadyExists {
		IniSection ini;
		
		try {
			ini = getSection(sectionName);
			throw new IniSectionAlreadyExists();
		} catch (IniSectionNotFound e) {
			// This is good, we don't want it to exist!
		}
		fileLines.add("");
		fileLines.add("[" + sectionName + "]");
		findSections();
		try {
            ini = getSection(sectionName);
        } catch (IniSectionNotFound e) {
            throw new AssertionError("IniSectionNotFound exception right after adding it. Ieeeeeeeee!");
        }
        return ini;
	}

	// remove a section from the array
	public void removeSection(String sectionName) {
		try {
            IniSection section = getSection(sectionName);
    		for (int i = section.getStartLine(); i <= section.getEndLine(); i++) {
	    		fileLines.remove(section.getStartLine()); //remove the first line over and over
		    }
            // remove any trailing empty lines
            int startLine = section.getStartLine();
            while( startLine < fileLines.size() && ((String)fileLines.get(startLine)).trim().length() == 0 )
                fileLines.remove(startLine);

        } catch (IniSectionNotFound e) {
            // section is already gone - quietly ignore?
        }
		findSections();
	}

	// return a hash full of the key/value pairs in a given section
	public IniSection getSection(String sectionName) throws IniSectionNotFound {
		ListIterator sectionIterator = sections.listIterator();
		while (sectionIterator.hasNext()) {
			IniSection section = (IniSection)sectionIterator.next();
			if (section.getName().equals(sectionName)) {
				return section;
			}
		}
        throw new IniSectionNotFound();
	}
	
	// replaces the section in the array	
	public void replaceSection(IniSection section) {
		removeSection(section.getName());
		Iterator keyIterator = section.getKeys().keySet().iterator();
		int insertLine = section.getStartLine();
		fileLines.add(insertLine++, "[" + section.getName() + "]");
		while (keyIterator.hasNext()) {
			String keyName = (String)keyIterator.next();
			String keyValue = (String)section.getKeys().get(keyName);
			String line = tab + keyName + " = " + keyValue;
			fileLines.add(insertLine++, line);
		}
// this eventually leads to a lot of leftover whitespace
//		fileLines.add(insertLine++, "");
		findSections();
	}
	
	// make sure the section name is legal
	// we don't allow the character %
	// or any of our reserved section names
	public boolean legalSectionName(String section) {
		String illegalWords[] = { "global", "ipc$", "admin$", "print$",
								"homes", "printers", "centeris$", "c$" };
		
		int notFound = -1;
		if (section.indexOf("%") != notFound) { return false; }

		int stringsMatch = 0;
		for (int i = 0; i < illegalWords.length; i++) {
			if (stringsMatch == section.compareToIgnoreCase(illegalWords[i])) {
				return false;
			}
		}
		return true;
	}
	
	
	// save the array back to disk
	public void commit() {
		ListIterator lineIterator = fileLines.listIterator();
		String line;
        try {
    		PrintStream ps = new PrintStream(new FileOutputStream(fileName), true, "UTF-8");
            while (lineIterator.hasNext()) {
		    	line = (String)lineIterator.next();
                ps.println(line);
            }
            ps.close();
        } catch (FileNotFoundException e) {
            throw new AssertionError("FileNotFoundException writing to config file?");
		} catch (UnsupportedEncodingException e2) {
            throw new AssertionError("UnsupportedEncodingException");
	}
  }
}
