/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

/*
 * This program is used by samba for 'addprinter/deleteprinter/enumports command'
 * This program gets used like:
 * SambaPrint add <path_to_smb_conf> <printername> <sharename> <portname> <driver> <location> <comment>
 * SambaPrint delete <path_to_smb_conf> <printername>
 * SambaPrint enumports
 * 
 * This one doesn't seem to be quite as well supported as add/remove shares. Some issues discovered:
 * - there is no way to set the printer name? - http://lists.samba.org/archive/samba/2004-May/086510.html
 * - samba is magically storing the location
 * - it looks like samba is magically mapping drivers for us
 * - the last few arguments are different from what the man page lists for addprinter
 * - they don't bother passing the smb.conf, so we'll add that as an argument manually
 * - printername on delete is really sharename, since they're the same in samba (see above)
 * - we're only going to save sharename and comment in the smb.conf (and the device portname pointed at)
 * - samba is passing in an extra argument ("2") to enumports
 */

package com.centeris.webservice.samba;

import com.centeris.webservice.utils.ExecException;
import com.centeris.webservice.utils.ExecUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.*;

public class SambaPrint {
	private static String commandName;
	private static String configFile;
	private static String printerName;
	private static String shareName;
	private static String portName;
	private static String driver;
	private static String location;
	private static String comment;
	private static IniPreserve ini = null;
	private static final String printerXMLFileName = "printer-mapping.xml";
    private static final String sambaDirName = "/etc/samba";
	private static InputStream printerXMLFileStream;
	
	
	public static void main(String[] args) {
        if (args.length < 1) {
            printSyntaxAndExit();
        }

		commandName = getArg(args,0).toLowerCase();
		configFile = getArg(args,1);
		printerName = getArg(args,2);
		shareName = getArg(args,3);
		portName = getArg(args,4);
		driver = getArg(args,5);
		location = getArg(args,6);
		comment = getArg(args,7);

        final ClassLoader currentClassLoader = SambaPrint.class.getClassLoader();

	    // see if the printer-mapping file exists in the /etc/samba directory

        // get the full pathname for the xml file
        // printerXMLFileStream = currentClassLoader.getResourceAsStream(printerXMLFileName);

        printerXMLFileStream = getPrinterMappingFile();

        if (printerXMLFileStream == null) {
            System.out.println("Cannot open "+printerXMLFileName);
			System.exit(1);
        }
        // stick this up here because samba is being stupid and passing in an extra "2" where i would expect configFile
		if ( commandName.equals("enumports") ) { enumportsCommand(); System.exit(0); }
		
		if (configFile != null) {
			try {
				ini = new IniPreserve(configFile);
			} catch (FileNotFoundException e) {
				System.out.println("File not found.");
				System.exit(1);
			} catch (IOException e) {
				System.out.println("Error reading from file");
				System.exit(1);
			}
		}
		
		// skipping number of args checking because samba is not passing the same args that are documented in the man page
		if ( commandName.equals("delete") ) { deleteCommand(); }
		else if ( commandName.equals("add") ) { addCommand(); }
		else { printSyntaxAndExit(); }
		ini.commit();
	}

    // return an input stream that comes from the printer mapping file in /etc/samba, or from the resource in the
    // jar, if it can't use that one. Note that a side effect of NOT finding the printer-mapping file in /etc/samba
    // is that printer-mapping.xml is extracted from the jar and placed in that directory...

    private static InputStream getPrinterMappingFile() {
        File f = new File(sambaDirName + "/" + printerXMLFileName);
        //System.out.println("looking for file " + sambaDirName + "/" + printerXMLFileName);
        if (!f.exists()) {
            // System.out.println(sambaDirName + "/" + printerXMLFileName + " doesn't exist, trying to copy");
            if (!copyOutPrintMap()) {
                final ClassLoader currentClassLoader = SambaPrint.class.getClassLoader();
                return currentClassLoader.getResourceAsStream(printerXMLFileName);
            }
        }
        try {
            InputStream fis = new FileInputStream(sambaDirName + "/" + printerXMLFileName);
            return fis;
        } catch (FileNotFoundException e) {
            System.out.println("Cannot open " + sambaDirName + "/" + printerXMLFileName + " for input.");

        }
        return null;
    }

    // Copy out the printer-mapping.xml resource to the /etc/samba directory, and use it from there.
    private static boolean copyOutPrintMap() {
        InputStream printerMappingStream;
        OutputStream fileOutputStream;
        int c;

        final ClassLoader currentClassLoader = SambaPrint.class.getClassLoader();

        try {
            fileOutputStream = new FileOutputStream(sambaDirName + "/" + printerXMLFileName);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot open " + sambaDirName + "/" + printerXMLFileName + " for write.");
            return false;
        }
        //System.out.println("Opened " + sambaDirName + "/" + printerXMLFileName + " for write.");
// get the full pathname for the xml file      c
        printerMappingStream = currentClassLoader.getResourceAsStream(printerXMLFileName);

        try {
            while ((c = printerMappingStream.read()) != -1) {
                fileOutputStream.write(c);
            }
            printerMappingStream.close();
            fileOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Cannot write " + printerXMLFileName);
        }
        return true;
    }

	private static void addCommand() {

        if (!ini.legalSectionName(shareName)) {
            System.exit(1);
        }
        IniSection newSection = null;
        String xpath = "/printer-mapping/printer[display-name='" + portName + "']/dev-name/text()";
        String deviceNames[] = (searchXML(printerXMLFileStream, xpath)); // we only want the first record of the array

        // add the print queue to cups
        // lpadmin -p printername -v devicename -E -m raw
        if (deviceNames.length == 0) {
            System.out.println("There is no port named '" + portName + "'\n");
            System.exit(1);
        }
        try {
            ExecUtils.runCommand("lpadmin -p " + shareName + " -v " + deviceNames[0] + " -E -m raw");
        } catch (ExecException ee) {
            System.out.println("STDOUT: " + ee.getStdOut() + "\n" + "STDERR: " + ee.getStdErr());
            System.exit(ee.getRetCode());
        }

        try {
            newSection = ini.addSection(shareName);
        } catch (IniSectionAlreadyExists e) {
            // BUG 377 - change the existing (e.g. rename)
            try {
                newSection = ini.getSection(shareName);
            } catch (IniSectionNotFound e2) {
                System.out.println("Can't find existing share " + shareName);
                System.exit(1);
            }
        }

        newSection.putKey("printable", "yes");
        newSection.putKey("printer name", shareName);
        newSection.putKey("path", "/var/spool/samba");
        newSection.putKey("comment", comment);
        ini.replaceSection(newSection);


//		samba expects the portname to get spit back out at the end, otherwise you get 'permission denied'
        System.out.println(portName);
    }
		
	private static void deleteCommand() {
		// delete the printer from cups
		// lpadmin -x printername
		// TODO: if lpadmin fails, we can choose to not update the smb.conf. This will give 'access denied'.
		// TODO: but maybe it's better to ignore the failure and update the smb.conf anyway?
		try
        {
            ExecUtils.runCommand("lpadmin -x " + printerName);
        }
        catch(ExecException ee)
        {
            System.out.println("STDOUT: " + ee.getStdOut() + "\n" + "STDERR: " + ee.getStdErr());
			System.exit(ee.getRetCode());
		}
		ini.removeSection(printerName);
	}
	
	private static void enumportsCommand() {
		String xpath = "/printer-mapping/printer/display-name/text()";
		String displayNames[] = searchXML(printerXMLFileStream, xpath);

		for (int i=0; i< displayNames.length; i++) {
			System.out.println(displayNames[i]);
		}
		System.exit(0);
	}

	private static String[] searchXML(InputStream xmlFile, String xpathExpression) {
		String xpathResults[] = null;
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse(xmlFile);

			try {
		        NodeList nodelist = org.apache.xpath.XPathAPI.selectNodeList(doc, xpathExpression);

				xpathResults = new String[nodelist.getLength()];
		        for (int i=0; i<nodelist.getLength(); i++) {
		        		Node displayName = nodelist.item(i);
		        		xpathResults[i] = displayName.toString();
		        }
		    } catch (javax.xml.transform.TransformerException e) {
		    		System.out.println("Error in xpath query");
		    		System.exit(1);
		    }
		
		
		} catch (ParserConfigurationException e) {
			System.out.println("Error parsing " + printerXMLFileName);
			System.exit(1);
		} catch (SAXException e) {
			System.out.println("Error parsing " + printerXMLFileName);
			System.exit(1);
		} catch (IOException e) {
			System.out.println("Error reading file " + printerXMLFileName);
			System.exit(1);
		}
		return xpathResults;
	}
	

	// return the index into the array, or null if we're out of range
	private static String getArg(String[] args, int index) {
		if (args.length > index) {
			return args[index];
		} else {
			return null;
		}
	}
	
	private static void printSyntaxAndExit() {
		System.out.println("Invalid Syntax");
		System.out.println("Syntax:");
		System.out.println("SambaPrint add <path_to_smb_conf> <printername> <sharename> <portname> <driver> <location> <comment>");
		System.out.println("SambaPrint delete <path_to_smb_conf> <printername>");
		System.out.println("SambaPrint enumports");
		System.exit(1);
	}
}
