/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

/*
 * This is an installer for the SambaShare class
 * It will add the appropriate lines to an smb.conf
 * run it like 'java -cp /centeris/SambaCommands.jar SambaShareInstaller /etc/smb.conf /centeris/SambaCommands.jar'
 * (it doesn't copy the jar file, just modifies the conf file) 
 */

package com.centeris.webservice.samba;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;
import java.util.Date;

import com.centeris.webservice.utils.*;

public class SambaShareInstaller {

	public static void main(String[] args) {
		if (args.length != 3){
			printSyntaxAndExit();
		}
        String smbConfPath = args[0];
        String sambaCommandsPath = args[1];
        String workgroup = args[2];

        try{
            doInstall(smbConfPath, sambaCommandsPath, workgroup );
        }catch (IOException e){
            System.out.println(e.toString());
        }
    }

    public static void doInstall( String smbConfPath, String sambaCommandsPath, String workgroup ) throws FileNotFoundException, IOException {

        IniPreserve ini = null;
        // open the smb.ini file
        ini = new IniPreserve(smbConfPath);


        // backup the file before we play with it
        FileUtils.copy(new File(smbConfPath), new File("/usr/share/likewise/conf/smb.conf.bak." + new Date().getTime()));
        createGlobalSection(ini, smbConfPath, sambaCommandsPath, workgroup );

        // we don't want the printers section!
        // we're setting up individual printer sections manually instead
        ini.removeSection("printers");

        // create our special samba shares.
        // These are:
        //  c$		->  /usr/share/centeris/data
        //  LikewiseOpenClient ->
        //                  /usr/share/likewise/open-agent/client
        //  centeris$	->  /usr/share/centeris/conf
        //     print$	->  /usr/share/centeris/print
        //
        addNewSystemShare(ini,
                          "c$",
                          "/usr/share/centeris/data",
                          "Likewise Open Agent Data Share");

        addNewSystemShare(ini,
                          "centeris$",
                          "/usr/share/centeris/conf",
                          "Likewise Open Agent Administrative Files");

        addNewSystemShare(ini,
                          "print$",
                          "/usr/share/centeris/print",
                          "Likewise Open Agent Printer Drivers");

        addNewSystemShare(ini,
                          "LikewiseOpenClient",
                          "/usr/share/likewise/open-agent/client",
                          "Likewise Open Agent Client Software");

        ini.commit();

        try {
            // make the smb.conf available via samba for backup by the user
            LinuxCommand.symbolicLink("/usr/share/centeris/conf/smb.conf", "/etc/samba/smb.conf");
        } catch (ExecException e) {
            // TODO: Log Install Warning
        }
    }


    /**
	 * Writes new [global] settings to smb.conf, overwriting
	 * any settings already tehre.
	 *
	 * @param ini
	 * @param smbConfPath
	 * @param sambaCommandsPath
	 */
	private static void createGlobalSection(IniPreserve ini,
                                            String smbConfPath,
                                            String sambaCommandsPath,
                                            String workgroup ) {
		// add lines to smb.conf for SambaShare
		final String GLOBAL_SECTION_NAME = "global";

		ini.removeSection(GLOBAL_SECTION_NAME);

		IniSection section;
		try {
			section = ini.addSection(GLOBAL_SECTION_NAME);
		} catch (IniSectionAlreadyExists e) {
			// note: this ahouldn't happen, because we
			// just deleted the section.
			//
			throw new RuntimeException(e);
		}

		section.putKey("workgroup", workgroup );
		section.putKey("printcap cache time", "750");
		section.putKey("username map", "/etc/samba/smbusers");
		section.putKey("encrypt passwords", "yes");
		// section.putKey("printer admin", "root");
		section.putKey("admin users", "root");
		section.putKey("security", "user");
		section.putKey("printcap name", "cups");
        section.putKey("create mask","0777"); // Bug 300a
        section.putKey("directory mask","0775"); // Bug 300a
        section.putKey("store dos attributes","yes"); // Bug 1308

		// allow smb/winbind to cache the first lookup for a given host.
		// We do this so that we can avoid problems with domains that have 2 dc's.
		// For some period after we join, one dc knows we joined, the other doesnt.
		// We want to use the one that we joined up with when we start our relationship.
		//
		// this might have adverse side effects unrelated to dc relationships but i cant
		// think of any.

		section.putKey("name cache timeout", "600");

//      don't allow guest in
//		section.putKey("map to guest", "Bad User");

		section.putKey("cups options", "raw");
		section.putKey("printing", "cups");
		section.putKey("read only", "no");
		section.putKey("client schannel", "no");
		

		// Event Log facilitators

		section.putKey("eventlog list",
			       "Application Security System SyslogLinux");

		// Service Control Facilitators
		section.putKey("enable svcctl","cups");

        // Bug 217 -- proper handling of the $admin share
        section.putKey("enable asu support", "no");

		// Samba Shutdown commands

		section.putKey("shutdown script",
			       "/usr/share/likewise/bin/shutdown %t %z %x %m %r");
		section.putKey("abort shutdown script",
			       "/usr/share/likewise/bin/abort_shutdown");

		final String sambaShareCommand = "sambaShare";
		final String sambaPrintCommand = "sambaPrint";

		// SambaShare commands
		section.putKey("add share command", sambaCommandsPath + sambaShareCommand + " add");
		section.putKey("change share command", sambaCommandsPath + sambaShareCommand + " change");
		section.putKey("delete share command", sambaCommandsPath + sambaShareCommand + " delete");

		// SambaPrint commands
		section.putKey("enumports command", sambaCommandsPath + sambaPrintCommand + " enumports");
		section.putKey("addprinter command", sambaCommandsPath + sambaPrintCommand + " add " + smbConfPath);
		section.putKey("deleteprinter command", sambaCommandsPath + sambaPrintCommand + " delete " + smbConfPath);

		ini.replaceSection(section);

	}

	/**
	 * Creates a new samba share with the given name, replacing any existing
	 * share that already exists.
	 *
	 * @param ini		An open handle to our smb.conf file.
	 * @param shareName	The name of the share, as exposed on the network.	
	 * @param sharePath	The path to the directory to be shared, on the local
	 * 					filesystem.
	 * @param comment	A text comment describing the share.  This will appear
	 *					in a SMB browser.
	 */
	private static void addNewSystemShare(IniPreserve ini,
                                          String shareName,
                                          String sharePath,
                                          String comment) {

        // in the smb.conf file, the share name is the section name
        //
        ini.removeSection(shareName);

        IniSection sectionNewShare;
        try {
            sectionNewShare = ini.addSection(shareName);
        } catch (IniSectionAlreadyExists e) {
            // note: this ahouldn't happen, because we
            // just deleted the section.
            //
            throw new RuntimeException(e);
        }
        // add the share path, note that we check for empty/null 'cause
        // we occasionally need to add sections that dont have paths.

        if (null != sharePath && !"".equals(sharePath))
            sectionNewShare.putKey("path", sharePath);

        if (null != comment && !"".equals(comment))
            sectionNewShare.putKey("comment", comment);

        sectionNewShare.putKey("path", sharePath);
        sectionNewShare.putKey("comment", comment);

        // 050623ju appears to be the minimal correct settings
        sectionNewShare.putKey("write list", "root");
        sectionNewShare.putKey("create mask", "0777");   // Bug 1314
        sectionNewShare.putKey("directory mask", "0775");
        sectionNewShare.putKey("read only", "no");

        ini.replaceSection(sectionNewShare);
    }

	private static void printSyntaxAndExit() {
		System.out.println("SambaShareInstaller <path to smb.conf> <path to SambaCommands.jar> <workgroup>");
		System.exit(1);
	}
}
