/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.systemmanager;
import com.centeris.webservice.webinterface.SystemInfoWS;
import com.centeris.webservice.samba.*;
import com.centeris.webservice.utils.*;

import java.io.*;
import java.rmi.RemoteException;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class SystemInfoImplWS implements SystemInfoWS
{
    // Provides the mechanism to discover the magic 'foo-release' file that
    // indicates the name and version of a given release of linux, where
    // foo=redhat, SuSE, etc.

    private class FilterFileRelease implements java.io.FileFilter
    {
	public boolean accept(File f)
	{
	    // we dont want to look at sub directories
	    if(f.isDirectory()) return false;
	    // BUGBUG:
	    // as of today, we just want the OS release file so
	    // we dont want to look at the LSB file
	    if(f.getName().endsWith("lsb-release")) return false;
	    return f.getName().endsWith("-release");
	}
    }

    protected final String suSeRelease = "/etc/SuSE-release";

    public String likewiseOpenAgentVersion() throws RemoteException, ExecException
    {
      String sRun = "rpm -q likewiseopenagt";

      String version = ExecUtils.runCommand(sRun).getStdOut();
      return (null == version) ? null : version.trim();
    }



// return an array of strings that looks like this:
//	The format of the file at this url is:
//		 
//		First 4 lines: System section
//		Next 4 lines: Registered to section
//		Next 5 lines: Computer section
//		 
//	--------- begin file -------- (not part of the file)
//	Centeris 1.2.3
//	Suse 1.3.4.5
//	 
//	 
//	Demo Customer
//	 
//	 
//	 
//	Intel Pentium FoodaMonda 3.50
//	CPU Speed 1.42
//	2.5 GB of RAM
//	 
//	 
//	----- end of file -----
    public String[] centerisSystemInfo() throws RemoteException
    {
        String SystemInfo[] = new String[13];
        String hostname;

        String linuxRelease = null;
        try
        {
            // the first line of /etc/SuSE-release contains something like:
            // SuSE Linux 9.1 (i586)

	    // invoke our release file filter on a search of the etc
	    // directory.  using the filter and the file list allows this
	    // section of code to work with any linux release.
	    // NB:
	    // listFiles returns an array, but we know that there will only
	    // be one member of the array, so we just ref 0

	    File fRelease = new File("/etc").listFiles(new FilterFileRelease())[0];

	    BufferedReader brRelease = new BufferedReader(new FileReader(fRelease));

	    // grab the first line from the release file for future use.
	    linuxRelease = brRelease.readLine();

	    brRelease.close();
        }
        catch(FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }

        BufferedReader cpuinfoFile;
        String cpuModel = null;
        String cpuSpeed = null;
        try
        {
            // we want the following lines out of /proc/cpuinfo:
            // model name      : Intel(R) Pentium(R) 4 CPU 3.00GHz
            // cpu MHz         : 3002.885
            cpuinfoFile = new BufferedReader(new FileReader("/proc/cpuinfo"));
            String line;
            while((line = cpuinfoFile.readLine()) != null)
            {
                if(line.startsWith("model name"))
                {
                    cpuModel = line.substring(line.indexOf(":") + 2);
                }
                if(line.startsWith("cpu MHz"))
                {
                    cpuSpeed = line.substring(line.indexOf(":") + 2);
                }
            }
            cpuinfoFile.close();
        }
        catch(FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }


        BufferedReader meminfoFile;
        String memAmount = null;
        try
        {
            // we want the following line out of /proc/meminfo:
            // MemTotal:       255812 kB
            cpuinfoFile = new BufferedReader(new FileReader("/proc/meminfo"));
            String line;
            while((line = cpuinfoFile.readLine()) != null)
            {
                if(line.startsWith("MemTotal"))
                {
                    // yuck. turn memAmount into megs from k
                    memAmount = (line.substring(line.indexOf(":") + 1,
                                                line.length() - 2)).trim();
                    int mem = Integer.parseInt(memAmount);
                    memAmount = Integer.toString(mem / 1024);
                }
            }
            cpuinfoFile.close();
        }
        catch(FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }


        try
        {
            SystemInfo[0] = "Likewise Open Agent " + likewiseOpenAgentVersion();
        }
        catch( Exception e )
        {
            SystemInfo[0] = "Likewise Open Agent version unknown";
        }
        // Bug 658
        try {
                InetAddress addr = InetAddress.getLocalHost();
                byte[] ipAddr=addr.getAddress();
                hostname = addr.getHostName();
        } catch (UnknownHostException e) {
            hostname="127.0.0.1";
        }

        SystemInfo[1] = linuxRelease;
        SystemInfo[2] = "";
        SystemInfo[3] = "";
        SystemInfo[4] = "root@"+hostname;
        SystemInfo[5] = "";
        SystemInfo[6] = "";
        SystemInfo[7] = "";
        SystemInfo[8] = cpuModel; // /proc/cpuinfo
        SystemInfo[9] = "CPU Speed " + cpuSpeed + "MHz"; // /proc/cpuinfo
        SystemInfo[10] = memAmount + " Megabytes of RAM"; // /proc/meminfo
        SystemInfo[11] = "";
        SystemInfo[12] = "";

        return SystemInfo;
    }

    /**
     * Generate an XML description of the devices on the server hosting this agent.
     * @return A String containing the XML description of the devices on the server hosting this agent.
     * (eg.)
     *  "<?xml version=\"1.0\" ?>\n" +
     *  "<deviceTree>\n" +
     *  "  <group>\n" +
     *  "    <name>Processors</name>\n" +
     *  "    <type>1</type>\n" +
     *  "    <device>\n" +
     *  "      <name>" + cpuModel + "</name>\n" +
     *  "    </device>\n" +
     *  "  </group>\n" +
     *  "  <group>\n" +
     *  "    <name>Drives</name>\n" +
     *  "    <type>2</type>\n" +
     *  "    <device>\n" +
     *  "      <name>" + diskName + "</name>\n" +
     *  "    </device>\n" +
     *  "  </group>\n" +
     *  "</deviceTree>\n";
     * @throws RemoteException
     * @throws ExecException
     */
        public String getDeviceTree()
            throws RemoteException, ExecException
        {
            String hwinfo = "/usr/share/likewise/bin/hwinfo_filter.pl";
            return ExecUtils.runCommand(hwinfo).getStdOut();        
	}




  // function for slurping facts out of the smb.conf, if smb.conf doesnt exist, dont throw an exception, return
  // an expository string
  // TODO We should go back to throwing the exception - the string is a workaround for getting called when there is
  //      no samba installed....
  //
  // TODO Make all callers of this function declare the same set of exceptions. i am not doing that today because i am
  //      loathe to mess with the WSDL at this point.


  private String getThingConfigured(String sThing2Get) throws FileNotFoundException, IOException, IniSectionNotFound
  {
    String smbConfPath = "/etc/samba/smb.conf";

    if(!new File(smbConfPath).isFile())  return "smb.conf missing";

    return (String) new IniPreserve(smbConfPath).getSection("global").getKeys().get(sThing2Get);
  }

  // Poke thru the smb.conf and see if we are configured as a domain
  // member. This asserts nothing about our actual membership at any
  // given moment. use testDomain() for that if we dont have an
  // smb.conf configured, we

  public String getConfiguredDomain() throws RemoteException, Exception
  {
    String smbItem = "realm";
    try
    {
      return getThingConfigured(smbItem);
    }
    catch(Exception e){
      throw new Exception("Error getting: " + smbItem + " from smb.conf",e);
    }
  }


  // Poke thru smb.conf and extract the description string

  public String getConfiguredDescription() throws RemoteException, IOException, FileNotFoundException, IniSectionNotFound
  {
    return getThingConfigured("server string");
  }

}
