/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

import org.apache.log4j.Logger;

import java.io.*;
import java.lang.reflect.InvocationTargetException;

import EDU.oswego.cs.dl.util.concurrent.*;

/**
 * User: rudec
 * Date: Sep 14, 2005
 * Time: 9:32:30 PM
 *
 * A fancy and complicated wrapper around java's exec() mechanism.
 */
public class ExecUtils
{
    static final Logger logger = Logger.getLogger(ExecUtils.class);

    /**
     * runCommand should be deprecated as a public method - call the command specific forms, like chown(),
     * groupadd(), and httpdRestart(); as soon they will be the only ones available
     * @param command - could be 'rm -rf /' for all we know, which is why this method should be private
     * @return The stdOut from the exec()'ed process
     * @throws ExecException - thrown when we get a nonzero return code: the caller should catch this and
     * decide whether nonzero return code is a bad thing for the exec()'ed process.  This ExecException contains
     * information about the command used, the stdOut and stdErr and return code for the process (if it could
     * collect any of these).
     */

    public static ProcessInfo runCommand(String command) throws ExecException
    {
        Process pExe;
        try
        {
            pExe = Runtime.getRuntime().exec(command);
        }
        catch (IOException ioe)
        {
            throw new ExecException("Exec("+command+") failed with an IOException:", command, ioe);
        }
        return processOutput(pExe, command, 0);
    }

    /**
     * Similar to runCommand(String), but calls the other version of exec().
     *
     * This should allow the caller to be more specific about what is the path to the
     * program and what are its arguments, when this is necessary.
     */
    public static ProcessInfo runCommand(String [] commandAndArgs) throws ExecException
    {
        Process pExe;
        try
        {
            pExe = Runtime.getRuntime().exec(commandAndArgs);
        }
        catch (IOException ioe)
        {
            throw new ExecException("Exec("+commandAndArgs[0]+") failed with an IOException:", commandAndArgs[0], ioe);
        }
        return processOutput(pExe, commandAndArgs[0], 0);
    }

    /**
     * Similar to runCommand(String), but calls the other version of exec(), and times
     * out if input is not received for a specified number of seconds.
     *
     * This should allow the caller to be more specific about what is the path to the
     * program and what are its arguments, when this is necessary.
     */
    public static ProcessInfo runCommand(String[] commandAndArgs, int timeoutSeconds) throws ExecException
    {
        Process pExe;
        try
        {
            pExe = Runtime.getRuntime().exec(commandAndArgs);
        }
        catch (IOException ioe)
        {
            throw new ExecException("Exec("+commandAndArgs[0]+") failed with an IOException:", commandAndArgs[0], ioe);
        }
        return processOutput(pExe, commandAndArgs[0], timeoutSeconds);
    }

    /**
     * This will wait for a process to exit while gathering its output.
     *
     * If timeoutSeconds is zero, this function will block until the process
     * terminates.
     *
     * If timeoutSeconds is nonzero, this function will forcibly terminate the
     * process if it is still running after timeoutSeconds.
     *
     * In any case, this function will return the full stdout and stderr streams
     * of the process, either as a return result or in an ExecException.
     *
     * @param pExe              the already-running process
     * @param command           the command line of the process (used for logging)
     * @param timeoutSeconds    maximum number of seconds to let the process run.  The process
     *                          will be forcibly terminated if it does not exit on its own within
     *                          this period.
     *                          If this value is zero, the process will not be terminated, and this
     *                          funtion will block indefinitely for it to complete.
     *
     * @return  a ProcessInfo bean representing the stdout, stderr, and return code of the process.
     *          This will only be returned of the process returned zero as its return code.
     *
     * @throws ExecException    if the process returns a non-zero error code, or if an exception
     *                          occured while executing the program or gathering its output.
     *                          If this exeption was caused by another exception, the ExecException's
     *                          "cause" property will be set appropriately.
     */
    private static ProcessInfo processOutput(Process pExe, String command, int timeoutSeconds)
            throws ExecException
    {
        {
            // close the child process's STDIN stream so that it
            // will not block for interactive input
            //
            OutputStream out = pExe.getOutputStream();
            try
            {
                // TODO: if we want to feed text to the process's STDIN,
                //       do an out.write() here before we close the stream.

                out.close();
            }
            catch (IOException e)
            {
                String s = "Could not close STDIN when executing ("+command+") :";
                logger.error(s,e);
                throw new ExecException(s, command, e);
            }
        }

        // Gather all of the stdout and stderr of the process.  This does all the dirty work.
        //
        ProcessInfo info = readStreamsWithTimeout(command, pExe, timeoutSeconds);

        if (null != info.getException())
        {
            logger.error("Error while running ("+command+")", info.getException());

            throw new ExecException(command, info);
        }


        if (0 != info.returnCode)
        {
            /*
              Sometimes, an error is the thing you expect to get back, ie
              when running grep or crontab you might not expect the thing
              that you are looking for to be there, thus the error code is
              expected and useful.  runCommand() is agnostic on this, and throws
              an ExecException with the stdErr and string for the caller to
              decide what to do with.
              */

            if ((null != info.stdErr) && (!"".equals(info.stdErr))) logger.debug(info.stdErr);
            if ((null != info.stdOut) && (!"".equals(info.stdOut))) logger.debug(info.stdOut);

            throw new ExecException(command, info);
        }

        // you have to read all the output from the program *before* waiting for it to exit
        // otherwise the wait will hang (atleast on commands with long output - bigger than one block perhaps?)
        logger.debug(command + ": " + info.stdOut);
        return info;
    }

    /**
     * Wait for a specified time for a given process to terminate.
     *
     * While it is waiting, it will actively read the STDERR and STDOUT streams from the
     * process, and store them in a string.
     *
     * @param pExe              the already-running process
     * @param timeoutSeconds    maximum number of seconds to let the process run.  The process
     *                          will be forcibly terminated if it does not exit on its own within
     *                          this period.
     *                          If this value is zero, the process will not be terminated, and this
     *                          funtion will block indefinitely for it to complete.
     * @return
     */
    private static ProcessInfo readStreamsWithTimeout(
                            final String command,
                            final Process pExe,
                            int timeoutSeconds)
    {
        // First, start the child threads to pump the output.
        //

        // When the output is gathered, these objects will contain the String
        // contents of the streams.
        //
        FutureResult futureStdout;
        FutureResult futureStderr;

        // This is used to track any errors we reach in this function.  If an error
        // occurs we will store it in this variable and continue to gather as much information
        // as possible about the failure, including the contents of the STDERR and STDOUT streams.
        //
        // This value will be used to set the "exception" property of the returned ProcessInfo bean.
        //
        Exception execFailure = null;

        try
        {
            // launch the two threads to suck stdout and stderr
            //
            futureStdout = readStreamOnSeparateThread(command,
                                                      pExe.getInputStream());

            futureStderr = readStreamOnSeparateThread(command,
                                                      pExe.getErrorStream());
        }
        catch (InterruptedException e)
        {
            // we couldn't start reading the streams for some reason.  This shouldn't
            // be likely to happen.
            //
            return new ProcessInfo(e);
        }

        long timeoutMillisecs = timeoutSeconds * 1000;

        Callable cmd = new Callable() {

                public Object call() throws Exception {
                    int returnCode = pExe.waitFor();
                    return new Integer(returnCode);
                }
            };

        if (timeoutMillisecs > 0)
        {
            // call the method with a timeout
            //
            // this will add watchdog code to ensure that the code above
            // does not execute for longer than timeoutMillisecs
            //
            cmd = new TimedCallable(cmd, timeoutMillisecs);
        }


        Integer returnCode;
        try
        {
            // wait for the process to exit.
            //
            // If we're running with an infinite timeout (timeoutMillisecs == 0)
            // then "cmd" will execute on the current thread.
            //
            // If it's greater than zero, then the TimedCallabable object we created
            // above will spawn a worker thread to do the waitFor(), and use the current
            // to wait timeoutMillisecs for this worker thread to complete.  This mess
            // is necessary because waitFor() does not allow the caller to specify a timeout.
            //
            returnCode = (Integer)cmd.call();
        }
        catch (Exception e)
        {
            // thread timed out.  Note the error, but try to gather
            // the output streams before failing.
            //
            execFailure = e;
            returnCode = new Integer(0);

            // An error occured while waiting for the process to exit, so
            // forcibly kill the process
            //
            pExe.destroy();
        }

        // note: we don't expect that this will block, since the
        //       process has either exited or been forcibly killed
        //
        try
        {
            pExe.waitFor();
        }
        catch (InterruptedException e)
        {
            // note: this shouldn't matter
        }


        // now try to see if our input streams are available
        //
        // the worker threads reading STDIN and STDOUT should now be in the
        // process of exiting, as the streams they were reading were closed
        // when the process exited
        //

        // this value is the number of milliseconds we'll wait for the threads
        // reading STDOUT and STDERR to finish collecting the input.  We always
        // expect these threads to exit once the process dies, but we don't want
        // to block indefinitely in order to be extra cautious.
        //
        final long INPUT_READER_THREAD_LAG_MILLISECS = 1000;

        String stdout = "";
        String stderr = "";
        try
        {
            stdout = (String)futureStdout.timedGet(INPUT_READER_THREAD_LAG_MILLISECS);
        }
        catch (InvocationTargetException e)
        {
            // the operation producing the value has encountered an exception.
            // Remember this.
            //
            if (null == execFailure)
            {
                execFailure = e;
            }
        }
        catch (TimeoutException e)
        {
            if (null == execFailure)
            {
                execFailure = e;
            }
        }
        catch (InterruptedException e)
        {
            // note: this shouldn't matter
        }

        try
        {
            stderr = (String)futureStderr.timedGet(INPUT_READER_THREAD_LAG_MILLISECS);
        }
        catch (InvocationTargetException e)
        {
            // the operation producing the value has encountered an exception.
            // Remember this.
            //
            if (null == execFailure)
            {
                execFailure = e;
            }
        }
        catch (TimeoutException e)
        {
            if (null == execFailure)
            {
                execFailure = e;
            }
        }
        catch (InterruptedException e)
        {
            // note: this shouldn't matter
        }

        return new ProcessInfo(stdout, stderr, returnCode.intValue(), execFailure);
    }

    private static FutureResult readStreamOnSeparateThread(
            final String command,
            final InputStream stream) throws InterruptedException
    {
        // this will execute the command on its own thread
        // note: we can change this to use a thread pool later if we so desire
        //
        Executor executor = new ThreadedExecutor();

        // when the thread completes, this will contain the complete output
        // of the stream
        //
        FutureResult futureString = new FutureResult();

        Runnable commandReader = futureString.setter(new Callable() {

            public Object call() { return readAllInput(command, stream); }
        });

        // this launches the above command on its own thread, as per
        // the policy of the executor
        //
        executor.execute(commandReader);

        return futureString;
    }

    private static String readAllInput(final String command,
                                       final InputStream stream)
    {
        StringBuffer sb = new StringBuffer();
        final BufferedReader reader = new BufferedReader(new InputStreamReader(stream));

        try
        {
            String sLine = "";
            while(null != (sLine = reader.readLine()) )
            {
                sb.append(sLine);
                sb.append('\n');
            }
        }
        catch (IOException e)
        {
            logger.error("Interrupted when reading command output for command " + command, e);
        }
        finally
        {
            return sb.toString();
        }
    }

}
