/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

import java.io.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

public class FileUtils {

    /**
     * Copies a file from one location to the other
     * @param source
     * @param dest
     * @throws IOException
     */
    public static void copy(File source, File dest) throws IOException {
        FileChannel input = null;
        FileChannel output = null;
        try {
            input = new FileInputStream(source).getChannel();
            output = new FileOutputStream(dest).getChannel();

            long size = input.size();
            MappedByteBuffer buffer = input.map(FileChannel.MapMode.READ_ONLY, 0, size);

            output.write(buffer);

        } finally {
            if (input != null) input.close();
            if (output != null) output.close();
        }
    }

    public static void copy (InputStream source, File dest) throws IOException {
        InputStream input = null;
        OutputStream output = null;

        try {
            input = new BufferedInputStream(source);
            output = new BufferedOutputStream(new FileOutputStream(dest));

            byte[] buf = new byte[1024];
            int len;
            while((len = input.read(buf)) > 0 ) {
                output.write(buf, 0, len);
            }
        } finally {
            if (output != null) output.close();
            if (input != null) input.close();
        }
    }


    // There seems to be a bug (in Windows especially) where the file will seem to still be in use even though it's not
    public static void delete(File source) {
        int retry = 0;
        while (!source.delete() && retry++ < 10) {
            System.gc();
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                // Ignore this
            }
        }
    }

    /*
    *  Read a file into a string.
    *  @param path file path
    *  @param maxSize maximum allowed file size or IOException is thrown (0 to skip size check)
    *  @return the content of the file as a string
    */
    public static String read( String path, long maxSize ) throws IOException {
        if ( null == path || 0 == path.length() )
            return new String("");

        File file = new File( path );
        long size = file.length();
        if ( maxSize > 0 && size > maxSize )
            throw new IOException( "File size is too large " + path );
        final FileReader fileReader = new FileReader(file);
        return getStringFromReader(fileReader, size);
    }

    protected static String getStringFromReader(final Reader in, long size)
            throws IOException
    {
        BufferedReader reader = new BufferedReader( in );
        char buf[] = new char[(int)size];
        reader.read( buf, 0, (int)size );
        reader.close();
        return new String( buf );
    }

    /*
    *  Read from a JAR resource file into a string.
    *  @param path Path to the resource file.
    *              This file must be present somewhere on the classpath (e.g. in our .jar).
    *  @return the content of the file as a string
    */
    public static String readFromResource( String resourcePath ) throws IOException {
        if ( null == resourcePath || 0 == resourcePath.length() )
            return new String("");

        final ClassLoader currentClassLoader = FileUtils.class.getClassLoader();
        InputStream s = currentClassLoader.getResourceAsStream(resourcePath);
        if (null == s) {
            throw new FileNotFoundException("Could not open file from from resource jar: " + resourcePath);
        }

        int size = s.available();
        InputStreamReader in = new InputStreamReader(s);

        return getStringFromReader(in, size);
    }

    public static void copyResourceToSystemFile(String resourceName,
                                                String destinationFile) throws IOException
    {
        copyResourceToSystemFile(resourceName,
                                 destinationFile,
                                 false);
    }

    public static void copyResourceToSystemFile(String resourceName,
                                                String destinationFile,
                                                boolean fNeverCreate) throws IOException
    {
        final ClassLoader currentClassLoader = FileUtils.class.getClassLoader();

        InputStream s = currentClassLoader.getResourceAsStream(resourceName);
        if (null == s) {
            throw new IOException("Could not load resource from JAR.  Resource name = " + resourceName);
        }

        final File dest = new File(destinationFile);
        if (fNeverCreate) {
            if (!dest.exists()) {
                return;
            }
        }
        FileUtils.copy(s, dest);
    }
}
