/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

import java.io.*;
import org.apache.log4j.*;

public class LinuxCommand
 {
    static Logger logger = Logger.getLogger(LinuxCommand.class);

    /**
     * <code>mkdir</code> runs <code>File.mkdirs(dirName)</code>, forcing creation of the parent directories
     * if they do not exist.
     * @param dirName
     * @return "Successfully created dirName" if successful, otherwise throws a new <code>ExecException</code>
     * @throws ExecException when File.mkdirs() fails.
     */
    public static String mkdir(String dirName) throws ExecException
    {
        File dirFile = new File(dirName);
        try
        {
            if (dirFile.exists()) { return "Directory already exists "+dirName; }
            if(dirFile.mkdirs()) { return "Successfully created "+dirName; }
            else throw new ExecException("mkdirs("+dirName+")", "", "Failed");
        }
        catch (SecurityException se)
        {
            throw new ExecException("Improper permissions in creating directory:" + dirName, se);
        }
    }

    /**
     * <code>chown</code> runs <code>exec("chown -v " + user + target)</code>, changing ownership of
     * target to the user
     * @param user can also be of the form 'sUser.sGroup' or 'sUser:sGroup'
     * @param target file or directory
     * @return stdOut passed up from <code>runCommand()</code>: should be either "changed ownership of
     * `target' to 'user'" or "ownership of `target' retained as 'user'"
     * @throws ExecException
     */
    public static String chown(String user, String target) throws ExecException
    {
        return ExecUtils.runCommand("chown -v " + user + " " + target).getStdOut();
    }

    /**
     * <code>chown</code> runs <code>exec("chown -v " + user + target)</code>, changing ownership of
     * target to the user
     * @param user can also be of the form 'sUser.sGroup' or 'sUser:sGroup'
     * @param target file or directory
     * @return stdOut passed up from <code>runCommand()</code>: should be either "changed ownership of
     * `target' to 'user'" or "ownership of `target' retained as 'user'"
     * @throws ExecException
     */
    public static String chownRecursively(String user, String target) throws ExecException {
        return ExecUtils.runCommand("chown -R -v " + user + " " + target).getStdOut();
    }

    /**
     * <code>chmod</code> runs <code>exec("chmod " + mode + target)</code>, changing permissions
     * on the file or directory to mode
     * @param mode
     * @param target
     * @return stdOut passed up from <code>runCommand()</code>: should be empty string
     * @throws ExecException
     */
    public static String chmod( String mode, String target ) throws ExecException {
        return ExecUtils.runCommand("chmod " + mode + " " + target ).getStdOut();
    }

    /**
     * <code>groupAdd</code> runs <code>exec("groupadd" + groupName)</code>
     * @param groupName
     * @return stdOut, as usual
     */
    public static String groupadd(String groupName) throws ExecException
    {
        try
        {
            return ExecUtils.runCommand("/usr/sbin/groupadd " + groupName).getStdOut();
        }
        catch (ExecException e)
        {
            if (9 == e.getRetCode())
            {
                // the group already exists... no error
                //
                return e.getStdOut();
            }
            throw e;
        }
    }

    /**
     * <code>exec("ln -s target link_name")</code>
     * will create a SYMBOLIC link to <code>target</code> in <code>link_name</code>
     * Example:
     * <code>LinuxCommand.symbolicLink("/var/log", "/home/blah/logs")</code> will result in the following:
     *
     * <code>lrwxrwxrwx  1 blah blah     11 Jul 14 14:08 logs -> /var/log/</code>
     *
     * in the <code>/home/blah/</code> directory
     * @param target the folder or file to which the link will point to
     * @param link_name the location and name of the folder where the link will be created
     * @throws ExecException
     */
    public static String symbolicLink(String target, String link_name) throws ExecException{
        String command = "ln -s";
        return ExecUtils.runCommand(command + " " + target + " " + link_name).getStdOut();
    }

    /**
     * <code>exec("mv from to")</code>
     * will move the <code>from</code> to the <code>to</code>
     *
     * @param from the initial location of the file/directory
     * @param to the final location of the file/directory
     * @throws ExecException
     */
    public static String move(String from, String to) throws ExecException {
        String command = "mv";
        return ExecUtils.runCommand(command + " " + from + " " + to).getStdOut();
    }

    /**
     * <code>exec("rm -rf location")</code>
     * will recursively delete everything from location.
     *
     * This command will be executed as the user currently running the application. In our case, this may
     * well be ROOT!!!!
     *
     * @param location the file/directory to be removed
     */
    public static String forceRemove(String location) throws ExecException{
        String command = "rm -rf";
        return ExecUtils.runCommand(command + " " + location).getStdOut();
    }


    /**
     * Returns the SID of a windows user, group or computer account by quering
     * the currently-configured domain controller.
     *
     * note: this function calls the "wbinfo" tool, which returns information
     *       from the windbindd daemin.
     *
     *       The winbindd(8) daemon must be configured and running for the
     *       wbinfo program to be able to return information.

     * @param userOrGroupName   name of windows user or group to lookup
     * @return  The string SID, in the form "S-1-0", or null if the SID lookup failed.
     * @throws FileNotFoundException  if there is no defined "wbinfo" command in our
     *                                system properties file.
     *          ExecException         if the user or group name was not found, or if
     *                                winbind was not properly configured
     */
    public static String lookupSid(String userOrGroupName) throws FileNotFoundException, ExecException
    {
        final int timeoutSeconds = 10;

        // escape all the "" in user name
        // TODO: do this more generally?
        String validUserOrGroupName = userOrGroupName.replaceAll("[;'\"]", "");
        String wbInfoCommand = SystemFileLocator.getCommandWbinfo();
        if ((null == wbInfoCommand) || "".equals(wbInfoCommand))
        {
            throw new FileNotFoundException("Could not execute 'wbinfo' command because the system property was undefined.");
        }

        String[] args = {wbInfoCommand,
                         "-n",
                         validUserOrGroupName};

        ProcessInfo info = ExecUtils.runCommand(args, timeoutSeconds);

        String wbInfoOutput = info.getStdOut();

        // the output should be of the form
        //      S-1-5-21-418081286-1191099226-2202501032-1171 Domain Group (2)
        // so we basically want everything before the first space.
        //
        String[] tokens = wbInfoOutput.split(" ", 2);
        String sid = tokens[0];

        return sid;
    }

    /**
     * Returns true if the given path exists and is a symbolic link. Returns false otherwise.
     *
     * @param path path to check
     */
    public static boolean isSymbolicLink( String path ) throws ExecException
    {
        String commands[] = { "ls", "-ld", path };
        return ExecUtils.runCommand(commands).getStdOut().indexOf('l') == 0;
    }
}
