/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

import java.io.*;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * this is a static utility class for getting the paths of important system files.
 * the paths are read from a property file that was chosen by the installer
 * at install time based on the system type.
 */
public class SystemFileLocator {
    public static final String  DISTRO_TYPE_RHEL = "rhel";
    public static final String  DISTRO_TYPE_SUSE = "suse";

    private static final String PROPERTY_HttpdConfFileLocation = "HttpdConf";
    private static final String PROPERTY_DistroFileLocation = "Release";
    private static final String PROPERTY_WebRoot = "Centeris.Web.Root";
    private static final String PROPERTY_LogRoot = "Centeris.Log.Root";
    private static final String PROPERTY_OverridePropertiesFileLocation = "Override.Properties.File.Location";
    private static final String PROPERTY_DistroType = "DistroType";
    private static final String PROPERTY_DistroVersion = "DistroVersion";
    private static final String PROPERTY_HttpdService = "Apache.httpd.service";
    private static final String PROPERTY_HttpdConfFileDisabled = "HttpdConf.disabled";
    private static final String PROPERTY_ApacheModulesPath = "ApacheModulesPath";
    private static final String PROPERTY_HttpdExec = "Apache.httpd.exec";
    private static final String PROPERTY_KnownGoodApacheConfFile = "Apache.backup.conf";
    private static final String PROPERTY_ExpandedDistroApacheConfFile = "Apache.expanded.distro.conf";
    private static final String PROPERTY_PamConfigDirectory = "PamConfigDirectory";
    private static final String PROPERTY_WbinfoExec = "Samba.wbinfo.exec";
    private static final String PROPERTY_ApachePamServiceName = "Apache.pam.service.name";

    private static final String PROPERTY_ResolvConfPath = "ResolvConfPath";

    private static final String SUSE_RELEASE_FILE = "/etc/SuSE-release";
    private static final String RHEL_RELEASE_FILE = "/etc/redhat-release";

	private static final String PROPERTY_Soap_Port_SSL		= "centeris.soap.port.ssl" ;
	private static final String PROPERTY_Soap_Port_Debug	= "centeris.soap.port.debug" ;
	private static final String PROPERTY_Soap_Debug			= "com.centeris.useDebug" ;
	private static final String PROPERTY_Soap_SSL			= "com.centeris.useSSL" ;

	private static final String default_Soap_Port_SSL		= "9105" ;
	private static final String default_Soap_Port_Debug		= "9106" ;


    // This is the location of the default properties file.
    // Any values set in this can be overridden by changing the values in
    // the distribution specific configuration files.
    public static final String systemPropertiesDefaultLocation =
            "properties/distribution-specific/systemFiles.properties";

    public static String userDefinedPropertiesFileLocation = null;

    protected static Properties prop;

    static
    {
        // copy the new properties file over
        copySystemPropertiesFile();
        try
        {
            reloadProperties();
            SystemFileLocator.touchUpProperties();
        }
        catch (IOException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }


    public static void copySystemPropertiesFile() {

        String sourceFile = null;

        try {
            if ( discoverDistroType().equals( SystemFileLocator.DISTRO_TYPE_SUSE ))
                sourceFile = "properties/distribution-specific/systemFiles.properties.suse";
            else if ( discoverDistroType().equals( SystemFileLocator.DISTRO_TYPE_RHEL ) )
                sourceFile = "properties/distribution-specific/systemFiles.properties.rhel";

            final String propertiesFileLocation = SystemFileLocator.getPropertiesFileLocation();
            final String propertiesFilePath = new File(propertiesFileLocation).getAbsoluteFile().getParent();

            LinuxCommand.mkdir(propertiesFilePath);

            FileUtils.copyResourceToSystemFile(sourceFile,
                                               propertiesFileLocation);
        } catch (IOException e) {
            e.printStackTrace(); // todo: do something with this
        }
        catch (ExecException e)
        {
            e.printStackTrace();  // todo: do something with this
        }
    }

    /*
    * Tries to figure out the Linux distribution type
    * @return Linux distribution type
    *         types: DISTRO_TYPE_SUSE, DISTRO_TYPE_RHEL
    */
    public static String discoverDistroType() throws FileNotFoundException {
        String distroType = "";
        if (new File(SystemFileLocator.SUSE_RELEASE_FILE).exists())
            distroType = SystemFileLocator.DISTRO_TYPE_SUSE;
        else if (new File(SystemFileLocator.RHEL_RELEASE_FILE).exists())
            distroType = SystemFileLocator.DISTRO_TYPE_RHEL;
        else throw new FileNotFoundException( "Release file not found on the system." );

        return distroType;
    }

    /*
    * Tries to figure out the Linux distribution version
    * @return Linux distribution version
    */
    public static String discoverDistroVersion () throws IOException {

        String version = "";

        if (discoverDistroType().equals( SystemFileLocator.DISTRO_TYPE_RHEL ))
        {
            // TODO: parse rhel release file
        }
        else if ( discoverDistroType().equals( SystemFileLocator.DISTRO_TYPE_SUSE ))
        {
            File release = new File(SystemFileLocator.SUSE_RELEASE_FILE);
            BufferedReader releaseReader = new BufferedReader(new FileReader(release));
            String line = releaseReader.readLine();
            line = releaseReader.readLine();
            line = line.trim();
            System.err.println("suse release version line is " + line);
            StringTokenizer tokenizer = new StringTokenizer(line, " ");
            tokenizer.nextToken();
            tokenizer.nextToken();
            version = tokenizer.nextToken();
        }

        return version;
    }

    /**
     * This allows for a different location of the properties file to be set.
     * It is meant to be a method used for testing purposes mostly.
     * In the case of a test, you could set newLocation to be a properties file that
     * contains testing paths.
     * <p/>
     * Note that the effect of this method lasts until the class is unloaded.  This may cause unintended side
     * effects when running unit tests as the class will not get unloaded between tests.  If you change
     * the prop those changes are persistent for the remainder of all tests being executed.
     * <p/>
     * This method should not be used to signal the permanent change in location of the
     * <code>systemPropertiesFileLocation</code> variable above. If the path to the properties
     * file changes, then the variable should be updated.
     * <p/>
     * This allows for tests to be run more independently as it removes the dependancy on
     * /usr/share/centeris/conf/systemFiles.properties.
     *
     * @param newLocation - the location of the properties file to use.
     */
    public static void setPropertiesFileLocation(String newLocation) {
        SystemFileLocator.userDefinedPropertiesFileLocation = newLocation;
        try
        {
            reloadProperties();
        }
        catch (Exception e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public static void reloadProperties() throws FileNotFoundException, IOException
    {

        Properties defaultProperties = getDefaultProperties();

        prop = new Properties(defaultProperties);

        File f = new File(getPropertiesFileLocation());
        if(!f.exists())
        {
                throw new FileNotFoundException("properties file " + getPropertiesFileLocation() + " does not exist");
        }
        InputStream s = new FileInputStream(f);
        prop.load(s);
        s.close();
    }

    private static Properties getDefaultProperties()
    {
        final ClassLoader currentClassLoader = SystemFileLocator.class.getClassLoader();

        Properties defaultProperties = new Properties();

        try
        {
            InputStream s = currentClassLoader.getResourceAsStream(SystemFileLocator.systemPropertiesDefaultLocation);
            defaultProperties.load(s);
            s.close();
        }
        catch (IOException e)
        {
        }
        return defaultProperties;
    }

    public static String getHttpdConf() {
        return prop.getProperty(SystemFileLocator.PROPERTY_HttpdConfFileLocation);
    }

    public static String getRelease() {
        return prop.getProperty(SystemFileLocator.PROPERTY_DistroFileLocation);
    }

    public static String getWebRoot() {
        return prop.getProperty(SystemFileLocator.PROPERTY_WebRoot);
    }

    public static String getLogRoot() {
        return prop.getProperty(SystemFileLocator.PROPERTY_LogRoot);
    }

    public static String getResolvConfPath(){
        return prop.getProperty(SystemFileLocator.PROPERTY_ResolvConfPath);
    }

    public static String getPropertiesFileLocation() {
        Properties propDefault = prop;
        if (null == propDefault)
        {
            propDefault = getDefaultProperties();
        }

        return userDefinedPropertiesFileLocation != null ?
                    userDefinedPropertiesFileLocation :
                    propDefault.getProperty(SystemFileLocator.PROPERTY_OverridePropertiesFileLocation);
    }

    public static String getDistroType() {
        return prop.getProperty(SystemFileLocator.PROPERTY_DistroType);
    }

    public static String getDistroVersion() {
        return prop.getProperty(SystemFileLocator.PROPERTY_DistroVersion);
    }

    public static String getApacheInitd() {
        return "/etc/init.d/" + prop.getProperty(SystemFileLocator.PROPERTY_HttpdService);
    }

    public static String getApacheService() {
        return prop.getProperty(SystemFileLocator.PROPERTY_HttpdService);
    }

    public static String getApacheExecutable() {
        return prop.getProperty(SystemFileLocator.PROPERTY_HttpdExec);
    }

    public static String getHttpdDisabledConf() {
        return prop.getProperty(SystemFileLocator.PROPERTY_HttpdConfFileDisabled);
    }

    public static String getApacheModulesPath() {
        return prop.getProperty(SystemFileLocator.PROPERTY_ApacheModulesPath);
    }

    public static String getKnownGoodApacheConfFile() {
        return prop.getProperty(SystemFileLocator.PROPERTY_KnownGoodApacheConfFile);
    }

    public static String getExpandedApacheDistroConfFile() {
        return prop.getProperty( PROPERTY_ExpandedDistroApacheConfFile );
    }

    public static String getPamConfigDirectory()
    {
        return prop.getProperty(SystemFileLocator.PROPERTY_PamConfigDirectory);
    }

    public static String getCommandWbinfo()
    {
        return prop.getProperty(SystemFileLocator.PROPERTY_WbinfoExec);
    }

    public static String getApachePamServiceName()
    {
        return prop.getProperty(SystemFileLocator.PROPERTY_ApachePamServiceName);
    }


	//
	// in both of these cases, we always want to assume the worse, which is why the defaults
	// are the more stringent to the two.
	//

	public static boolean useSoapDebugging()
	{
		String tmp = prop.getProperty(SystemFileLocator.PROPERTY_Soap_Debug) ;

		if( (tmp != null) && ((tmp.compareToIgnoreCase("Y") == 0) || (tmp.compareToIgnoreCase("YES") == 0)) )
		{
           return( true ) ;
		}

		return(false) ;
	}


	public static boolean useSoapSSL()
	{
		String tmp = prop.getProperty(SystemFileLocator.PROPERTY_Soap_SSL) ;

		if( (tmp != null) && ((tmp.compareToIgnoreCase("N") == 0) || (tmp.compareToIgnoreCase("NO") == 0)) )
		{
           return( false ) ;
		}

		return(true) ;
	}

	public static String getSoapPortSslString()
	{
		String tmp = prop.getProperty(SystemFileLocator.PROPERTY_Soap_Port_SSL) ;

		if( tmp == null )
		{
			return( default_Soap_Port_SSL ) ;
		}

		return(tmp) ;
	}

	public static String getSoapPortDebugString()
	{
		String tmp = prop.getProperty(SystemFileLocator.PROPERTY_Soap_Port_Debug) ;

		if( tmp == null )
		{
			return(default_Soap_Port_Debug ) ;
		}

		return(tmp) ;
	}


	public static int getSoapPortSsl()
	{
		String tmp = getSoapPortSslString() ;

		int port = Integer.parseInt(tmp) ;

		return(port) ;
	}


	public static int getSoapPortDebug()
	{
		String tmp = getSoapPortDebugString() ;

		int port = Integer.parseInt(tmp) ;
		
		return(port) ;
	}


    public static void touchUpProperties() {
        // Touch up properties
		String distroType = SystemFileLocator.getDistroType();
            if(distroType.equals("rhel"))
            {

            }
            else if(distroType.equals("suse"))
            {
                try {
                    String version = discoverDistroVersion();
                    System.err.println("version is " + version);
                    prop.setProperty(SystemFileLocator.PROPERTY_DistroVersion, version);
                    SystemFileLocator.saveChanges();
                }
                catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
                catch (IOException e) {
                    e.printStackTrace();
                }
            }
    }

    private static void saveChanges() {
        try {
            prop.store(new FileOutputStream(getPropertiesFileLocation()), "Saving properties after fixup");
        }
        catch (IOException e) {
            // we can't save the changes. that's bad news
            e.printStackTrace();
        }
    }
}
