/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

// unit tests for IniPreserve class

package com.centeris.webservice.samba;

import junit.framework.TestCase;

import java.io.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

public class IniPreserveTest extends TestCase {
    private File tempFile;


    // get the full pathname for a file by searching for it in the classpath
    private String getFileName(String filename) {
        return this.getClass().getResource(filename).getFile(); 	
    }

    // copy a file
    private void copyFile(File source, File dest) throws IOException {
        FileChannel in = null, out = null;
        try {
            in = new FileInputStream(source).getChannel();
            out = new FileOutputStream(dest).getChannel();

            long size = in.size();
            int beginningOfFile = 0;
            MappedByteBuffer buf = in.map(FileChannel.MapMode.READ_ONLY, beginningOfFile, size);

            out.write(buf);

        } finally {
            if (in != null)  in.close();
            if (out != null) out.close();
        }
    }

    // compare files to see if they have the same contents
    private boolean filesAreIdentical(File file1, File file2) throws IOException {
        FileChannel in = null, out = null;
        try {
            in = new FileInputStream(file1).getChannel();
            out = new FileInputStream(file2).getChannel();

            long size = file1.length();
            if (size != file2.length()) { return false; } // bah, files aren't even the same size
            int beginningOfFile = 0;
            MappedByteBuffer inBuf = in.map(FileChannel.MapMode.READ_ONLY, beginningOfFile, size);
            MappedByteBuffer outBuf = out.map(FileChannel.MapMode.READ_ONLY, beginningOfFile, size);
            int buffersAreEqual = 0;
            if (buffersAreEqual == inBuf.compareTo(outBuf)) {
                return true;
            } else {
                return false;
            }
        } finally {
            if (in != null)  in.close();
            if (out != null) out.close();
        }
    }

    protected void setUp( ) throws IOException
    {
        tempFile = File.createTempFile("testIni", ".conf"); 
//        tempFile.deleteOnExit();
    }

    public void testIniCreation() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            assertTrue(true); 
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    public void testIniEmptyFile() {
        try {
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            assertTrue(true);
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }
 
    public void testIniNoFile() {
        try {
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath() + "asdfasdfasdf");
            fail("expected FileNotFoundException");
        } catch (FileNotFoundException e) {
            assertTrue(true);
        } catch (IOException e) {
            fail("IOException");
        }
    }
    
    public void testIniAddEmptySection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            ini.addSection("NewSection"); 
            ini.commit();
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionAlreadyExists e) {
        		fail("IniSectionAlreadyExists");
        }
        assertTrue(true); 
    }

    public void testIniAddSection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            IniSection newSection = ini.addSection("NewSection"); 
            newSection.putKey("foo", "bar");
            newSection.putKey("description", "\"something cool\"");
            ini.replaceSection(newSection);
            ini.commit();
// compare files here
            File goodTestData = new File("resources/add_section_smb.conf");
            assertTrue( filesAreIdentical( tempFile, goodTestData ) );
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionAlreadyExists e) {
        		fail("IniSectionAlreadyExists");
        }
    }

    public void testIniDeleteSection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            ini.removeSection("OldSection"); 
            ini.commit();
// compare files here
            File goodTestData = new File("resources/delete_section_smb.conf");
            assertTrue( filesAreIdentical( tempFile, goodTestData ) );
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    public void testIniModifySection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            IniSection newSection = ini.getSection("OldSection"); 
            newSection.putKey("foo", "bar");
            newSection.putKey("description", "\"something cool\"");
            ini.replaceSection(newSection);
            ini.commit();
// compare files here
            File goodTestData = new File("resources/modify_section_smb.conf");
            assertTrue( filesAreIdentical( tempFile, goodTestData ) );
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionNotFound e) {
            fail("IniSectionNotFound");
        }
    }

    public void testIniGetMissingSection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            IniSection newSection = ini.getSection("SectionIsNotThere"); 
            newSection.putKey("foo","bar");
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionNotFound e) {
            assertTrue(true); 
        }
    }

    public void testIniModifyMissingSection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            IniSection newSection = ini.getSection("SectionIsNotThere"); 
            newSection.putKey("foo","bar");
            ini.replaceSection(newSection);
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionNotFound e) {
            assertTrue(true); 
        }
    }

    public void testIniDeleteMissingSection() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            ini.removeSection("SectionIsNotThere"); 
            ini.commit();
            assertTrue(true); 
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    public void testIniAddSectionThatExists() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            IniSection newSection = ini.addSection("OldSection");
            fail("No IniSectionAlreadyExists exception caught");
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        } catch (IniSectionAlreadyExists e) {
        		// hooray, this is normal behaviour
        		assertTrue(true);
        }
    }

    public void testIniReservedSectionName() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            assertTrue(ini.legalSectionName("legal"));
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    
    public void testIniIllegalSectionName() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            assertFalse(ini.legalSectionName("c$"));
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    public void testIniPercentSectionName() {
        try {
            copyFile(new File("resources/smb.conf"), tempFile);
            IniPreserve ini = new IniPreserve(tempFile.getAbsolutePath());
            assertFalse(ini.legalSectionName("illegal%m"));
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException");
        } catch (IOException e) {
            fail("IOException");
        }
    }

    
}
