/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

package com.centeris.webservice.utils;

import java.io.IOException;
import java.io.File;
import java.io.FilenameFilter;

import junit.framework.TestCase;
import EDU.oswego.cs.dl.util.concurrent.TimeoutException;

/**
 * User: rudec
 * Date: Sep 15, 2005
 * Time: 1:25:42 PM
 *
 */
public class ExecUtilsTest
        extends TestCase
{
    private final String testScriptsDir = "./test/exec-scripts/";

    /*
    * Setup for tests.
    * ensures that all the scripts we're going to run are executable
    */
    public void setUp() throws IOException, ExecException {
        String [] scripts = getListOfScripts();
        for (int i = 0; i < scripts.length; i++)
        {
            String script = scripts[i];
            LinuxCommand.chmod("755", testScriptsDir + script);
        }
    }

    public String [] getListOfScripts()
    {
        File dir = new File(testScriptsDir);

        // It is also possible to filter the list of returned files.
        // This example does not return any files that start with `.'.
        FilenameFilter filter = new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.endsWith(".pl");
            }
        };

        String[] scripts = dir.list(filter);
        return scripts;
    }

    /*
    * Cleanup.
    * Delete the certificate file and directory structure.
    */
    public void tearDown() throws IOException, ExecException {

        // TODO: remove core files, etc?
    }

    public ProcessInfo testTimedExec(String [] commandAndArgs, int timeoutSeconds, int maxTimeoutSeconds) throws ExecException
    {
        long originalTime = System.currentTimeMillis();
        ExecException err = null;

        ProcessInfo info = null;
        try
        {
            info = ExecUtils.runCommand(commandAndArgs, timeoutSeconds);
        }
        catch (ExecException e)
        {
            // remember this for later
            err = e;
        }

        long newTime = System.currentTimeMillis();

        long duration = newTime - originalTime;

        // it should stop after 10 seconds, but give it a few extra.
        // if it fails entirely, we will wait for 30 seconds.
        //

        assertTrue("broken command was not interrupted: " + commandAndArgs[0],
                   duration < (maxTimeoutSeconds * 1000));

        if ( null != err )
        {
            throw err;
        }
        return info;
    }


    public void test4args()
    {
        final String[] cmd = {"test/exec-scripts/4_args.pl", "one arg", "two args", "three args", "four args"};

        try
        {
            ProcessInfo info = ExecUtils.runCommand(cmd);
            assertEquals("", info.getStdOut());
            assertEquals("", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("could not execute command with arguments.");
        }

        // if we get here, it worked
    }

    public void test5args()
    {
        final String[] cmd = {"test/exec-scripts/4_args.pl", "one arg", "two args", "three args", "four args", "five args"};

        try
        {
            // runs with no args, should fail
            ExecUtils.runCommand(cmd);
            fail("too many parameters given, should have failed.");
        }
        catch (ExecException e)
        {
            assertEquals(1, e.getRetCode());
            assertEquals("", e.getStdErr());
            assertEquals("", e.getStdOut());
        }

        // if we get here, it worked
    }

    public void test3args()
    {
        final String[] cmd = {"test/exec-scripts/4_args.pl", "one arg", "two args", "three args"};

        try
        {
            // runs with no args, should fail
            ExecUtils.runCommand(cmd);
            fail("not enough parameters given, should have failed.");
        }
        catch (ExecException e)
        {
            assertEquals(1, e.getRetCode());
            assertEquals("", e.getStdErr());
            assertEquals("", e.getStdOut());
        }

        // if we get here, it worked
    }

    public void testNoargs()
    {
        final String cmd = "test/exec-scripts/4_args.pl";

        try
        {
            // runs with no args, should fail
            ExecUtils.runCommand(cmd);
            fail("no parameters given, should have failed.");
        }
        catch (ExecException e)
        {
            assertEquals(1, e.getRetCode());
            assertEquals("", e.getStdErr());
            assertEquals("", e.getStdOut());
        }

        // if we get here, it worked
    }


    public void testBackgroundSleep30()
    {
        final String [] cmd = { "test/exec-scripts/background_sleep30.pl" };

        try
        {
            // we'll terminate this process after 10 seconds, but it
            // should return immediately
            //
            ProcessInfo info = testTimedExec(cmd, 10, 5);
            assertTrue(info.getStdOut().startsWith("forked process"));
            assertEquals("", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("we hung when child process forked zombie process");
        }
    }

    public void testCore()
    {
        final String [] cmd = { "test/exec-scripts/core.pl" };

        try
        {
            ExecUtils.runCommand(cmd);
        }
        catch (ExecException e)
        {
            // TODO: what to do here?
            assertEquals("expected no stderr on core", e.getStdErr(), "");
            assertEquals("expected no stdout on core", e.getStdOut(), "");
            assertEquals("weird return result", e.getRetCode(), 129);

            return;
        }

        fail("core dump not detected");

    }

    public void testExit0() throws ExecException
    {
        final String cmd = "test/exec-scripts/exit0.pl";

        // this should jsut return fine
        ProcessInfo info = ExecUtils.runCommand(cmd);

        assertEquals("", info.getStdOut());
        assertEquals("", info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testExit1()
    {
        final String cmd = "test/exec-scripts/exit1.pl";

        try
        {
            ExecUtils.runCommand(cmd);
        }
        catch (ExecException e)
        {
            assertEquals("expected no stderr on failure", e.getStdErr(), "");
            assertEquals("expected no stdout on failure", e.getStdOut(), "");
            assertEquals("weird return result", e.getRetCode(), 1);

            return;
        }

        fail("error return code not detected");
    }

    public void testSleep30()
    {
        final String[] cmd = {"test/exec-scripts/sleep30.pl"};

        try
        {
            // we'll terminate this process after 1 second, and
            // if we don't it should run for 30
            //
            ProcessInfo info = testTimedExec(cmd, 1, 5);
            fail("Should have terminated process but instead got return result " + info.getReturnCode());
        }
        catch (ExecException e)
        {
            assertNotNull(e.getCause());
            Throwable cause = e.getCause();
            assertTrue(cause instanceof TimeoutException);
            assertEquals("", e.getStdErr());
            assertEquals("", e.getStdOut());
            assertEquals(0, e.getRetCode());
        }

    }

    public void testStderr() throws ExecException
    {
        final String cmd = "test/exec-scripts/stderr.pl";

        ProcessInfo info = ExecUtils.runCommand(cmd);
        assertEquals("", info.getStdOut());
        assertEquals("stuff on stderr\n", info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    private static String makeBigOutput()
    {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < 10000; ++i)
        {
            sb.append("it's a beautiful day in the neighborhood.\n");
        }
        return sb.toString();
    }

    public void testStderrBig() throws ExecException
    {
        final String cmd = "test/exec-scripts/stderr_big.pl";

        final String stderrexpected = makeBigOutput();

        ProcessInfo info = ExecUtils.runCommand(cmd);
        assertEquals("", info.getStdOut());
        assertEquals(stderrexpected, info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testStderrExit1()
    {
        final String cmd = "test/exec-scripts/stderr_exit1.pl";

        try
        {
            ExecUtils.runCommand(cmd);
        }
        catch (ExecException e)
        {
            assertEquals("stuff on stderr\n", e.getStdErr());
            assertEquals("expected no stdout on failure", "", e.getStdOut());
            assertEquals("weird return result", 1, e.getRetCode());
            assertNull(e.getCause());

            return;
        }

        fail("error return code not detected");
    }

    public void testStderrStdout() throws ExecException
    {
        final String cmd = "test/exec-scripts/stderr_stdout.pl";

        ProcessInfo info = ExecUtils.runCommand(cmd);
        assertEquals("stuff on stdout\n", info.getStdOut());
        assertEquals("stuff on stderr\n", info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testStderrStdoutBig() throws ExecException
    {
        final String cmd = "test/exec-scripts/stderr_stdout_big.pl";

        final String strexpected = makeBigOutput();

        ProcessInfo info = ExecUtils.runCommand(cmd);
        assertEquals("expected lots of input on stdout", strexpected, info.getStdOut());
        assertEquals("expected lots of input on stderr", strexpected, info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testStderrStdoutExit1()
    {
        final String cmd = "test/exec-scripts/stderr_stdout_exit1.pl";

        try
        {
            ExecUtils.runCommand(cmd);
        }
        catch (ExecException e)
        {
            assertEquals("stuff on stderr\n", e.getStdErr());
            assertEquals("stuff on stdout\n", e.getStdOut());
            assertEquals("weird return result", 1, e.getRetCode());
            assertNull(e.getCause());

            return;
        }

        fail("error return code not detected");
    }

    public void testStdin()
    {
        final String [] cmd = { "test/exec-scripts/stdin.pl" };

        try
        {
            // this process should return immediately, unless it's
            // blocked on stdin.  We'll terminate it after 10 seconds, but
            // if it takes more than 5 seconds, we failed.
            //
            ProcessInfo info = testTimedExec(cmd, 10, 5);
            assertEquals("", info.getStdOut());
            assertEquals("", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("we hung when child process read stdin");
        }

    }

    public void testStdinFirstStdoutStderr()
    {
        final String [] cmd = { "test/exec-scripts/stdin_first_stdout_stderr.pl" };

        try
        {
            // this process should return immediately, unless it's
            // blocked on stdin.  We'll terminate it after 10 seconds, but
            // if it takes more than 5 seconds, we failed.
            //
            ProcessInfo info = testTimedExec(cmd, 10, 5);
            assertEquals("it's a beautiful day in the neighborhood.\n", info.getStdOut());
            assertEquals("it's a beautiful day in the neighborhood.\n", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("we hung when child process read stdin");
        }
    }

    public void testStdinStdout()
    {
        final String [] cmd = { "test/exec-scripts/stdin_stdout.pl" };

        try
        {
            // this process should return immediately, unless it's
            // blocked on stdin.  We'll terminate it after 10 seconds, but
            // if it takes more than 5 seconds, we failed.
            //
            ProcessInfo info = testTimedExec(cmd, 10, 5);
            assertEquals("it's a beautiful day in the neighborhood.\n", info.getStdOut());
            assertEquals("expected no input on stderr", "", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("we hung when child process read stdin");
        }
    }

    public void testStdinStdoutStderr()
    {
        final String [] cmd = { "test/exec-scripts/stdin_stdout_stderr.pl" };

        try
        {
            // this process should return immediately, unless it's
            // blocked on stdin.  We'll terminate it after 10 seconds, but
            // if it takes more than 5 seconds, we failed.
            //
            ProcessInfo info = testTimedExec(cmd, 10, 5);
            assertEquals("it's a beautiful day in the neighborhood.\n", info.getStdOut());
            assertEquals("it's a beautiful day in the neighborhood.\n", info.getStdErr());
            assertEquals(0, info.getReturnCode());
            assertEquals(null, info.getException());
        }
        catch (ExecException e)
        {
            fail("we hung when child process read stdin");
        }
    }

    public void testStdout() throws ExecException
    {
        final String cmd = "test/exec-scripts/stdout.pl";

        ProcessInfo info = ExecUtils.runCommand(cmd);

        assertEquals("stuff on stdout\n", info.getStdOut());
        assertEquals("expected no output onstderr", "", info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testStdoutBig() throws ExecException
    {
        final String cmd = "test/exec-scripts/stdout_big.pl";
        final String strexpected = makeBigOutput();

        ProcessInfo info = ExecUtils.runCommand(cmd);

        assertEquals("expected lots of input on stdout\n", strexpected, info.getStdOut());
        assertEquals("expected no output onstderr", "", info.getStdErr());
        assertEquals(0, info.getReturnCode());
        assertEquals(null, info.getException());
    }

    public void testStdoutExit1()
    {
        final String cmd = "test/exec-scripts/stdout_exit1.pl";

        try
        {
            ExecUtils.runCommand(cmd);
            fail("failed to detect error");

        }
        catch (ExecException e)
        {
            assertEquals("stuff on stdout\n", e.getStdOut());
            assertEquals("expected no stderr on failure", "", e.getStdErr());
            assertEquals("weird return result", 1, e.getRetCode());
            assertNull(e.getCause());

            return;
        }
    }


    public void testInvalidCommand()
    {
        final String cmd = "test/exec-scripts/this-command-does-not-exist.pl";

        try
        {
            ExecUtils.runCommand(cmd);
            fail("failed to error when unknown command passed");
        }
        catch (ExecException e)
        {
            assertEquals("expected no stdout on failure\n", "", e.getStdOut());
            assertEquals("expected no stderr on failure", "", e.getStdErr());
            assertEquals("expected magic return result when command does not exist", -102, e.getRetCode());
            assertNotNull(e.getCause());

            return;
        }
    }

    public void testStdoutStderrSleep30()
    {
        // this writes to stdout and stderr and then sleeps 30 seconds
        //
        final String [] cmd = { "test/exec-scripts/stdout_stderr_sleep30.pl" };

        try
        {
            // this process will return after 30 seconds normally, but we'll
            // interrupt it after 3.  If it takes more than 5 seconds to return
            // we failed to interrupt it, and when it returns we should have
            // read its output.
            //
            testTimedExec(cmd, 3, 5);
            fail("failed to interrupt process");
        }
        catch (ExecException e)
        {
            assertEquals("terrible stderr\n", e.getStdErr());
            assertEquals("wonderful stdout\n", e.getStdOut());
            assertEquals("command name wrong", cmd[0], e.getCommand());
            assertEquals(0, e.getRetCode());
            Throwable cause = e.getCause();
            assertTrue(cause instanceof TimeoutException);
        }
    }

    public void testInvalidCommandName()
    {
        final String cmd = "test/exec-scripts/this-command-does-not-exist.pl";

        try
        {
            ExecUtils.runCommand(cmd);
            fail("failed to error when invalid command passed");

        }
        catch (ExecException e)
        {
            assertEquals("expected no stdout on failure\n", "", e.getStdOut());
            assertEquals("expected no stderr on failure", "", e.getStdErr());
            assertEquals("expected magic return result", -102, e.getRetCode());
            assertNotNull(e.getCause());

            return;
        }
    }

}
