/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Runtime.InteropServices;
using System.Net;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// Wraps calls to UI to prompt for new credentials.
	/// </summary>
	public class Credentials : ICredentials
	{
		#region Instance data

		// Note that sUser is of the form domain\username if these are ntlm credentials
		protected string sTarget;
		protected string sUser;
		protected string sPassword;

		#endregion

		#region Constructor

		public Credentials(string sTarget, string sUser, string sPassword)
		{
			this.sTarget = sTarget;
			this.sUser = sUser;
			this.sPassword = sPassword;
		}

		#endregion

		#region Static and instance methods

		/// <summary>
		/// Looks to see if there are any stored credentials for the given target name. Note
		/// that, as long as the name can be resolved to an IP address, different forms of the
		/// name (e.g. short and FQDNs) will resolve to the same credentials.
		/// </summary>
		/// <param name="sTarget"></param>
		/// <returns>
		///		An initialized Credentials object
		/// </returns>
		public static Credentials Lookup(string sTarget)
		{
			string sUser;
			string sPassword;

			try
			{
				if (LookupCredentials(sTarget.ToLower(), out sUser, out sPassword)==0)
					return new Credentials(sTarget, sUser, sPassword);
				else
					return null;
			}
			catch(Exception)
			{
				return null;
			}
		}

		/// <summary>
		/// Prompts for new credentials for the given target
		/// </summary>
		/// <param name="hwndParent"></param>
		/// <param name="sTarget"></param>
		/// <returns>
		///		An initialized Credentils object
		/// </returns>
		public static Credentials Prompt(uint hwndParent, string sTarget)
		{
			// initialize the dialog with the current user name
			string szUserIn = System.Environment.UserDomainName + "\\" + System.Environment.UserName;
			string sUserOut;
			string sPasswordOut;

			try
			{
				if (PromptForCredentials(hwndParent, sTarget, szUserIn, "", out sUserOut, out sPasswordOut)==0)
					return new Credentials(sTarget, sUserOut, sPasswordOut);
				else
					return null;
			}
			catch(Exception)
			{
				return null;
			}

		}

		/// <summary>
		/// Saves the credentials in the credentials store
		/// </summary>
		/// <returns>
		///		true if it succeeded
		/// </returns>
		public bool Save()
		{
			try
			{
				if (SaveCredentials(sTarget, sUser, sPassword)==0)
					return true;
				else
					return false;
			}
			catch(Exception)
			{
				return false;
			}
		}

		/// <summary>
		/// Stores the credentials set in a soap service
		/// </summary>
		/// <param name="client"></param>
		public void SetCredentials(System.Web.Services.Protocols.SoapHttpClientProtocol client)
		{
			if (IsNTLM)
			{
				client.Credentials = this;
			}
			else
			{
				// TODO: remove the preauthenticate stuff since it doesn't seem to help
				client.PreAuthenticate = true;
				client.Credentials = this;
			}
		}

		/// <summary>
		/// Sets the credentials on a soap service call to the current (default) user credentials
		/// </summary>
		/// <param name="client"></param>
		public static void SetDefaultCredentials(System.Web.Services.Protocols.SoapHttpClientProtocol client)
		{
			client.Credentials = CredentialCache.DefaultCredentials;
		}

		#endregion

		#region ICredentials Members

		/// <summary>
		/// Called by objects that need an ICredentials implementor. By returning ONLY the available
		/// set of credentials (NTLM or BASIC) we can work with either form of authentication
		/// </summary>
		/// <param name="uri"></param>
		/// <param name="authType"></param>
		/// <returns>
		///		A created NetworkCredential object or null if not available for this authType
		/// </returns>
		public NetworkCredential GetCredential(Uri uri, string authType)
		{
			if (authType.ToLower()=="basic" && !IsNTLM)
				return new NetworkCredential(User, Password);
			else if (authType.ToLower()=="ntlm" && IsNTLM)
				return new NetworkCredential(User2, Password, Domain);

			return null;
		}

		#endregion

		#region Property accessors

		/// <summary>
		/// Returns the target associated with the credentials
		/// </summary>
		public string Target
		{
			get
			{
				return sTarget;
			}
		}

		/// <summary>
		/// Returns the username (or domain\username) associated with the credentials
		/// </summary>
		public string User
		{
			get
			{
				return sUser;
			}
		}

		/// <summary>
		/// Returns only the short username associated with the credentials
		/// </summary>
		public string User2
		{
			get
			{
				if (IsNTLM)
				{
					// strip the domain
					return sUser.Substring(sUser.IndexOf('\\')+1);
				}
				else
					return sUser;
			}
		}

		/// <summary>
		/// Returns the password associated with the credentials
		/// </summary>
		public string Password
		{
			get
			{
				return sPassword;
			}
		}

		/// <summary>
		/// Returns the domain associated with the credentials or NULL if these aren't NTLM credentials
		/// </summary>
		public string Domain
		{
			get
			{
				int islash = sUser.IndexOf('\\');
				if (islash<0)
					return null;
				else
					return sUser.Substring(0, islash);
			}
		}

		/// <summary>
		/// Returns the true if the credentials are NTLM credentials (specify a domain name)
		/// </summary>
		public bool IsNTLM
		{
			get
			{
				return Domain!=null;
			}
		}

		#endregion

		#region PInvoke declarations for DStoreDLL

		[DllImport("dstoredll.dll", EntryPoint="_PromptForCredentials@24", ExactSpelling=true, CharSet=CharSet.Unicode)]
		public static extern uint PromptForCredentials(uint hwndParent, string sTarget, string sUser, string sPassword, out string sUserOut, out string sPassOut);

		[DllImport("dstoredll.dll", EntryPoint="_LookupCredentials@12", ExactSpelling=true, CharSet=CharSet.Unicode)]
		public static extern uint LookupCredentials(string sTarget, out string sUserOut, out string sPassOut);

		[DllImport("dstoredll.dll", EntryPoint="_SaveCredentials@12", ExactSpelling=true, CharSet=CharSet.Unicode)]
		public static extern uint SaveCredentials(string sTarget, string sUser, string sPassword);

		#endregion

	}
}
