//#define USE_SSL

/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Runtime.InteropServices;
using Ironring.Management.MMC;
using WF=System.Windows.Forms;
using System.Net;
using System.Xml;
using System.IO;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// Summary description for DeviceNode.
	/// </summary>
	public class DeviceNode : FormNode
	{
		#region Constants

		// TODO: put into string resources?
		protected const string sDisplayName = "Device Manager";
		protected const string sPrimaryModeDisplayName = "Linux Device Manager";

		// error messages
		protected const string szErrorAccessDenied = "Access denied. The current user is not an administrator";

		// menu strings
		protected const string sMenuConnect = "&Connect to another computer...";

		// hover tips
		protected const string sTipConnect = "Manage a different computer.";

		// the "well-known name" of the clipboard format used by the Microsoft
		// MMC computer management snap-ins for the purposes of communicating the
		// target server name
		protected const string scfMachineName = "MMC_SNAPIN_MACHINE_NAME";

		// maximum name length of a machine
		// TODO: get this from somewhere?
		protected const int cbMaxNameLength = 256*2;

		// the SSL port used by Centeris
		protected const int CENTERIS_SSL_PORT = 9105;
		protected const int CENTERIS_HTTP_PORT = 9106;

		// help file
		protected const string sDeviceHelpTopic = "LW-open.chm::/C/Likewise-Open/about.htm";

		#endregion

		#region Data

		// server being looked at. Setting it to "" means "local machine"
		protected string sServerName="";

		// the handle associated with this clipboard format
		protected uint cfMachineName=0;

		// flag that tells us whether we're loaded as a primary MMC or as an extension
		protected bool bPrimaryMode=false;

		// pointer to embedded treeview
		protected MyTree tree = null;

		// data retrieved from linux box
		protected string xmlData = null;

		#endregion

		#region Constructor and initialization

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="snapin"></param>
		public DeviceNode(SnapinBase snapin) : base(snapin)
		{
			DisplayName = sDisplayName;
			ControlType = Type.GetType("Centeris.MMC.DeviceSnapin.MyTree");
			Pasteable = false;
			Cutable = false;

			m_Images = new ImageList();
			m_Images.Add("Centeris.MMC.DeviceSnapin.res.scope16.ico");
			m_Images.Add("Centeris.MMC.DeviceSnapin.res.scope32.ico");
			OpenImageIndex = ClosedImageIndex = SmallImageIndex = 0;
			LargeImageIndex = 1;

		}

		/// <summary>
		/// Initializes a new Device node. Note that we do not populate the tree yet
		/// until later.
		/// </summary>
		public override void Initialize()
		{
			// perform base initialization
			base.Initialize ();

			// register the funky clipboard format for remote connections
			cfMachineName = RegisterClipboardFormat(scfMachineName);

			// add the top level menu items
			if (PrimaryMode)
				AddTopMenuItem(new MenuItem(sMenuConnect, sTipConnect, new MenuCommandHandler(MenuHandlerConnect)));
		}

		#endregion

		#region Property accessors

		/// <summary>
		/// Property accessor for sServerName. If the server name has been
		/// set to "", return the real machine name. This simplifies processing
		/// for menu handlers and treats all cases as "remote"
		/// </summary>
		public string ServerTarget
		{
			get
			{ 
				return sServerName;
			}
		}


		/// <summary>
		/// returns true if we're the primary snap-in (we're not extending anyone)
		/// </summary>
		public bool PrimaryMode
		{
			get
			{
				return bPrimaryMode;
			}
			set
			{
				bPrimaryMode = value;
			}
		}
		#endregion

		#region Overrides

		/// <summary>
		/// Called when the current snapin is extending another one and the "parent" snapin node
		/// is being expanded. This is a convenient point to query what machine we are being
		/// told to work against.
		/// </summary>
		/// <param name="lpDataObject"></param>
		/// <param name="relativeId"></param>
		public override void OnExtensionExpand(IDataObject lpDataObject, int relativeId)
		{
			// check what machine we're talking to
			string sMachine = GetMachineName(lpDataObject);

			// cache the server name and flush old data
			SetServerTarget(sMachine);

			// let the base do its thing (basically, to insert the node into the scope pane)
			base.OnExtensionExpand(lpDataObject, relativeId);
		}

		/// <summary>
		/// Override this so that we can refresh the result pane
		/// </summary>
		public override void OnRefresh()
		{	
			// refetch our data
			FetchData();

			// tell the tree control to refetch
			tree.RefreshData();

			// let the base class do anything it needs to do
			base.OnRefresh();

		}

		/// <summary>
		/// Called when node is selected in the scope pane
		/// </summary>
		public override void OnSelectScope()
		{
			// enable the Refresh verb
			SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, true);

			// do what it's gotta do
			base.OnSelectScope();
		}

		/// <summary>
		/// Called when the user presses "Help" on a node
		/// </summary>
		public override void OnContextHelp()
		{
			IDisplayHelp dh = (IDisplayHelp) Snapin.Console;
			dh.ShowTopic(sDeviceHelpTopic);

			base.OnContextHelp();
		}

		/// <summary>
		/// OCXNode suppresses the loading of result pane images. Avoid
		/// this suppression!
		/// </summary>
		/// <param name="il"></param>
		public override void OnAddResultPaneImages(IImageList il)
		{
			if (ResultPaneImages != null)
				ResultPaneImages.LoadImageList(il, m_iCookie);

			// Now add all the children images
			foreach(BaseNode node in m_ChildNodes)
				node.OnAddResultPaneImages(il);

			m_events.Fire(OnAddResultPaneImagesKey, this, new AddImagesArgs(il));
		}


		#endregion

		#region Other methods

		/// <summary>
		/// Callback for embedded tree view to tell us about itself
		/// </summary>
		/// <param name="tv"></param>
		public void SetTree(MyTree tree)
		{
			this.tree = tree;
		}

		/// <summary>
		/// This method is called whenever the snap-in is looking at a new machine. It
		/// deletes any previous data in the data table and caches the new server name.
		/// </summary>
		/// <param name="sNewServerName"></param>
		public void SetServerTarget(string sNewServerName)
		{
			// don't do anything if the machine name is the same
			if (sServerName!=sNewServerName)
			{
				// cache
				if (sNewServerName==null)
					throw new Exception("Error! SetServerTarget() called with null argument");
				else
					sServerName = sNewServerName.ToUpper();

				// change in server. Fetch new data.
				FetchData();
			}
		}

		/// <summary>
		/// Handles the "Connect to another computer..." menu item. Calls a separate
		/// function since the other function can be invoked independently
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerConnect(object o, BaseNode node)
		{
			// get the new name
			HandleConnectForm();

			// have to tell the scope pane about the change

			// first, peg the console pointer
			IConsoleNameSpace2 cns = Snapin.ConsoleNameSpace;

			// now create and set up an sdi structure to describe
			// the change
			SCOPEDATAITEM sdi = new SCOPEDATAITEM();
			sdi.ID = this.HScopeItem;
			sdi.mask = (uint)SDI.STR;
			sdi.displayname = (IntPtr) (-1);	// -1 means "ask me for it"

			// set it
			cns.SetItem(ref sdi);

			// force a refresh of the result pane
			OnRefresh();
		}

		/// <summary>
		/// Note that this function can be called when the snap-in is first
		/// added (during the MMC Add/Remove snap-in procedure) before the
		/// scope pane or anything else is set up. As a result, it is 
		/// important to not do anything here that assumes that the rest of
		/// the UI is already running. If such is the case, it probably means
		/// that the operation is only relevant LATER and should be done in the
		/// menu handler (above).
		/// </summary>
		/// <returns></returns>
		public WF.DialogResult HandleConnectForm()
		{
			ConnectForm cf = new ConnectForm(ServerTarget);
			WF.DialogResult dr = cf.ShowDialog();
			PrimaryMode = true;
			if (dr==WF.DialogResult.OK)
			{
				SetServerTarget(cf.Target);
				DisplayName = sPrimaryModeDisplayName + " (" + ServerTarget + ")";

				// set the state to DIRTY so that we store the user-specified server name
				Snapin.SnapinState.Dirty = true;
			}
			else
			{
				SetServerTarget("");
				DisplayName = sPrimaryModeDisplayName + " (INVALID)";
			}

			return dr;
		}

		/// <summary>
		/// Returns a string of XML with an error node in it containing
		/// the specified message.
		/// </summary>
		/// <param name="sMessage"></param>
		/// <returns></returns>
		protected string CreateErrorNode(string sMessage)
		{
			XmlDocument xdoc = new XmlDocument();
			XmlNode xn = xdoc.CreateElement("error");
			xn.InnerText = sMessage;
			xdoc.AppendChild(xn);

			StringWriter sw = new StringWriter();
			xdoc.Save(sw);
			return sw.ToString();
		}

		/// <summary>
		/// Fetches new data from the Linux machine. Note that the data
		/// is not displayed here. It is fetched later by the tree control form
		/// </summary>
		protected void FetchData()
		{
			if (ServerTarget!="")
			{
				try
				{
					xmlData = GetXMLFromServer(this.Snapin.Handle, ServerTarget);
				}
				catch(Exception e)
				{
					if (e is WebException)
					{
						WebException we = (WebException) e;
						if (we.Status==WebExceptionStatus.ProtocolError)
						{
							// an access denied error
							xmlData = CreateErrorNode(szErrorAccessDenied);
							return;
						}
					}

					// Create a more generic error node
					xmlData = CreateErrorNode(e.Message);
				}
			}
			else
			{
				// create a specific error node
				xmlData = CreateErrorNode("Internal error: no server name specified");
			}
		}

		public static string GetXMLFromServer(System.IntPtr hwndParent, string server)
		{
			// prepare WS call
			SystemInfo sm = new SystemInfo();
			sm.CookieContainer = new CookieContainer();

			// change the URL to reference the corrent machine and to use the SSL port
			UriBuilder ub = new UriBuilder(sm.Url);
			ub.Host = server;

#if USE_SSL
			ub.Scheme = "https";
			ub.Port = CENTERIS_SSL_PORT;
#else
			ub.Scheme = "http";
			ub.Port = CENTERIS_HTTP_PORT;
#endif
			sm.Url = ub.ToString();

			// change the service point manager so that we can apply our own rules for certificate checking
			ServicePointManager.CertificatePolicy = new CenterisCertPolicy();

			// set the credentials to use current creds
			sm.Credentials = CredentialCache.DefaultCredentials;
			
			// perform the operation (throws exception on failure
			return sm.getDeviceTree();

		}

		/// <summary>
		/// This method is called by the tree in order to fetch the latest
		/// xml data string
		/// </summary>
		/// <returns></returns>
		public string GetDeviceData()
		{
			return xmlData;
		}

		#endregion

		#region PInvoke code to interact with IDataObject
		
		/// <summary>
		/// The following declarations are PInvoke definitions that allow us
		/// to query a data object for a prescribed clipboard format. We used
		/// this to figure out if the Computer Management snapin has specified
		/// name of the computer that we're connected to.
		/// </summary>
		
		// Used to get/set the global handle associated with a particular clipboard format
		[DllImport("user32.dll", CharSet=CharSet.Unicode)]
		protected static extern ushort RegisterClipboardFormat(String format);

		/// <summary>
		/// Asks the designated IDataObject for the target machine name.
		/// </summary>
		/// <param name="lpDataObject"></param>
		/// <returns></returns>
		protected string GetMachineName(IDataObject lpDataObject)
		{
			// some constants for interacting with lpDataObject.GetDataHere
			uint DVASPECT_CONTENT = 1;	// HACK: should have an enum somewhere
			uint TYMED_HGLOBAL = 1;		// HACK: should have an enum somewhere
			int cbData = cbMaxNameLength;
		    
			// prepare the FORMATETC and STGMEDIUM structures in order to call GetDataHere
			FORMATETC formatetc = new FORMATETC();
			formatetc.cfFormat = (int) cfMachineName;
			formatetc.ptd = 0;
			formatetc.dwAspect = DVASPECT_CONTENT;
			formatetc.lindex = -1;
			formatetc.tymed = TYMED_HGLOBAL;
			STGMEDIUM stgmedium = new STGMEDIUM();
			stgmedium.tymed = TYMED_HGLOBAL;
			stgmedium.hGlobal = (int) Marshal.AllocHGlobal(cbData);
			stgmedium.pUnkForRelease = null;

			if (stgmedium.hGlobal==0)
				throw new Exception("GetMachine() Failed to allocate memory");

			// request the designated data
			lpDataObject.GetDataHere(ref formatetc, ref stgmedium);

			// convert from a COM pointer to a string. Returns "" if no value
			// was found
			string sMachine = Marshal.PtrToStringUni((IntPtr) stgmedium.hGlobal);

			// free the buffer allocated to hold the returned data
			Marshal.FreeHGlobal((IntPtr)stgmedium.hGlobal);    

			// return the data in string form
			return sMachine;

		}

		#endregion
	}
}
