/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.IO;
using Ironring.Management.MMC;
using System.Data;
using Microsoft.Win32;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// MainClass is the snapin class for the Centeris Device Manager snapin. It
	/// extends the IronRing SnapinBase class.
	/// 
	/// The snapin can run standalone or as an extension to the System Tools node
	/// of the computer management snapin.
	/// </summary>
	[
		SnapinIn(Constants.DEVICE_SNAPIN_NAME,
			"Centeris Corp",
			"1.0.0.0",     // NOTE: This is overridden later in the About dialog
			true),
		ProgId(Constants.DEVICE_SNAPIN_PROGID),
		Guid(Constants.DEVICE_SNAPIN_CLSID),
		ExtendsNode(Constants.SYSTEM_TOOLS_NODETYPE, ExtensionType.NameSpace,
			@"SOFTWARE\Microsoft\MMC\NodeTypes.Centeris")
	]
	public class MainClass : SnapinBase, ISnapinHelp2
	{
		// Likewise help file
		protected const string sHelpFile = "LW-open.chm";

		// The fully qualified help file name 
		protected string sFullHelpFilePath = null;

		// The name of the resource that contains the images for the scope pane
		protected const string sScopePaneImages = "Centeris.MMC.DeviceSnapin.res.scope16.ico";

		// The name of the resource that contains the images for the result pane
		//protected const string sResultPaneImages = "Centeris.MMC.DeviceSnapin.res.result.ico";

		// windows error numbers (don't define as const or you'll get compiler errors!)
		protected uint E_NOTIMPL = 0x80004001;

		// Flag to assure that the ConnectForm is only shown once, when in 
		// primary mode
		protected bool bShowLogon = true;

		/// <summary>
		/// The main constructor for the snapin
		/// </summary>
		public MainClass()
		{
			// Create the icons used by the scope and result panes
			initIcons();

			// create a form node for the root
			FormNode r = new DeviceNode(this);

			// add a property page to it
			// NOTE: Because we are implementing this as an OCXnode, we can't use the MMCLIB property
			// sheet stuff (I don't think).
			// r.AddPropertyPage("Properties", Type.GetType("Centeris.MMC.DeviceSnapin.PropPage"));

			// initialize this member or nothing shows up
			base.m_Component = new Component(this);

			// set the root node for the snapin
			base.RootNode = r;

			// look up help info in the registry
			// first, figure out where we're installed

			// open the local machine hive
			RegistryKey rkcr = Registry.ClassesRoot;

			// now, get CLSID\{GUID}\InprocServer32
			string sSnapKey = @"CLSID\{" + Constants.DEVICE_SNAPIN_CLSID + @"}\InProcServer32";
			RegistryKey rkSnap = rkcr.OpenSubKey(sSnapKey);

			// look for the codebase
			if (rkSnap!=null)
			{
				string sSnapPath = (string) rkSnap.GetValue("CodeBase");
				if (sSnapPath!=null)
				{
					// canon
					sSnapPath = sSnapPath.ToLower();

					// strip off the protocol stuff
					if (sSnapPath.StartsWith("file:///"))
						sSnapPath = sSnapPath.Substring(8);		// length of "file:///"

					// strip off dll name
					string sDll = Path.GetFileName(sSnapPath);
					if (sDll!=null)
					{
						int iDll = sSnapPath.LastIndexOf(sDll);
						if (iDll>=0)
							sSnapPath = sSnapPath.Substring(0, iDll);
					}

					sFullHelpFilePath = sSnapPath + sHelpFile;
				}
			}


		}

		/// <summary>
		/// Creates the icons used by the scope and result panes
		/// </summary>
		protected void initIcons()
		{
			// add-in all the images used in the scope pane
			AddImage(sScopePaneImages);
		}

		#region Base overrides

		// If you load a snapin in the MMC (so a not saved snapin(.msc)) the mmc sends a msg to the
		// Snapin with the question 'do you have a SnapinManager for me?'
		// as this happens we set up our wizard, and we make sure that the Wizard is shown once. 
		// after that we let the base Class handle the QueryDataObject
		//
		// you could use a SnapinManager for Startup configuration, like the Microsoft Services plugin
		// it asks if you want to connect to a remote server or want work on the local machine.
		public override void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject)
		{
			if(type==(uint)CCT.SNAPIN_MANAGER && bShowLogon)
			{
				DeviceNode n = (DeviceNode) RootNode;
				if (n.HandleConnectForm()!=System.Windows.Forms.DialogResult.OK)
				{
					// set this flag to avoid multiple showings
					bShowLogon = false;

					// you would think this would be enuf!
					ppDataObject = null;

					// Throw an exception to terminate the operation
					throw new NotImplementedException();
				}

				// set this flag to avoid multiple showings
				bShowLogon = false;
			} 
			base.QueryDataObject(cookie, type, out ppDataObject);
		}

		/// <summary>
		/// Override creates a different state object
		/// </summary>
		/// <returns></returns>
		public override SnapinStateBase CreateSnapinState() 
		{
			return new DeviceState(this);
		}
		#endregion


		#region ISnapinHelp2 Members

		void Ironring.Management.MMC.ISnapinHelp2.GetLinkedTopics(out System.IntPtr lpCompiledHelpFiles)
		{
			// through an S_FALSE return
			throw new SFalseException();
		}

		void Ironring.Management.MMC.ISnapinHelp2.GetHelpTopic(out System.IntPtr lpCompiledHelpFile)
		{
			if (sFullHelpFilePath!=null)
			{
				lpCompiledHelpFile = System.Runtime.InteropServices.Marshal.StringToCoTaskMemAuto(sFullHelpFilePath);
			}
			else
			{
				// through an S_FALSE return
				throw new SFalseException();
			}
		}

		#endregion
	}

	#region Self-registration stuff

	/// <summary>
	/// Must declare this in order to be self-installed by Wise installer
	/// </summary>
	[System.ComponentModel.RunInstaller(true)]
	public class RegisterMe: RegisterSnapin
	{
	}

	public class SFalseException: Exception
	{
		public SFalseException()
		{
			this.HResult = 1;	// S_FALSE;
		}
	}

	#endregion

}
