/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Reflection;
using Ironring.Management.MMC;

namespace Centeris.MMC.DeviceSnapin
{
	/// <summary>
	/// Summary description for MyTree.
	/// </summary>
	public class MyTree : System.Windows.Forms.UserControl, ISnapinLink 
	{

		#region Constants

		// error strings
		protected const string sServiceError = "Unable to retrieve device information from {0}";
		protected const string sServiceError2 = "Not connected to a valid Centeris-enabled Linux server";

		#endregion

		#region Data

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.ContextMenu cm;
		private System.Windows.Forms.MenuItem miProperties;
		private System.Windows.Forms.TreeView tv;

		// backreference to device node
		protected DeviceNode devnode;

		#endregion

		#region Constuctor, destruction and initialization

		public MyTree()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			Assembly a = Assembly.GetExecutingAssembly();
			System.Windows.Forms.ImageList wil = new System.Windows.Forms.ImageList();

			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.device.ico"));
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.cpu.ico"));
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.disk.ico"));
            wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.cd-dvd.ico"));
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.computer.ico"));	// BIOS
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.parallel.ico"));	// parallel
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.computer.ico"));	// memory
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.display.ico"));	// video/vga
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.usb.ico"));		// usb
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.ide.ico"));		// ide
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.card.ico"));		// network
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.keyboard.ico"));
			wil.Images.Add(LoadImage(a, "Centeris.MMC.DeviceSnapin.res.mouse.ico"));

			tv.ImageList = wil;

		}

		protected Icon LoadImage(Assembly a, string name)
		{
			string[] names = a.GetManifestResourceNames();

			Stream s = a.GetManifestResourceStream(name);
			return new Icon(s);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Methods

		/// <summary>
		/// Gets data from the host DeviceNode and rebuilds the tree control
		/// </summary>
		public void RefreshData()
		{
			// get the data from the DeviceNode and fill out the tree control
			string xml = DeviceNode.GetDeviceData();
			ProcessDeviceData(xml);
		}

		/// <summary>
		/// Property accessor for the host DeviceNode back reference. When this
		/// value is set, we ask the host for its latest data and display it.
		/// </summary>
		protected DeviceNode DeviceNode
		{
			get
			{
				return devnode;
			}
			set
			{
				// keep thenew value
				devnode = value;

				// set the back reference
				devnode.SetTree(this);

				// refresh our data
				RefreshData();
			}
		}

		private void ShowProps()
		{
			// check the type of node that is selected
			PropPage pp;
			if (tv.SelectedNode.Tag!=null)
				pp = new PropPage(tv.SelectedNode);
			else
				pp = new PropPage(	tv.SelectedNode.Text, 
									tv.ImageList.Images[tv.SelectedNode.SelectedImageIndex],
									tv.SelectedNode.Nodes.Count!=0 );
			pp.Show();
		}

		/// <summary>
		/// Double-click brings up property page
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tv_DoubleClick(object sender, System.EventArgs e)
		{
			if (tv.SelectedNode!=null && tv.SelectedNode.Nodes.Count==0)
				ShowProps();
		}

		protected override void OnPaint(PaintEventArgs pea)
		{
			// do a full refresh.
			// TODO: something smarter! Without this, though, we get repaint errors!
			Refresh();

			// base.OnPaint(pea);
		}

		private void miProperties_Click(object sender, System.EventArgs e)
		{
			ShowProps();
		}

		private void cm_Popup(object sender, System.EventArgs e)
		{
			if (tv.SelectedNode==null)
				miProperties.Enabled = false;
			else
				miProperties.Enabled = true;
		}

		private void tv_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button==System.Windows.Forms.MouseButtons.Right)
			{
				// get the selected node at the current position
				TreeNode tn = tv.GetNodeAt(e.X, e.Y);
				if (tn!=null)
					tv.SelectedNode = tn;
			}
		}

		#endregion

		#region XML processing

		/// <summary>
		/// processes an incoming xml device data string. if xml is null, an error is displayed, instead
		/// </summary>
		/// <param name="xml"></param>
		protected void ProcessDeviceData(string xml)
		{
			// zap everything
			tv.Nodes.Clear();

			// see if error state
			if (xml==null)
			{
				string sServer = DeviceNode.ServerTarget;
				string sError;
				if (sServer!="")
					sError = string.Format(sServiceError, DeviceNode.ServerTarget);
				else
					sError = sServiceError2;
				TreeNode tn = new TreeNode(sError, 0, 0);
				tv.Nodes.Add(tn);
				return;
			}
		
			try
			{
				XmlDocument xd = new XmlDocument();
				xd.LoadXml(xml);

				// skip document node?
				XmlNode node = (XmlNode) xd;
				node = node.FirstChild;
				
				// skip "xml" node if present
				if (node.Name.ToUpper()=="XML")
					node = node.NextSibling;

				// add stuff to tree
				tv.Nodes.Add(ProcessDeviceTree(node));
				tv.Nodes[0].Expand();

			}
			catch (Exception e)
			{
				tv.Nodes.Add(e.Message);
			}
		}

		/// <summary>
		/// Handles the deviceTree tag
		/// </summary>
		/// <param name="node"></param>
		/// <returns></returns>
		protected TreeNode ProcessDeviceTree(XmlNode node)
		{
			// create a tree node for the current xml node
			TreeNode tn = new TreeNode();

			// check to see if this is an error tag
			if (node.Name=="error")
			{
				// yep, create an error node
				tn.Text = node.InnerText;
				tn.ImageIndex = tn.SelectedImageIndex = 0;
				return tn;
			}

			// validate
			if (node.Name!="deviceTree")
			{
				tn.Text = "Error! Missing <deviceTree> tag";
				return tn;
			}

			tn.Text = DeviceNode.ServerTarget;

			// process the group nodes now
			for (int ic=0; ic<node.ChildNodes.Count; ic++)
			{
				XmlNode xc = node.ChildNodes[ic];
				TreeNode tnNewGroup = ProcessGroup(xc);

				// insert, alphabetically into tree.
				InsertAlphabeticallyWithMerge(tn, tnNewGroup);
			}

			// set these the same for simplicity
			tn.ImageIndex = 0;
			tn.SelectedImageIndex = tn.ImageIndex;

			return tn;
		}

		/// <summary>
		/// Inserts a treenode into another's child nodes. The insert is 
		/// done alphabetically. If a given node name already exists, the subchildren are
		/// merged (recursively)
		/// </summary>
		/// <param name="tnParent"></param>
		/// <param name="tnNew"></param>
		protected void InsertAlphabeticallyWithMerge(TreeNode tnParent, TreeNode tnNew)
		{
			// Look through the parent's children, finding the right place to insert.
			// If we find the same name, merge the children (recursively).
			int iCur;
			bool bHandled = false;
			for (iCur=0; iCur<tnParent.Nodes.Count; iCur++)
			{
				TreeNode tnCur = tnParent.Nodes[iCur];
				int nGroupNameDiff = tnCur.Text.CompareTo(tnNew.Text);

				if (nGroupNameDiff > 0)
				{
					// found the right place to insert
					tnParent.Nodes.Insert(iCur, tnNew);
					bHandled = true;
					break;
				}
				else if (nGroupNameDiff==0)
				{
					// found another node with the same group name, consolidate the children
					foreach(TreeNode tnc in tnNew.Nodes)
						InsertAlphabeticallyWithMerge(tnCur, tnc);

					bHandled = true;
					break;
				}
				// else, keep loooping
			}

			// if we ran off the end, then append
			if (!bHandled)
				tnParent.Nodes.Add(tnNew);
		}
	
		/// <summary>
		/// Handles the group tag
		/// </summary>
		/// <param name="xn"></param>
		/// <returns></returns>
		protected TreeNode ProcessGroup(XmlNode xn)
		{
			TreeNode tn = new TreeNode();
			if (xn.Name!="group")
			{
				tn.Text = "Error! Missing <group> tag";
				return tn;
			}

			// process children
			for (int ic=0; ic<xn.ChildNodes.Count; ic++)
			{
				XmlNode xc = xn.ChildNodes[ic];

				// handle some special cases
				if (xc.Name=="name")
					tn.Text = xc.InnerText;
				else if (xc.Name=="type")
					tn.ImageIndex = int.Parse(xc.InnerText) + 1;
				else if (xc.Name=="device")
				{
					System.Diagnostics.Debugger.Log(1, "note", "Group: " + tn.Text + " Icon: " + tn.ImageIndex + "\n");
					TreeNode tndev = ProcessDevice(xc, tn.ImageIndex);
					tn.Nodes.Add(tndev);
				}
				else
					tn.Text = "Error! Unexpected node type: " + xc.Name;
			}

			// set these the same for simplicity
			tn.SelectedImageIndex = tn.ImageIndex;

			return tn;	
		}

		/// <summary>
		/// Handles the device tag
		/// </summary>
		/// <param name="xn"></param>
		/// <returns></returns>
		protected TreeNode ProcessDevice(XmlNode xn, int iImage)
		{
			TreeNode tn = new TreeNode();
			Hashtable ht = new Hashtable();

			// set these the same for simplicity
			tn.ImageIndex = iImage;
			tn.SelectedImageIndex = iImage;

			for (int ic=0; ic<xn.ChildNodes.Count; ic++)
			{
				XmlNode xc = xn.ChildNodes[ic];

				// grab the tag name and inner text. Clean up the inner text
				string sKey = xc.Name;
				string sValue = xc.InnerText.Trim();
				if (sValue.StartsWith("\"") && sValue.EndsWith("\""))
					sValue = sValue.Substring(1, sValue.Length-2);

				// use the name tag as the tree node display string
				if (sKey=="Name")
					tn.Text = sValue;
				else
				{
					// use other nodes for property page values
					if (ht.ContainsKey(sKey))
						ht[sKey] = ((string) ht[sKey]) + " " + sValue;
					else
						ht.Add(sKey, sValue);
				}
			}

			// if stuff in the hashtable, keep the hash in the tag field of the tree node
			if (ht.Count>0)
				tn.Tag = ht;
			else
				tn.Tag = null;

			return tn;
		}
		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.tv = new System.Windows.Forms.TreeView();
			this.cm = new System.Windows.Forms.ContextMenu();
			this.miProperties = new System.Windows.Forms.MenuItem();
			this.SuspendLayout();
			// 
			// tv
			// 
			this.tv.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.tv.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tv.HideSelection = false;
			this.tv.ImageIndex = -1;
			this.tv.Location = new System.Drawing.Point(0, 0);
			this.tv.Name = "tv";
			this.tv.SelectedImageIndex = -1;
			this.tv.Size = new System.Drawing.Size(456, 352);
			this.tv.TabIndex = 0;
			this.tv.MouseUp += new System.Windows.Forms.MouseEventHandler(this.tv_MouseUp);
			this.tv.DoubleClick += new System.EventHandler(this.tv_DoubleClick);
			// 
			// cm
			// 
			this.cm.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																			   this.miProperties});
			this.cm.Popup += new System.EventHandler(this.cm_Popup);
			// 
			// miProperties
			// 
			this.miProperties.Index = 0;
			this.miProperties.Text = "&Properties";
			this.miProperties.Click += new System.EventHandler(this.miProperties_Click);
			// 
			// MyTree
			// 
			this.ContextMenu = this.cm;
			this.Controls.Add(this.tv);
			this.Name = "MyTree";
			this.Size = new System.Drawing.Size(456, 352);
			this.ResumeLayout(false);

		}
		#endregion

		#region ISnapinLink Members

		public BaseNode ContextNode
		{
			get
			{
				return DeviceNode;
			}
			set
			{
				DeviceNode = (DeviceNode) value;
			}
		}

		#endregion
	}
}
