/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

using System;
using System.Data;
using System.IO;
using Ironring.Management.MMC;
using System.Runtime.InteropServices;
using System.Diagnostics;
using WF=System.Windows.Forms;


namespace Centeris.MMC.PrinterSnapin
{
	/// <summary>
	/// This class handles the scope node for the Centeris Printers MMC snapin
	/// 
	/// The node is basically a report node, but with filtering turned off. It
	/// gets its data by using the NetShare* API which should work with both
	/// Linux and Windows
	/// </summary>
	public class PrinterNode: ReportNode
	{
		// Error message
		protected const string sErrorCaption = "Shared Printers";
		
		// internal constants
		protected const string sNodeName = "Shared Printers";
		protected const string sLocalSuffix = " (Local)";

		// server being looked at. Setting it to "" means "local machine"
		protected string sServerName="";

		// menu strings
		protected const string sMenuConnect = "&Connect to another computer...";
		protected const string sMenuAdd = "&Add A Printer...";
		protected const string sMenuDriver = "Add A Printer D&river...";
		protected const string sMenuManage = "&Manage Printer Jobs";
		protected const string sMenuProps = "&Change Printer Properties";

		// hover tips
		protected const string sTipConnect = "Manage a different computer.";
		protected const string sTipAdd = "Add a printer to your network.";
		protected const string sTipDriver = "Add a printer driver to a server.";
		protected const string sTipManage = "Cancel, pause or resume jobs in the printer queue.";
		protected const string sTipProps = "Change properties for the selected printer.";

		// help file
		protected const string sPrinterHelpTopic = "LW-open.chm::/C/Likewise-Open/about.htm";

		// cached menu item  for enable/disable
		protected MenuItem miProps;
		protected MenuItem miManage;

		// names of programs to run when menu's are invoked
		protected const string sPrintProg = @"rundll32.exe";
		protected const string sAddArgs = @"printui.dll,PrintUIEntry /c\\{0} /il";
		protected const string sAddDriverArgs = @"printui.dll,PrintUIEntry /c\\{0} /id";
		protected const string sPropArgs = @"printui.dll,PrintUIEntry /p /n ""\\{0}\{1}""";
		protected const string sManageArgs = @"printui.dll,PrintUIEntry /o /n ""\\{0}\{1}""";

		// confirmation messages and captions
		protected const string sConfirmMessage = @"Delete printer {0}?";
		protected const string sConfirmCaption = @"Shared Printers";

		// the "well-known name" of the clipboard format used by the Microsoft
		// MMC computer management snap-ins for the purposes of communicating the
		// target server name
		protected const string scfMachineName = "MMC_SNAPIN_MACHINE_NAME";

		// the handle associated with this clipboard format
		protected uint cfMachineName=0;

		// maximum name length of a machine
		// TODO: get this from somewhere?
		protected const int cbMaxNameLength = 256*2;

		// flag that tells us whether we're loaded as a primary MMC or as an extension
		protected bool bPrimaryMode=false;

		// flag that tells us whether we're in the error state. If we are, we don't try to refresh
		protected bool bInErrorState = false;

		// autorefresh timer
		protected const int msRefreshTimer = 15000;	// 15 seconds
		protected WF.Timer timer;

		// constructors
		public PrinterNode(SnapinBase snapin) : base(snapin)
		{
			DisplayName = sNodeName;
			Pasteable = false;
			Cutable = false;

			// setup image list
			m_Images = new ImageList();
			m_Images.Add("Centeris.MMC.PrinterSnapin.res.wprintfld16.ico");
			m_Images.Add("Centeris.MMC.PrinterSnapin.res.wprintfld32.ico");
			OpenImageIndex = ClosedImageIndex = 0;
			SmallImageIndex = 0;
			LargeImageIndex = 1;

			SetupTable();

			// create but don't start timer
			timer = new WF.Timer();
			timer.Interval = msRefreshTimer;
			timer.Tick += new EventHandler(TimerEventProcessor);

		}

		public void Finalize()
		{
			// stop any running timers
			timer.Stop();
		}

		/// <summary>
		/// Initializes a new Printer node. Note that we do not populate the data table
		/// until later.
		/// </summary>
		public override void Initialize()
		{
			// perform base initialization
			base.Initialize ();

			// register the funky clipboard format for remote connections
			cfMachineName = RegisterClipboardFormat(scfMachineName);

			// add the top level menu items
			if (PrimaryMode)
				AddTopMenuItem(new MenuItem(sMenuConnect, sTipConnect, new MenuCommandHandler(MenuHandlerConnect)));
			AddTopMenuItem(new MenuItem(sMenuAdd, sTipAdd, new MenuCommandHandler(MenuHandlerAdd)));
			AddTopMenuItem(new MenuItem(sMenuDriver, sTipDriver, new MenuCommandHandler(MenuHandlerAddDriver)));

			// cache these for disable/enable
			miManage = new MenuItem(sMenuManage, sTipManage, new MenuCommandHandler(MenuHandlerManage));
			miProps = new MenuItem(sMenuProps, sTipProps, new MenuCommandHandler(MenuHandlerChangeProps));

			// set all to hidden, by default
			miManage.Visible = false;
			miProps.Visible = false;

			// add em
			AddTopMenuItem(miManage);
			AddTopMenuItem(miProps);
		}

		


		/// <summary>
		/// Because changes can be made both inside and outside of MMC, we update
		/// the display on a periodic basis.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="args"></param>
		protected void TimerEventProcessor(object sender, EventArgs args)
		{
			// only refresh if we're not in the error state
			if (!bInErrorState)
				OnRefresh();
		}

		/// <summary>
		/// Menu handlers
		/// </summary>

		/// <summary>
		/// Handles the "Connect to another computer..." menu item. Calls a separate
		/// function since the other function can be invoked independently
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerConnect(object o, BaseNode node)
		{
			// get the new name
			if (HandleConnectForm()==WF.DialogResult.OK)
			{

				// have to tell the scope pane about the change

				// first, peg the console pointer
				IConsoleNameSpace2 cns = Snapin.ConsoleNameSpace;

				// now create and set up an sdi structure to describe
				// the change
				SCOPEDATAITEM sdi = new SCOPEDATAITEM();
				sdi.ID = this.HScopeItem;
				sdi.mask = (uint)SDI.STR;
				sdi.displayname = (IntPtr) (-1);	// -1 means "ask me for it"

				// set it
				cns.SetItem(ref sdi);

				// force a refresh of the result pane(s)
				OnRefresh();
			}
		}

		/// <summary>
		/// Note that this function can be called when the snap-in is first
		/// added (during the MMC Add/Remove snap-in procedure) before the
		/// scope pane or anything else is set up. As a result, it is 
		/// important to not do anything here that assumes that the rest of
		/// the UI is already running. If such is the case, it probably means
		/// that the operation is only relevant LATER and should be done in the
		/// menu handler (above).
		/// </summary>
		/// <returns></returns>
		public WF.DialogResult HandleConnectForm()
		{
			ConnectForm cf = new ConnectForm(!IsLocal, ServerTarget);
			WF.DialogResult dr = cf.ShowDialog();
			PrimaryMode = true;
			if (dr==WF.DialogResult.OK)
			{
				SetServerTarget(cf.Target);

				// set the state to DIRTY so that we store the user-specified server name
				Snapin.SnapinState.Dirty = true;
			}
			else
				SetServerTarget("");

			if (IsLocal)
				DisplayName = sNodeName + sLocalSuffix;
			else
				DisplayName = sNodeName + " (" + ServerTarget.ToUpper() + ")";

			return dr;
		}

		/// <summary>
		/// Handles the Add A Printer... menu item by launching
		/// the associated wizard
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerAdd(object o, BaseNode node)
		{
			// %SystemRoot%\System32\rundll32.exe printui.dll,PrintUIEntry /c\\%Server% /il

			// compose args
			string sArgs = string.Format(sAddArgs, ServerTarget);

			// run
			Exec(sPrintProg, sArgs);
		}

		/// <summary>
		/// Handles the Add A Printer Driver... menu item by launching
		/// the associated wizard
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerAddDriver(object o, BaseNode node)
		{
			// %SystemRoot%\System32\rundll32.exe printui.dll,PrintUIEntry /c\\%Server% /il

			// compose args
			string sArgs = string.Format(sAddDriverArgs, ServerTarget);

			// run
			Exec(sPrintProg, sArgs);
		}

		/// <summary>
		/// Handles the Manage Printer Jobs... menu item by launching printui with
		/// yet another set of args
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerManage(object o, BaseNode node)
		{
			// Get the selected row index
			int nSelRow = Snapin.GetSelectedResultRow();

			// validate
			if (nSelRow<0)
				return;

			// Fetch the share name from the first column from the table
			string sShare = (string) Table.Rows[nSelRow][PrintAPI.NameCol];

			// %SystemRoot%\System32\rundll32.exe printui.dll,PrintUIEntry /o /n \\%Server%\share /il

			// compose args
			string sArgs = string.Format(sManageArgs, ServerTarget, sShare);

			// run
			Exec(sPrintProg, sArgs);
		}

		/// <summary>
		/// Handles the Change Printer Properties menu item. It does this by launching printui with
		/// yet another set of args		\
		/// </summary>
		/// <param name="o"></param>
		/// <param name="node"></param>
		public void MenuHandlerChangeProps(object o, BaseNode node)
		{
			// Get the selected row index
			int nSelRow = Snapin.GetSelectedResultRow();

			// validate
			if (nSelRow<0)
				return;

			// Fetch the share name from the first column from the table
			string sShare = (string) Table.Rows[nSelRow][0];

			// %SystemRoot%\System32\rundll32.exe printui.dll,PrintUIEntry /p /n \\%Server%\share /il

			// compose args
			string sArgs = string.Format(sPropArgs, ServerTarget, sShare);

			// run
			Exec(sPrintProg, sArgs);
		}


		/// <summary>
		/// Override this to handle double clicking in the result pane - sames as showing props
		/// </summary>
		/// <returns></returns>
		public override bool OnDblClick()
		{
			int nSelRow = Snapin.GetSelectedResultRow();

			if (nSelRow>=0)
			{
				MenuHandlerChangeProps(null, this);
				return true;
			}
			else
				return base.OnDblClick();
		}

		/// <summary>
		/// Called when node is selected in the scope pane
		/// </summary>
		public override void OnSelectScope()
		{
			// enable the Refresh verb
			SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, true);

			// start the refresh timer
			timer.Start();

			// do what it's gotta do
			base.OnSelectScope();
		}

		/// <summary>
		/// Called when node is deselected in the scope pane
		/// </summary>
		public override void OnDeselectScope()
		{
			// stop the refresh timer
			timer.Stop();

			base.OnDeselectScope ();
		}

		/// <summary>
		/// Called when node is selected in the result pane
		/// </summary>
		public override void OnSelectResult()
		{
			// show relevant menu item since there's a selection
			miManage.Visible = true;
			miProps.Visible = true;

			// start the timer here, too, since OnDeselectScope will have been called
			// and this will have stopped the timer
			timer.Start();

			// enable the Refresh verb
			SetVerbState(MMC_CONSOLE_VERB.REFRESH, MMC_BUTTON_STATE.ENABLED, true);
			SetVerbState(MMC_CONSOLE_VERB.DELETE, MMC_BUTTON_STATE.ENABLED, true);

			// do what it's gotta do
			base.OnSelectResult();
		}

		/// <summary>
		/// Called when node is deselected in the result pane
		/// </summary>
		public override void OnDeselectResult()
		{
			// hide relevant menu items since there's no selection
			miManage.Visible = false;
			miProps.Visible = false;

			// stop the refresh timer (it will be restarted if focus is going back to the
			// printer scope pane node.
			timer.Stop();

		}

		/// <summary>
		/// Called when node is expanding. NOW fetch the data.
		/// </summary>
		public override void OnExpand()
		{
			// Get latest data
			FetchPrinterData();

			// start the refresh timer
			// NOTE: this is the only canonical place where we can do this
			// regardless of how (multiple views, primary, extension, etc.) our snapin can beused.
			timer.Start();

			// let the base do its thing
			base.OnExpand();
		}

		/// <summary>
		/// Called when the current snapin is extending another one and the "parent" snapin node
		/// is being expanded. This is a convenient point to query what machine we are being
		/// told to work against.
		/// </summary>
		/// <param name="lpDataObject"></param>
		/// <param name="relativeId"></param>
		public override void OnExtensionExpand(IDataObject lpDataObject, int relativeId)
		{
			// check what machine we're talking to
			string sMachine = GetMachineName(lpDataObject);

			// cache the server name and flush old data
			SetServerTarget(sMachine);

			// let the base do its thing (basically, to insert the node into the scope pane)
			base.OnExtensionExpand(lpDataObject, relativeId);
		}

		public override void OnViewChangeResult(BaseNode node)
		{
			// cache the right value here!
			m_Console = m_Snapin.ResultViewConsole;

			// Remove (and reshow) the data
			RemoveAllItems();

			base.OnViewChangeResult (node);
		}


		/// <summary>
		/// Override this so that we can refresh the result pane
		/// </summary>
		public override void OnRefresh()
		{	
			// get the data again
			FetchPrinterData();

			// tell the views to update themselves
			Snapin.ResultViewConsole.UpdateAllViews(null, 0, 0);

			// let the base class do anything it needs to do
			base.OnRefresh();
		}

		public override void OnDelete()
		{
				// Get the selected row index
				int nSelRow = Snapin.GetSelectedResultRow();

				// validate
				if (nSelRow<0)
					return;

				// Fetch the share name from the first column from the table
				string sShare = (string) Table.Rows[nSelRow][0];

				// ask for confirmation
				string sPrompt = string.Format(sConfirmMessage, sShare);
				if (WF.MessageBox.Show(sPrompt, sConfirmCaption, WF.MessageBoxButtons.OKCancel)==WF.DialogResult.OK)
				{
					try
					{
						// do it
						DeleteShare(sShare);

						// force a refresh of the result pane(s)
						OnRefresh();
					}
					catch(Exception e)
					{
						WF.MessageBox.Show(e.Message, sErrorCaption, WF.MessageBoxButtons.OK, WF.MessageBoxIcon.Warning);
					}

					base.OnDelete();
				}
		}

		/// <summary>
		/// Called when the user presses "Help" on a node
		/// </summary>
		public override void OnContextHelp()
		{
			IDisplayHelp dh = (IDisplayHelp) Snapin.Console;
			dh.ShowTopic(sPrinterHelpTopic);

			base.OnContextHelp();
		}

		/// <summary>
		/// Runs a program with a given set of arguments
		/// </summary>
		/// <param name="sProg"></param>
		/// <param name="sArgs"></param>
		protected void Exec(string sProg, string sArgs)
		{
			
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.WorkingDirectory = System.Environment.SystemDirectory;
			psi.FileName = sProg;
			psi.Arguments = sArgs;
			
			Process.Start(psi);
			
		}

		/// <summary>
		/// Performs a specific shell verb on a given filename
		/// </summary>
		/// <param name="sPath"></param>
		/// <param name="sVerb"></param>
		protected void ShellExec(string sPath, string sVerb)
		{
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.WorkingDirectory = System.Environment.SystemDirectory;
			psi.FileName = sPath;
			psi.UseShellExecute = true;
			psi.Verb = sVerb;
			psi.WindowStyle = ProcessWindowStyle.Normal;

			Process.Start(psi);
		}

		/// <summary>
		/// Overridable Helper method to populate the column header
		/// </summary>
		/// <param name="header"></param>
		public override void OnShowHeader(IHeaderCtrl2 header) 
		{
			try 
			{
				if (m_table != null)
					// Let's put in the column titles
					foreach (DataColumn col in m_table.Columns) 
					{
						string name = col.Caption;
						int ord = col.Ordinal;

						// Autowidth doesn't seem to help since we don't have data here yet
						// Thus, ask for help from someone who has a clue
						int nWidth = PrintAPI.GetColumnWidth(ord);

						header.InsertColumn(ord, name, (int)ColumnHeaderFormat.LEFT, nWidth);
					}
			}
			catch(COMException e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				throw e;
			}
			catch(Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				throw e;
			}
		}
		/// <summary>
		/// This method is called whenever the snap-in is looking at a new machine. It
		/// deletes any previous data in the data table and caches the new server name.
		/// Note that it doesn't actually fetch any data.
		/// </summary>
		/// <param name="sNewServerName"></param>
		public void SetServerTarget(string sNewServerName)
		{
			// don't do anything if the machine name is the same
			if (sServerName!=sNewServerName)
			{
				// cache
				if (sNewServerName==null)
					sServerName = "";
				else
					sServerName = sNewServerName.ToUpper();

				// change in server. Redo table.
				DataTable dt = this.Table;
				dt.Rows.Clear();

			}
		}

		/// <summary>
		/// Property accessor for sServerName. If the server name has been
		/// set to "", return the real machine name. This simplifies processing
		/// for menu handlers and treats all cases as "remote"
		/// </summary>
		public string ServerTarget
		{
			get
			{ 
				if (sServerName=="")
					return System.Environment.MachineName.ToUpper();
				else
					return sServerName.ToUpper();
			}
		}

		/// <summary>
		/// Returns true if the server name is blank or set to the same as the local
		/// computer name
		/// </summary>
		public bool IsLocal
		{
			get
			{
				return sServerName=="" || sServerName==System.Environment.MachineName.ToUpper();
			}
		}

		/// <summary>
		/// returns true if we're the primary snap-in (we're not extending anyone)
		/// </summary>
		public bool PrimaryMode
		{
			get
			{
				return bPrimaryMode;
			}
			set
			{
				bPrimaryMode = value;
			}
		}

		/// <summary>
		/// Helper function to create the data table.
		/// </summary>
		protected void SetupTable()
		{
			Table = PrintAPI.CreateDataTable();
		}


		/// <summary>
		/// Fills the data table with information retrieved from the targeted server (as
		/// per the sServerName member)
		/// </summary>
		protected void FetchPrinterData()
		{
			// delete any existing rows
			Table.Rows.Clear();

			// get new ones
			try
			{
				PrintAPI.FetchPrinterData(ServerTarget, Table);
				bInErrorState = false;
			}
			catch(Exception e)
			{
				bInErrorState = true;
				WF.MessageBox.Show(e.Message, sErrorCaption, WF.MessageBoxButtons.OK, WF.MessageBoxIcon.Warning);
			}
		}

		/// <summary>
		/// Removes a shared resource
		/// </summary>
		/// <param name="sShare">The "share name" of the resource</param>
		/// <returns></returns>
		public void DeleteShare(string sShare)
		{
			PrintAPI.DeletePrinter(ServerTarget, sShare);
		}

		/// <summary>
		/// The following declarations are PInvoke definitions that allow us
		/// to query a data object for a prescribed clipboard format. We used
		/// this to figure out if the Computer Management snapin has specified
		/// name of the computer that we're connected to.
		/// </summary>
		
		// Used to get/set the global handle associated with a particular clipboard format
		[DllImport("user32.dll", CharSet=CharSet.Unicode)]
		protected static extern ushort RegisterClipboardFormat(String format);

		/// <summary>
		/// Asks the designated IDataObject for the target machine name.
		/// </summary>
		/// <param name="lpDataObject"></param>
		/// <returns></returns>
		protected string GetMachineName(IDataObject lpDataObject)
		{
			// some constants for interacting with lpDataObject.GetDataHere
			uint DVASPECT_CONTENT = 1;	// HACK: should have an enum somewhere
			uint TYMED_HGLOBAL = 1;		// HACK: should have an enum somewhere
			int cbData = cbMaxNameLength;
		    
			// prepare the FORMATETC and STGMEDIUM structures in order to call GetDataHere
			FORMATETC formatetc = new FORMATETC();
			formatetc.cfFormat = (int) cfMachineName;
			formatetc.ptd = 0;
			formatetc.dwAspect = DVASPECT_CONTENT;
			formatetc.lindex = -1;
			formatetc.tymed = TYMED_HGLOBAL;
			STGMEDIUM stgmedium = new STGMEDIUM();
			stgmedium.tymed = TYMED_HGLOBAL;
			stgmedium.hGlobal = (int) Marshal.AllocHGlobal(cbData);
			stgmedium.pUnkForRelease = null;

			if (stgmedium.hGlobal==0)
				throw new Exception("GetMachine() Failed to allocate memory");

			// request the designated data
			lpDataObject.GetDataHere(ref formatetc, ref stgmedium);

			// convert from a COM pointer to a string. Returns "" if no value
			// was found
			string sMachine = Marshal.PtrToStringUni((IntPtr) stgmedium.hGlobal);

			// free the buffer allocated to hold the returned data
			Marshal.FreeHGlobal((IntPtr)stgmedium.hGlobal);    

			// return the data in string form
			return sMachine;

		}

		public override bool HandleCustomClipboardFormat(ushort cf, BinaryWriter writer)
		{
			if (cfMachineName == cf)
			{
				// note: just calling the "String" method of BinaryWriter::Write() doesn't work
				//       because it will be written in ASCII, rather than unicode.  Using
				//       String::ToCharArray() doesn't work, because we're 
				//
				writer.Write(ServerTarget.ToCharArray());

				return true;
			}
			return false;
		}


	}
}
