using System;
using System.Drawing;
using System.Runtime.InteropServices;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
    ///  Component implements the IComponent, IExtendContextMenu and IExtendControlbar
    ///  interfaces that control the result view side of the MMC control.  
    ///  Many event notifications pass through this class on the way to the node
    /// </summary>
	public class Component :
		IComponent
		,IExtendContextMenu
		,IExtendControlbar
		,IExtendPropertySheet
        //,IExtendTaskPad
		//,IResultDataCompareEx
	{
		protected bool m_MultiSelectionMode = false;

		public bool MultiSelectionMode
		{
			get{return m_MultiSelectionMode;}
		}

		/// <summary>
		/// cache the MMC Console interface 
		/// </summary>
		protected IConsole2 m_Console;        
    	
		/// <summary>
		/// The control bar set by MMC when each node is selected
		/// Careful since it can go null!
		/// </summary>
		protected IControlbar m_pControlbar;

		//Added by Alexander Kachalkov
		/// <summary>
		/// Toolbar buttons array
		/// </summary>
		protected MMCBUTTON[] m_ToolbarButtons;

		/// <summary>
		/// cache the MMC Toolbar interface
		/// </summary>
		protected IToolbar m_pToolbar;

		/// <summary>
		/// Toolbar image index in the global collection
		/// </summary>
		protected int m_ToolbarImageIndex;
		
		//Added by Alexander Kachalkov
		/// <summary>
		/// Get or Set Toolbar buttons
		/// </summary>
		public MMCBUTTON[] ToolbarButtons
		{
			get{return m_ToolbarButtons;}
			set{m_ToolbarButtons = value;}
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Index of the Toolbar bitmap in the global Image list 
		/// </summary>
		public int ToolbarImageIndex
		{
			get{return m_ToolbarImageIndex;}
			set{m_ToolbarImageIndex = value;}
		}

        /// <summary>
        /// Back reference to the Snapin Base object 
        /// </summary>
	    protected SnapinBase m_Snapin;

        //////////////////////////////////////////////////////////////
        //
        // Implementation
        //
		#region

        
        /// <summary>
        /// ctor with a back ref to the base snapin
        /// </summary>
        /// <param name="snapin"></param>
        public Component(SnapinBase snapin)
        {
		    m_Snapin = snapin;
        }


        /// <summary>
        /// return the cached console reference
        /// </summary>
        public IConsole2 Console 
        {
            get { return m_Console; }
        }


        /// <summary>
        /// Return the controlbar.  It may be null 
        /// </summary>
        public IControlbar ControlBar 
        {
            get { return m_pControlbar; }
        }

		/// <summary>
		/// Return the toolbar.  It may be null 
		/// </summary>
		public IToolbar ToolBar
		{
			get { return m_pToolbar; }
		}
		#endregion
		//////////////////////////////////////////////////////////////
        //
        // IComponent Implementation
        //
		#region
        /// <summary>
        /// Called by MMC on startup
        /// </summary>
        /// <param name="pConsole"></param>
        public void Initialize(Object pConsole)
        { 
            m_Console = pConsole as IConsole2;
        }
        
        /// <summary>
        /// This is the primary MMC communication chanel concerning, it will 
        /// call Notify, we will delegate to the relevent node or handle it directly
        /// </summary>
        /// <param name="lpDataObject"></param>
        /// <param name="aevent"></param>
        /// <param name="arg"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public int Notify(IntPtr lpDataObject, uint aevent, IntPtr arg, IntPtr param)
        {
            int hr = HRESULT.S_FALSE;

			m_Snapin.SetCurrentComponent(this); 

            try
            {
                // There are some sneaky values for the IDataObject parameters that make 
                // interop with this interface a little trick.  This is why lpDataObject is an IntPtr
                // not an actual IDataObject interface.
                if (lpDataObject == (IntPtr)DataObject.CUSTOMOCX || lpDataObject == (IntPtr)DataObject.CUSTOMWEB)
                    return hr;

                // marshal the IDataObject interfacer now that we know its likely a valid interface pointer
                IDataObject ido = null;
				if (lpDataObject != (IntPtr)0)
					ido = (IDataObject)Marshal.GetObjectForIUnknown(lpDataObject);
                   
                // TBD: this may fail for "multiselect" notificatyions
				//Added by Alexander Kachalkov
				// Fixed some issues, but it could still fail...
				hr = HRESULT.S_OK;

                // Do the same old trick - since we only hand out DataObject we can cast back if it non-null.
                // if the IDataObject is null MMC is signalling us to operate on the root/static node.
				BaseNode node = null;

				//node = (ido == null) ? m_Snapin.RootNode : ((DataObject)ido).Node;
				//Added by Alexander Kachalkov
				// TBD: if ido is null this is multiselect DELETE event
				// At present don't know how to make it Interop.MMC.MULTI_SELECT_COOKIE (-2).
				if (ido != null)
				{
					// NOTE: Manny Vellon 8/15/2005
					// If the current snapin is an extension snapin, then the incoming lpDataObject
					// refers to the parent node which might NOT be an IronRing snapin. If this is the
					// case, fallback to using the RootNode.

					if ( ido is DataObject)
						node = ((DataObject)ido).Node;
					else
						node = m_Snapin.RootNode;
				}
				else if (lpDataObject != IntPtr.Zero)
					//Just get the node of the selected item
					//This is working, but may be i miss something...
				{
					RESULTDATAITEM rdi = m_Snapin.GetSelectedItem((IResultData)m_Console);
					if (rdi.nIndex != -1)
						node = m_Snapin.FindNode(rdi.lParam);
				}
				else 
				{
					// MAM: don't fallback to the RootNode.
					node = m_Snapin.CurrentScopeNode;
					if(node == null) 
					{
						System.Diagnostics.Debug.WriteLine("Huh? no current scope node. Falling back to rootnode");
						node = m_Snapin.RootNode;

					}
				}

                // dispatch the evnt to the node
                MMCN_Notify notify = (MMCN_Notify)aevent;
                System.Diagnostics.Debug.WriteLine("IComponent::Notify - " + notify.ToString());

                switch(notify)
                {
                    case MMCN_Notify.ACTIVATE:
                    {
                        bool bActivate = arg.ToInt32() != 0;
                        node.OnActivate(bActivate);
                        break;
                    }
                    case MMCN_Notify.ADD_IMAGES:
                    {
                        // arg actually contains the IImageList interface. We need to 
                        // marshall that manually as well.

                        IImageList il = (IImageList)Marshal.GetObjectForIUnknown(arg);

                        // param contains the HScopeItem. Let's get the node it represents
                        BaseNode nLuckyGuy = m_Snapin.FindNodeByHScope((int)param);

						// Manny Vellon 8/14/2005
						// If this is an extension, then param might not be relevant(?) and we
						// should fall back to the rootnode
						if (nLuckyGuy==null)
							nLuckyGuy = node;
           				
                        // TBD: what is the diff between nLuckyGuy and node
                        nLuckyGuy.OnAddResultPaneImages(il);

						break;
                    }
                    case MMCN_Notify.BTN_CLICK:
                    {
                        // CmdID of the button equal to a value of the MMC_COMMANDS enum type.
                        int cmdid = param.ToInt32();
                        node.OnBtnClick(cmdid);
                        break;
                    }
                    case MMCN_Notify.CLICK:
                    {
                        node.OnClick();
                        break;
                    }
                    case MMCN_Notify.COLUMN_CLICK:
                    {
                        int nCol = (int)arg;
                        bool bAscending = (int)param == 0;
                        node.OnColumnClick(nCol, bAscending);
                        break;
                    }
                    case MMCN_Notify.CUTORMOVE:
                    {
						//Changed by Wojciech Sobocinski
						IDataObject anido = (IDataObject)Marshal.GetObjectForIUnknown(arg);
						if(!node.OnCutOrRemove(anido))
							hr = HRESULT.S_FALSE;

						break;
					}
                    case MMCN_Notify.DBLCLICK:
                    {
						//Added by Alexander Kachalkov
						hr = HRESULT.S_FALSE;
						if (node.OnDblClick())
							hr = HRESULT.S_OK;
                        break;
                    }
                    case MMCN_Notify.DELETE:
                    {
						node.OnDelete();
                        break;
                    }
                    case MMCN_Notify.DESELECT_ALL:
                    {
                        node.OnDeselectAll();
                        break;
                    }
                    case MMCN_Notify.EXPAND:
                    {
                        if ((int)arg == 0)
                            node.OnExpand();
                        else
                            node.OnCollapse();
                        break;
                    }
                    case MMCN_Notify.MENU_BTNCLICK:
                    {
                        MENUBUTTONDATA data = (MENUBUTTONDATA)(Marshal.PtrToStructure(param, typeof(MENUBUTTONDATA)));
                        node.OnMenuBtnClick(data.idCommand, data.x, data.y);
                        break;
                    }
                    case MMCN_Notify.MINIMIZED:
                    {
                        if (arg.ToInt32() == 0)
                            node.OnMaximized();
                        else
                            node.OnMinimized();
                        break;
                    }
                    case MMCN_Notify.PASTE:
                    {
                        // TBD: arg = IDataObject for multi select
                        // param = NULL for move as opposed to cut
                        //       = BOOL* for for single item pasting...return TRUE if good
                        //       = IDataObject for multiselect paste
                        // return S_FALSE for no can do or S_OK if...OK
                        //				node.OnPaste();
						//Changed by Wojciech Sobocinski ** defaults to cut right now
						IDataObject anido = (IDataObject)Marshal.GetObjectForIUnknown(arg);
						if(node.OnPaste(anido))
						{
							//
							// need to set the param ptr
							//
							param = Marshal.GetIUnknownForObject(anido);
						}
						else
							hr = HRESULT.S_FALSE;
						break;
                    }
                    case MMCN_Notify.PROPERTY_CHANGE:
                    {
                        // lParam = user object
                        // node == null

                        //node.OnPropertyChange();
                        break;
                    }
                    case MMCN_Notify.QUERY_PASTE:
                    {
                        // TBD: arg = IDataObject for multi select
                        // return S_FALSE for no can do or S_OK if...OK
                        IDataObject anido = (IDataObject)Marshal.GetObjectForIUnknown(arg);
                        if (!node.OnQueryPaste(anido))
                            hr = HRESULT.S_FALSE;

                        break;
                    }
                    case MMCN_Notify.REFRESH:
                    {
						//Added by Alexander Kachalkov
						//Clear Result view
						//shouldn't this happen if the user wants this in OnRefresh ?
						//This allow not to clear items while OnShow event
						int pvo = 0;
						node.GetResultViewType(ref pvo);
						if((pvo & (int)MMC_VIEW_OPTIONS.NOLISTVIEWS)!=0)
						{
							// put this in a try/catch block since it can fail
							// TODO: why does it fail?
							try
							{
								if (m_Console is IResultData)
									((IResultData)m_Console).DeleteAllRsltItems();
							}
							catch(Exception)
							{
							}
						}
						node.OnRefresh();
						break;
                    }
					//Added by Attila Kapostyak
					case MMCN_Notify.PRINT:			
					{									
						node.OnPrint();					
						break;							
					}									
					case MMCN_Notify.REMOVE_CHILDREN:
                    {
                        BaseNode thenode = m_Snapin.FindNodeByHScope((int)arg);
                        thenode.OnRemoveChildren();
                        break;
                    }
                    case MMCN_Notify.RENAME:
                    {
                        // can we rename?
                        if ((int)arg == 0)
                        {
                            // can we rename?
                            if (!node.OnTryRename())
                                hr = HRESULT.S_FALSE;
                        }
                            // rename editing complete - here is the string
                        else // arg == 1
                        {
                            // is the renamed value ok?
                            if (!node.OnRename(Marshal.PtrToStringAuto(param)))
                                hr = HRESULT.S_FALSE;
                        }

                        break;
                    }
                    // If an item is selected, we should set flags for verbs available
                    // for the item (from the dropdown menu)
                    case MMCN_Notify.SELECT:
                    {
                        // low word - scope pane or result
                        bool bScope = ((short)arg) != 0;
                        // high word - selected or not
                        bool bSelect = ((((int)arg) >> 16) & 0xffff) != 0;

                        if (bScope)
                        {
                            if (bSelect)
                                node.OnSelectScope();
                            else
                                node.OnDeselectScope();
                        }
                        else
                        {
                            if (bSelect)
                                node.OnSelectResult();
                            else
                                node.OnDeselectResult();
                        }

                        break;
                    }
                    case MMCN_Notify.SHOW:
                    {
                        // TDB: am I sure this property has been set yet?

                        BaseNode theNode = m_Snapin.FindNodeByHScope((int)param);
                        bool bSelecting = (int)arg != 0;
                        // TBD: not sure if i should pass through if we are not selecting
                        if (bSelecting)
                            theNode.OnShow();

						// Added by Alexander Kachalkov
						theNode.OnShow(bSelecting);

                        break;
                    }
                    case MMCN_Notify.VIEW_CHANGE:
                    {
                        bool bScope = (int)arg != 0;
                        
						IDataObject anido = null;
						if (param != (IntPtr)0)
							 anido = (IDataObject)Marshal.GetObjectForIUnknown(param);

                        BaseNode nodeChanged = (anido == null) ? m_Snapin.RootNode : ((DataObject)anido).Node;

                        if (bScope)
                            node.OnViewChangeScope(nodeChanged);
                        else
                            node.OnViewChangeResult(nodeChanged);

                        break;
                    }
                    case MMCN_Notify.SNAPINHELP:
                    {
                        m_Snapin.OnHelp();
                        hr = HRESULT.S_FALSE;
                        break;
                    }
                    case MMCN_Notify.CONTEXTHELP:
                    {
                        node.OnContextHelp();
                        break;
                    }
                    case MMCN_Notify.INITOCX:
                    {
                        // param is the IUnknown of the OCX
                        object objOCX = Marshal.GetObjectForIUnknown(param);
                        if (node.OnInitOcx(objOCX))
                            hr = HRESULT.S_OK;
                        else
                            hr = HRESULT.S_FALSE;
                        break;
                    }
                    case MMCN_Notify.FILTER_CHANGE:
                    {                        
						node.OnFilterChange((MMC_FILTER_CHANGE_CODE)arg.ToInt32(), param.ToInt32());
						// Notify MMC we handled it.
						hr = HRESULT.S_OK;

                        break;
                    }
                    case MMCN_Notify.GET_FILTER_MENU:
                    {
						RECT data = (RECT)(Marshal.PtrToStructure(param, typeof(RECT)));
                        if (!node.OnGetFilterMenu(arg.ToInt32(), data))
                            hr = HRESULT.S_FALSE;

                        break;
                    }
                    case MMCN_Notify.FILTER_OPERATOR:
                    {
                        node.OnFilterOperator();
						hr = HRESULT.S_OK;
						break;
                    }
					//Added by Alexander Kachalkov
					case MMCN_Notify.COLUMNS_CHANGED:        
					{
						// COLUMNDATA:  Returning S_OK makes the console
						hr = HRESULT.S_OK;     // persist the new column configuration.
						break;
					}
                    default:
                    {
                        // We don't support the Notification message we got
                        hr = HRESULT.S_FALSE;
                        break;
                    }
                }
            }
            catch(SnapinException e)
            {
                hr = HRESULT.S_FALSE;
                System.Diagnostics.Debug.WriteLine("IComponent::Notify SnapinException - " + e.ToString());
                // eat the exception and return s_false indicating that 
                // we didn't handle the notification
            }
            catch(Exception e)
            {
                hr = HRESULT.S_FALSE;
                System.Diagnostics.Debug.WriteLine("IComponent::Notify Exception - " + e.ToString());
                //throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
            }
               
            return hr;
        }

        /// <summary>
        /// Called at shutdown
        /// </summary>
        /// <param name="i"></param>
        public void Destroy(int i)
        {
        }
        
        /// <summary>
        /// Called when MMC wants an IDataObject for a node by cookie 
        /// </summary>
        /// <param name="cookie"></param>
        /// <param name="type"></param>
        /// <param name="ppDataObject"></param>
        public void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject)
        {
			ppDataObject = null;
			BaseNode node = null;
			try
			{
				//Added by Alexander Kachalkov
				//Check if this a MULTI_SELECT_COOKIE then just find
				//first selected item and return the node it depends to.
				if (cookie == (int)MMC.MULTI_SELECT_COOKIE)
				{
					RESULTDATAITEM rdi = m_Snapin.GetSelectedItem((IResultData)m_Console);
					node = m_Snapin.FindNode(rdi.lParam);
					ppDataObject = createDataObject(node);
					node.MultiSelectMode = true;
					m_MultiSelectionMode = true;

					System.Diagnostics.Debug.WriteLine(string.Format("IComponent::QueryDataObject: nIndex={0} ", rdi.nIndex.ToString()));
				}
				else
				{
					node = m_Snapin.FindNode(cookie);
					ppDataObject = createDataObject(node);
					node.MultiSelectMode = false;
					m_MultiSelectionMode = false;
				}
			}
			catch(SnapinException e)
			{
				System.Diagnostics.Debug.WriteLine("IComponent::QueryDataObject SnapinException - " + e.ToString());
				// eat the exception and return s_false indicating that 
				// we didn't handle the notification
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("IComponent::QueryDataObject Exception - " + e.ToString());
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
        }
        
        /// <summary>
        /// Called to get Result view type information.
        /// </summary>
        /// <param name="cookie"></param>
        /// <param name="ppViewType"></param>
        /// <param name="pViewOptions"></param>
        /// <returns></returns>
        public int GetResultViewType(int cookie, out IntPtr ppViewType, out int pViewOptions)
        {
		    pViewOptions = 0;

		    BaseNode node = m_Snapin.FindNode(cookie);
		    if (node != null)
		    {
			    string viewType = node.GetResultViewType(ref pViewOptions);
			    ppViewType = Marshal.StringToCoTaskMemUni(viewType);
			    return HRESULT.S_OK;
		    }
		    else
            {
                ppViewType = IntPtr.Zero;
                return HRESULT.S_FALSE;
            }
		    // TBD: When switching from a standard list view to a custom view, the snap-in must call 
		    // IConsole2::SelectScopeItem to reselect the item and force MMC to call GetResultViewType again
        }


        /// <summary>
        /// Called to get result view information.
        /// </summary>
        /// <param name="ResultDataItem"></param>
        public void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem)
        {
            // The low word in the lParam contains the index of the node
            // we're interested in.  If we are in report view we mash in the row
		    // nu,ber is the high word.
            BaseNode NodeWeWant = m_Snapin.FindNode((int)ResultDataItem.lParam);

            // We'll let the node take care of its own Result Data
            NodeWeWant.GetDisplayInfo(ref ResultDataItem);
        }

		/// <summary>
        /// Called to compare 2 nodes - we use cookie values to test equality
        /// </summary>
        /// <param name="lpDataObjectA"></param>
        /// <param name="lpDataObjectB"></param>
        /// <returns></returns>
        public int CompareObjects(IDataObject lpDataObjectA, IDataObject lpDataObjectB)
        {
            DataObject doItem1, doItem2;

            // These data items should be DataObject's in disguise.....
            doItem1 = (DataObject)lpDataObjectA;
            doItem2 = (DataObject)lpDataObjectB;
            
            if (doItem1.Node.Cookie != doItem2.Node.Cookie)
            {
                // These are different objects. We need to return S_FALSE
            return HRESULT.S_FALSE;
            }

            return HRESULT.S_OK;
        }
		#endregion
        //////////////////////////////////////////////////////////////
        //
        // IExtendContextMenu Implementation
        //
		#region
        /// <summary>
        /// This function allows us to add items to the context menus 
        /// </summary>
        /// <param name="piDataObject"></param>
        /// <param name="piCallback"></param>
        /// <param name="pInsertionAllowed"></param>
        public void AddMenuItems(IntPtr piDataObject, IContextMenuCallback piCallback, ref int pInsertionAllowed)
        {
			BaseNode node = null;
			DataObject item = null;
			try
			{
				//Added by Alexander Kachalkov
				if (piDataObject == (IntPtr)DataObject.CUSTOMOCX
					|| piDataObject == (IntPtr)DataObject.CUSTOMWEB)
				{
					// TBD: Add some code that handle this case
					return;
				}

				//Added by Alexander Kachalkov
				IDataObject iDataObject = (IDataObject)Marshal.GetObjectForIUnknown(piDataObject);
				if (MultiSelectionMode)
				{
					RESULTDATAITEM rdi = m_Snapin.GetSelectedItem((IResultData)m_Console);
					node = m_Snapin.FindNode(rdi.lParam);

					item = createDataObject(node);
				}
				else
				{
					// The piDataObject is really a DataObject is disguise....
					item = (DataObject)iDataObject;
				}


				if (item.Node != null)
					item.Node.AddMenuItems(piCallback, ref pInsertionAllowed);
			}
			catch(SnapinException e)
			{
				System.Diagnostics.Debug.WriteLine("Component::AddMenuItems SnapinException - " + e.ToString());
				// eat the exception and return s_false indicating that 
				// we didn't handle the notification
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("Component::AddMenuItems Exception - " + e.ToString());
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
        }

	    virtual protected internal DataObject createDataObject(BaseNode node)
	    {
	        return new DataObject(node);
	    }

	    /// <summary>
        /// Called menu item is invoked.  We delegate to the node 
        /// to let it handle it.
        /// </summary>
        /// <param name="lCommandID"></param>
        /// <param name="piDataObject"></param>
        public void Command(int lCommandID, IntPtr piDataObject)
        {
			//Added by Alexander Kachalkov
			BaseNode node = null;
			DataObject item = null;
			try
			{
				//Added by Alexander Kachalkov
				if (piDataObject == (IntPtr)DataObject.CUSTOMOCX
					|| piDataObject == (IntPtr)DataObject.CUSTOMWEB)
				{
					// TBD: Add some code that handle this case
					return;
				}

				//Added by Alexander Kachalkov
				IDataObject iDataObject = (IDataObject)Marshal.GetObjectForIUnknown(piDataObject);
				if (MultiSelectionMode)
				{
					RESULTDATAITEM rdi = m_Snapin.GetSelectedItem((IResultData)m_Console);
					node = m_Snapin.FindNode(rdi.lParam);

					item = createDataObject(node);
				}
				else
				{
					// The piDataObject is really a DataObject is disguise....
					item = (DataObject)iDataObject;
				}

				// pass the command on to the node
				//DataObject item = (DataObject)piDataObject;
				if (item.Node != null)
					item.Node.Command(lCommandID);
			}
			catch(SnapinException e)
			{
				System.Diagnostics.Debug.WriteLine("Component::AddMenuItems SnapinException - " + e.ToString());
				// eat the exception and return s_false indicating that 
				// we didn't handle the notification
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("Component::AddMenuItems Exception - " + e.ToString());
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
        }
		#endregion
		//////////////////////////////////////////////////////////////
        //
        // IExtendControlbar Implementation
        //
		#region
		/// <summary>
        /// Called to set and reset the control bar interface.
		/// Implemented by Alexander Kachalkov
		/// </summary>
        /// <param name="pControlbar">Can be null to reset</param>
        public void SetControlbar(IControlbar pControlbar)
        {
			if (m_pToolbar != null) m_pToolbar = null;

			if (m_pControlbar != null) m_pControlbar = null;

			if (pControlbar == null) return;

            m_pControlbar = pControlbar;

			object pToolBar;
			m_pControlbar.Create(MMC_CONTROL_TYPE.TOOLBAR, (IExtendControlbar)this, out pToolBar);
			m_pToolbar = (IToolbar)pToolBar;

			// Get an Hicon for myBitmap.
			IntPtr hBitmap = m_Snapin.Images.GetBitmapHandle(m_ToolbarImageIndex);

			if ((hBitmap.ToInt32() == 0) 
				|| (ToolbarButtons == null)) return;

			m_pToolbar.AddBitmap(ToolbarButtons.Length, hBitmap, 16, 16, Color.Black.ToArgb());

			//ToolbarButtons = new MMCBUTTON[11];
			for(int i = 0; i <= ToolbarButtons.Length - 1; i++)
				m_pToolbar.InsertButton(0, ref ToolbarButtons[i]);

			//Did not work, don't know how to make it works
			//m_pToolbar.AddButtons(ToolbarButtons.Length, ref ToolbarButtons);
		}

        /// <summary>
        /// Called when nodes are selected and controlbar commands fire.
        /// </summary>
        /// <param name="aevent"></param>
        /// <param name="arg"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public int ControlbarNotify(uint aevent, IntPtr arg, IntPtr param)
        {
            int hr = HRESULT.S_OK;

            MMCN_Notify notify = (MMCN_Notify)aevent;

            switch(notify)
            {
                case MMCN_Notify.SELECT:
                {
                    // low word - scope pane or result
                    bool bScope = ((short)arg) != 0;
                    // high word - selected or not
                    bool bSelect = ((((int)arg) >> 16) & 0xffff) != 0;

                    // null or < 0 (bad juju) is passed when selecting resultviews
                    if ((int)param > 0)
                    {
                        IDataObject ido = (IDataObject)Marshal.GetObjectForIUnknown(param);
                        BaseNode node = (ido == null) ? m_Snapin.RootNode : ((DataObject)ido).Node;

						//Added by Alexander Kachalkov
						node.OnControlbarNotify(ControlBar, ToolBar, bScope, bSelect);
/*
						if (bScope)
                        {
							if (bSelect)
                                node.OnSelectScope();
                            else
                                node.OnDeselectScope();
                        }
                        else
                        {
                            if (bSelect)
                                node.OnSelectResult();
                            else
                                node.OnDeselectResult();
                        }
						*/
					}
                    break;
                }
				case MMCN_Notify.BTN_CLICK:
				{
					/*
					// low word - scope pane or result
					bool bScope = ((short)arg) != 0;
					// high word - selected or not
					bool bSelect = ((((int)arg) >> 16) & 0xffff) != 0;
					*/
					// null or < 0 (bad juju) is passed when selecting resultviews
					if ((int)arg > 0)
					{
						IDataObject ido = (IDataObject)Marshal.GetObjectForIUnknown(arg);
						BaseNode node = (ido == null) ? m_Snapin.RootNode : ((DataObject)ido).Node;
						node.OnControlbarCommand(ControlBar, ToolBar, (MMC_CONSOLE_VERB)param.ToInt32());
					}
					break;
				}
                default:
                {
                    hr = HRESULT.S_FALSE;
                    break;
                }
            }

            return hr;
        }
		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// IExtendPropertySheet implementation
		//
		#region
		// MAM: made virtual so it can be overriden.
		/// <summary>
		/// MMC wants a property sheet created of the given node 
		/// </summary>
		/// <param name="lpProvider"></param>
		/// <param name="handle"></param>
		/// <param name="lpDataObject"></param>
		/// <returns></returns>
		public virtual int CreatePropertyPages(IPropertySheetCallback lpProvider, IntPtr handle, IDataObject lpDataObject)
		{
			DataObject dataObject = (DataObject)lpDataObject;
	    
			// Let's see if this node has property pages
			if (dataObject.Node.HavePropertyPages)
				dataObject.Node.CreatePropertyPages(lpProvider, handle);

			// don't use the MMC property page facilities - just roll our own.
			return HRESULT.S_FALSE;
		}

		// MAM: made virtual so it can be overriden.
		/// <summary>
		/// MMC calls this at initialization time to find out if the node  
		/// supports property pages - this is the one and only time to respond.
		/// </summary>
		/// <param name="lpDataObject">IDataObject for Node in question</param>
		/// <returns></returns>
		public virtual int QueryPagesFor(IDataObject lpDataObject)
		{
			// This snapin does have property pages, so we should return S_OK
			// (which will happen automatically). If the snapin didn't have
			// any property pages, then we should return S_FALSE

			DataObject dataObject = (DataObject)lpDataObject;
	    
			//---added by Roman Kiss
			dataObject.Node.OnQueryProperties();

			// Let's see if this node has property pages
			if (dataObject.Node.HavePropertyPages)
				return HRESULT.S_OK;

			return HRESULT.S_FALSE;
		}
		#endregion

		//////////////////////////////////////////////////////////////////////
		//
		// IResultDataCompareEx implementation
		//
		#region
		public void Compare(ref RDCOMPARE prdc, out int pnResult)
		{
			int iResult = 0;

			try
			{
				if (m_Snapin.CurrentScopeNode != null)
				{
					BaseNode node = (m_Snapin.CurrentScopeNode as BaseNode);
					if (node != null)
					{
						RDITEMHDR item1 = (RDITEMHDR)Marshal.PtrToStructure(prdc.prdch1, typeof(RDITEMHDR));
						RDITEMHDR item2 = (RDITEMHDR)Marshal.PtrToStructure(prdc.prdch2, typeof(RDITEMHDR));

						IComparable comparer1 = node.GetSortCompareInfo(prdc.nColumn, item1.cookie.ToInt32());
						IComparable comparer2 = node.GetSortCompareInfo(prdc.nColumn, item2.cookie.ToInt32());

						if ((comparer1 != null) && (comparer2 != null))
						{
							iResult = comparer1.CompareTo(comparer2);
						}
					}
				}
			}
			catch(Exception ex)
			{
				string method = this.GetType().FullName + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "(): ";
				System.Diagnostics.Trace.WriteLine(method, "EXCEPTION");
				throw ex;
			}

			pnResult = iResult;
		}
		#endregion
	}
}
