using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using System.Diagnostics;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Resources;
using System.Reflection;

namespace Ironring.Management.MMC
{
	//Added by Alexander Kachalkov
	/// <summary>
	/// For Invoke method when marshaling from PropertySheet thread
	/// </summary>
	public delegate void OnUserDelegate(string key, object val);

	#region PropertySheet
	/// <summary>
	/// This implements the equivalent of a Win32 PropertySheet 
	/// </summary>
	public class PropertySheet : Form
	{
		// Controls on the page        

		// Lesley v. Zijl: made protected for subclasses.
		/// <summary>
		/// The OK button
		/// </summary>
		protected Button m_btnOk;        

		// Lesley v. Zijl: made protected for subclasses.
		/// <summary>
		/// The Cancel button
		/// </summary>
		protected Button m_btnCancel;    

		// Lesley v. Zijl: made protected for subclasses.
		/// <summary>
		/// Apply button
		/// </summary>
		protected Button m_btnApply;     

		/// <summary>
		/// The tab control
		/// </summary>
		PropertySheetTabControl m_tc;           

		/// <summary>
		/// Typed collection of PropertyPages to display
		/// </summary>
		PropertyPageList m_alPages = new PropertyPageList();

		/// <summary>
		/// closed flag used to hide and not dispose of the form
		/// </summary>
		bool m_fClosed;        

		/// <summary>
		/// The node that we represent properties for
		/// </summary>
		BaseNode m_ParentNode; 

		//Added by Alexander Ovchinnikov for close PS after remove node
		public OnUserDelegate m_CloseHandle = null;

		/// <summary>
		/// Construct provides linkage to the snapin node 
		/// </summary>
		/// <param name="node"></param>
		public PropertySheet(BaseNode node) {
			m_ParentNode = node;

			// Make sure nobody can resize this form
			FormBorderStyle = FormBorderStyle.FixedDialog;
			
			Icon = null;
			ShowInTaskbar = false;
			MaximizeBox = false;
			MinimizeBox = false;
			
			m_fClosed = false;

			//Added by Alexander Kachalkov
			//This call is required by the Windows Form Designer.
			InitializeComponent();

			//Added by Alexander Ovchinnikov for close PS after remove node
			//initialise closeDelegate
			if( m_CloseHandle == null ){
				m_CloseHandle += new OnUserDelegate( ClosePropertySheet );
			}
		}

		/// <summary>
		/// The Typed pages collection we display
		/// </summary>
		public PropertyPageList Pages
		{
			get{ return m_alPages; }
		}

		/// <summary>
		/// The Tab control
		/// </summary>
		public PropertySheetTabControl Tabs
		{
			get{ return m_tc; }
		}

		/// <summary>
		/// the node this propertysheet represents
		/// </summary>
		public BaseNode Node
		{
			get { return m_ParentNode; }
		}
		 
		/// <summary>
		/// Is the fake closed flag enabled on?  i.e. is the form hidden?
		/// </summary>
		public bool isClosed
		{
			get { return m_fClosed; }
		}

		/// <summary>
		/// Added by Lesley van Zijl
		/// Ask for the state of the ApplyButton
		/// </summary>
		/// <returns></returns>
		public bool IsApplyEnabled()
		{
			return m_btnApply.Enabled;
		}

		/// <summary>
		/// Prepares the property sheet to be shown and then displays it
		/// </summary>
		public new DialogResult ShowDialog()
		{
			Controls.Clear();

			// Figure out how big to make the tab control 
			// by sizing to the largest UserControl

			int nWidth = 240; 
			int nHeight = 0;

			// We'll be as wide as the widest and tall as the tallest page
			foreach (PropertyPage page in m_alPages)
			{
				if (page.Size.Width > nWidth)
					nWidth = page.Size.Width;
				if (page.Size.Height> nHeight)
					nHeight = page.Size.Height;
			}

			// Tab Control should be on the top...
			m_tc = new PropertySheetTabControl();

			// Add the pages to the sheet
			foreach (PropertyPage page in m_alPages)
			{
				// Initialize the Property Pages
				page.PropertySheet = this;
				// Create a Tab page that we can use
				m_tc.TabPages.Add(page.CreateTabPage());
			}

			// Add the height of the tab thingy and the indentation
			m_tc.Location = new Point(4, 5);
			m_tc.SelectedIndexChanging += new CheckValidation(onTabPageChanged);

			m_tc.TabIndex = 0;
			Controls.Add(m_tc);

			// Adjust the height of of the tab control to account for the
			// margin and the tabs up top.

			// We need to do this so the Tab Control gets created on the Win32 side of
			// things, which makes m_tc.ItemSize.Height valid.
			IntPtr handle = m_tc.Handle;

			nHeight += m_tc.Location.Y + m_tc.ItemSize.Height;

			// Added by Pietro Di Gregorio : Calculation of the size to allow a multiline display of the tabs
			m_tc.Size = new System.Drawing.Size(nWidth, nHeight);
			int tp_Height_BeforeAdj = m_tc.TabPages[0].Size.Height;
			m_tc.Multiline=true;
			m_tc.SizeMode = TabSizeMode.FillToRight;
			int tp_Height_AfterAdj = m_tc.TabPages[0].Size.Height;
			nHeight += tp_Height_BeforeAdj-tp_Height_AfterAdj;

			m_tc.Size = new System.Drawing.Size(nWidth, nHeight);

			// Now put the Ok, Cancel, and Apply buttons on the page.
            ResourceManager resourceManager = new ResourceManager("Ironring.Management.MMC.MMCLibResource", System.Reflection.Assembly.GetExecutingAssembly());

            // We'll have a 15 pixel buffer between the tab control and the buttons
			nHeight += 15;

			m_btnApply = new Button();
			m_btnApply.Size = new System.Drawing.Size(75, 23);
			m_btnApply.Location = new Point(nWidth-m_btnApply.Width+4, nHeight);
			m_btnApply.Text = resourceManager.GetString("Apply");
			m_btnApply.Enabled = false;
			m_btnApply.TabIndex = 3;
			m_btnApply.Click += new EventHandler(onApplyClick);
			//Added by Alexander Kachalkov
			m_btnApply.FlatStyle = FlatStyle.System;
			Controls.Add(m_btnApply);

			m_btnCancel = new Button();
			m_btnCancel.Size = new System.Drawing.Size(75, 23);
			m_btnCancel.Location = new Point(m_btnApply.Location.X - m_btnCancel.Width-5, nHeight);
			m_btnCancel.Text = resourceManager.GetString("Cancel");
			m_btnCancel.Click += new EventHandler(onCancelClick);
			m_btnCancel.TabIndex = 2;
			//Added by Alexander Kachalkov
			m_btnCancel.FlatStyle = FlatStyle.System;
			Controls.Add(m_btnCancel);

			m_btnOk = new Button();
			m_btnOk.Size = new System.Drawing.Size(75, 23);
			m_btnOk.Location = new Point(m_btnCancel.Location.X - m_btnOk.Width - 5, nHeight);
			m_btnOk.Text = resourceManager.GetString("OK");
			m_btnOk.TabIndex = 1;
			m_btnOk.Click += new EventHandler(onOkClick);
			//Added by Alexander Kachalkov
			m_btnOk.FlatStyle = FlatStyle.System;
			Controls.Add(m_btnOk);


			// Make sure we account for the height of the button
			nHeight +=m_btnOk.Height;
            
			// We have a 4 pixel margin on each side of the tab control, and a 5 pixel margin
			// between the buttons and the bottom of the page
			ClientSize = new System.Drawing.Size(nWidth + 8, nHeight + 5);
			this.AcceptButton = m_btnOk;
			this.CancelButton = m_btnCancel;
            
			//Added by Alexander Kachalkov
			base.Closing += new CancelEventHandler(onCloseClick);
			// And finally, make us actually show
			return base.ShowDialog();
		}
	
		//Added by Attila Kapostyak
		/// <summary>
		/// Event handler that pops up when the user tries to  
		/// change property pages. Make sure that the data on the
		/// page they're leaving is correct.
		/// </summary>
		/// <returns></returns>
		bool onTabPageChanged()
		{   
			bool isValidData;

			PropertyPage page = m_alPages[m_tc.SelectedIndex];
			isValidData = page.ValidateData();
			if (isValidData)
			{
				foreach(PropertyPage pages in m_alPages)
					pages.TabChange();
			}
			return isValidData;
		}

		//Added by Alexander Kachalkov
		//to be able to show form again after closing it
		void onCloseClick(Object o, CancelEventArgs e)
		{
			Node.EditMode = false;
			e.Cancel = false;
			FakeClose();
		}

		/// <summary>
		/// The user clicked ok. Treat it as an Apply, and then close the page
		/// </summary>
		/// <param name="o"></param>
		/// <param name="e"></param>
		void onOkClick(Object o, EventArgs e)
		{
			PropertyPage page = m_alPages[m_tc.SelectedIndex];
            
			if (page.ValidateData())
			{
				//Added by Alexander Kachalkov
				base.DialogResult = DialogResult.OK;

				onApplyClick(o, e);
				FakeClose();
			}
		}

		/// <summary>
		/// The user clicked cancel. 
		/// </summary>
		/// <param name="o"></param>
		/// <param name="e"></param>
		void onCancelClick(Object o, EventArgs e)
		{
			if(IsApplyEnabled() == true)
			{
                ResourceManager resourceManager = new ResourceManager("Ironring.Management.MMC.MMCLibResource", System.Reflection.Assembly.GetExecutingAssembly());
                
				switch(MessageBox.Show(resourceManager.GetString("DataHasChanged"), 
					resourceManager.GetString("SaveChanges"),
					MessageBoxButtons.YesNoCancel))
				{
					case DialogResult.Yes:
						onApplyClick(o, e);
						break;
					case DialogResult.No:
						base.DialogResult = DialogResult.Cancel;
						
						foreach(PropertyPage page in m_alPages)
							page.onCancel();
						FakeClose();
						break;
					case DialogResult.Cancel:

						base.DialogResult = DialogResult.None;
						break;
					default:
						base.DialogResult = DialogResult.Cancel;
						
						foreach(PropertyPage page in m_alPages)
							page.onCancel();
						FakeClose();
						break;
				}
			}
			else
			{
				base.DialogResult = DialogResult.Cancel;

				foreach(PropertyPage page in m_alPages)
					page.onCancel();
				FakeClose();
			}

		}


		/// <summary>
		/// The user clicked apply. Validate the current page's 
		/// data, and then send an apply message to every page
		/// </summary>
		/// <param name="o"></param>
		/// <param name="e"></param>
		void onApplyClick(Object o, EventArgs e)
		{
			PropertyPage psp = m_alPages[m_tc.SelectedIndex];
            
			if (psp.ValidateData())
			{
				SendApplyToAll();
				m_btnApply.Enabled = false;
			}
		}

		/// <summary>
		/// Turn on the apply button 
		/// </summary>
		public void ActivateApplyButton()
		{   
			if (m_btnApply != null)
				m_btnApply.Enabled = true;
		}

		/// <summary>
		/// Turn off the apply button 
		/// </summary>
		public void DeactivateApplyButton()
		{   
			if (m_btnApply != null)
				m_btnApply.Enabled = false;
		}


		/// <summary>
		/// This sends the apply message to every 
		/// </summary>
		private void SendApplyToAll()
		{
			foreach(PropertyPage page in m_alPages)
				page.onApply();
		}

		//Added by Alexander Kachalkov
		private void InitializeComponent()
		{
			// 
			// PropertySheet
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Name = "PropertySheet";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			//this.TopMost = true;
		}


		/// <summary>
		/// We need to close this page (make it invisible), but
		/// we don't want it to dispose itself. Instead, we'll
		/// just muck with the form so the next time it's made visible,
		/// it looks brand new
		/// </summary>
		private void FakeClose()
		{
			m_fClosed = true;
			Close();
		}

		//Added by Alexander Ovchinnikov for Close PropertySheet
		public virtual void ClosePropertySheet( string key, object val ){
			base.DialogResult = DialogResult.Cancel;

			foreach(PropertyPage page in m_alPages)
				page.onCancel();
			FakeClose();	
		}
		
	}

	#endregion

	#region PropertyPage
	/// <summary>
	/// This implements the equivalent of a Win32 PropertyPage 
	/// </summary>
	public class PropertyPage
	{
		/// <summary>
		/// The property sheet we belong to
		/// </summary>
		protected PropertySheet m_PropSheet;    

		/// <summary>
		/// The title of our page - string on the tab
		/// </summary>
		protected String m_sTitle;       
	
		/// <summary>
		/// the type of control to activate int he page client area
		/// </summary>
		protected Type m_ControlType;     

		/// <summary>
		/// the instance of the control once activated 
		/// </summary>
		protected UserControl m_Control = null;        


		/// <summary>
		/// Event fires when page must validate page contents
		/// </summary>
		public event EventHandler validate;

		//Added by Attila Kapostyak
		/// <summary>
		/// Event fires when selected tab is changed
		/// </summary>
		public event EventHandler tabchanged;

		/// <summary>
		/// Fires when page must apply page contents
		/// </summary>
		public event EventHandler apply;

	
		/// <summary>
		/// Fires when user cancels the propsheet
		/// </summary>
		public event EventHandler cancel;

		/// <summary>
		/// constructor provides context
		/// </summary>
		/// <param name="title"></param>
		public PropertyPage(string title)
		{
			m_sTitle = title;
			//m_Node = node;
		}

		/// <summary>
		/// Set the type of User Control to serve as the property page guts
		/// </summary>
		public Type ControlType
		{
			get { return m_ControlType; } 
			set { m_ControlType = value; }
		}


		/// <summary>
		/// Sets the PropertySheet we're the property page of
		/// </summary>
		public PropertySheet PropertySheet
		{
			set	{ m_PropSheet = value; }
			get { return m_PropSheet; }
		}


		protected UserControl MainControl 
		{
			get 
			{ 
				try
				{
					if (m_Control == null)
					{
						m_Control = (UserControl)Activator.CreateInstance(ControlType);

						// check if the usercontrol implements IPropertyPageContext
						// if so provide a back reference to ourselves.
						IPropertyPageContext ctx = m_Control as IPropertyPageContext;
						if(ctx != null)
							ctx.PropertyPage = this;
					}
				}
				catch(Exception e)
				{
					System.Diagnostics.Debug.WriteLine("PropertyPage Exception - " + e.ToString());
					throw new SnapinException ("Failed to create user control", e);
				}
				return m_Control;
			}
		}


		/// <summary>
		/// Generate the actual TabPage that will  used in the PropertySheet.
		/// We don't keep re-using the same TabPage each time
		/// the property page is created because it makes the 
		/// updating of the property pages a little difficult.
		/// Creating a new one each time makes sure we have fresh
		/// data in the property page.
		/// </summary>
		/// <returns>the actual window used as the propertypage</returns>
		public TabPage CreateTabPage()
		{   
			TabPage tp = new TabPage();

			UserControl ctrl = MainControl;

			ctrl.Location = new Point(0, 0);
			ctrl.Size = Size;
			ctrl.TabIndex = 0;

			// Parent this control to our user control
			tp.Controls.Add(ctrl);

			tp.Size = Size;
			tp.Text = Title;
		
			return tp;
		}



		/// <summary>
		/// The string displayed int he tab
		/// </summary>
		public String Title
		{
			get { return m_sTitle; }
			set { m_sTitle = value; }
		}
      

		/// <summary>
		/// size of the property sheet page
		/// </summary>
		public Size Size
		{
			get { return MainControl.Size; }
			set { MainControl.Size = value; }
		}


		/// <summary>
		/// Method that gets called to validate the values in the TabPage 
		/// </summary>
		/// <returns></returns>
		public virtual bool ValidateData()
		{   
			if (validate != null)
			{
				ValidateEventArgs args = new ValidateEventArgs();
				validate(this, args);
				
				// if no one set the invalid page all is good
				return args.Valid;
			}

			return true;
		}


		//Added by Attila Kapostyak
		/// <summary>
		/// Method that gets called to validate the values in the TabPage 
		/// </summary>
		/// <returns></returns>
		public virtual void TabChange()
		{   
			if (tabchanged != null)
			{
				EventArgs args = new EventArgs();
				tabchanged(this, args);		
			}
		}

		/// <summary>
		/// Method that gets called to apply the values in the TabPage 
		/// </summary>
		public virtual void onApply()
		{
			if (apply != null)
				apply(this, null);
		}

		/// <summary>
		/// Method that gets called when cancel is clicked
		/// </summary>
		public virtual void onCancel()
		{
			if (cancel != null)
				cancel(this, null);
		}

		/// <summary>
		/// Method to call that turns on the apply button 
		/// </summary>
		protected void ActivateApply()
		{
			m_PropSheet.ActivateApplyButton();
		}

	}

	#endregion

	#region Validate
	/// <summary>
	/// Validate Event Args
	/// </summary>
	public class ValidateEventArgs : System.EventArgs
	{
		protected bool valid = true;

		public bool Valid
		{
			get { return valid; }
			set { valid = value; }
		}
	}


	/// <summary>
	/// Validation Delegate
	/// </summary>
	public delegate bool CheckValidation(); 
	#endregion

	#region PropertySheetTabControl
	/// <summary>
	/// Custom TabControl to simulate a property sheet tabs
	/// </summary>
	public class PropertySheetTabControl : TabControl
	{
		private CheckValidation onSelectedIndexChanging;

		public event CheckValidation SelectedIndexChanging 
		{
			add 
			{
				onSelectedIndexChanging += value;
			}
			remove 
			{
				onSelectedIndexChanging -= value;
			}
		}

		protected override void WndProc(ref Message m)
		{
			try
			{
				// See if this is a special Winforms notify message
				if (m.Msg == ((int)WM.NOTIFY + 0x0400 + 0x1C00)) // WM_USER + WM_REFLECT
				{
					// lParam really is a pointer to a NMHDR structure....
					NMHDR nm = new NMHDR();
					nm = (NMHDR)Marshal.PtrToStructure(m.LParam, nm.GetType());

					// See if this is our command
					if (nm.code == unchecked((0U-550U)-2))  // TCN_SELCHANGING  = TCN_FIRST - 2
					{
						if (onSelectedIndexChanging != null) 
							if (!onSelectedIndexChanging())       
							{
								m.Result = (IntPtr)1;
								return;
							}
					}
				}
				base.WndProc(ref m);
			}
			catch (Exception e)
			{
				System.Diagnostics.Debug.WriteLine("Error in winProc: " + e.Message);
			}
		}
	}

	#endregion

	#region PropertyPageList
	/// <summary>
	/// Typed collection of PropertyPage objects to help out PropertySheet
	/// </summary>
	public class PropertyPageList : ArrayList
	{
		public new PropertyPage this[int index]
		{
			get
			{
				return (PropertyPage)(base[index]);
			}
		}

		// Added by Alex Ovchinnikov for Dispose method in Propertysheet
			
		public override void Clear( ){
			for( int iIndex = 0; iIndex < base.Count; iIndex++ ){
				base[ iIndex ] = null;
			}
			base.Clear( );
		}
	}

	#endregion

}
