using System;
using System.Data;
using System.Runtime.InteropServices;
using System.Collections;
using System.Text.RegularExpressions;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	// TBD: overridable data binding methods to augment GetDisplayInfo
	/// <summary>
	/// Configures the resultview as a Listview control. 
	/// Supports binding to a DataTable to provide the schema and data
	/// </summary>
	public class ReportNode : BaseNode {

		//Added by Alexander Kachalkov
		/// <summary>
		/// A cached reference to the MMC console 
		/// </summary>
		protected IConsole2 m_Console = null;          

		//Added by Alexander Kachalkov
		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IHeaderCtrl2 HeaderCtrl {
			get { return (IHeaderCtrl2)m_Console; }
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IResultData ResultData {
			get { return (IResultData)m_Console; }
		}

		/// <summary>
		/// Underlying template for the listview control
		/// </summary>
		protected DataTable m_table = null;

		// MAM: the filtersets that are active.
		/// <summary>
		/// The currently active filters. The keys are the columns (integer), the values
		/// are filter expressions. The expressions are simply matched
		/// by testing if they occur anywhere within the value.
		/// </summary>
		protected Hashtable m_filters = null;

		//Added by Arunjeet Singh
		/// <summary>
		/// Variable to hold the AutoRefresh property
		/// </summary>
		protected bool m_AutoRefresh = false;

		/// <summary>
		/// ctor takes the snapin to register with
		/// </summary>
		/// <param name="snapin"></param>
		public ReportNode(SnapinBase snapin) : base(snapin) {
		}

		/// <summary>
		/// ctor takes the snapin to register with
		/// </summary>
		/// <param name="snapin"></param>
		/// <param name="parentNode"></param>
		public ReportNode(SnapinBase snapin, BaseNode parentNode) : base(snapin, parentNode) {
		}
		
		/// <summary>
		/// Access the underlying DataTable 
		/// </summary>
		public DataTable Table {
			get { return m_table; }
			set 
			{
				m_table = value; 
				if(AutoRefresh)
				{
					Snapin.SelectScopeNode(this);
				}
			}
		}


		//Added by Arunjeet Singh
		/// <summary>
		/// Property that controls whether or not the resultview is refreshed when the
		/// Table property changes.
		/// </summary>
		public bool AutoRefresh
		{
			get
			{
				return m_AutoRefresh;
			}
			set
			{
				m_AutoRefresh = value;
			}
		}
		/// <summary>
		/// BaseNode override to populate the treeview control
		/// with column information and data
		/// </summary>
		public override void OnShow() {
			m_Console = m_Snapin.ResultViewConsole;

			// setup the header for the columns we want to show
			//Changed by Alexander Kachalkov
			//IHeaderCtrl2 header = console as IHeaderCtrl2;
			//OnShowHeader(header);
			OnShowHeader(HeaderCtrl);
			
			// add the actual data items 
			//Changed by Alexander Kachalkov
			//IResultData rdata = console as IResultData;
			//OnShowData(rdata);
			OnShowData(ResultData);

			// MAM: make it possible to override the (default) viewtype per scope-node.
			ResultData.SetViewMode(this.GetViewType());

			base.OnShow();
		}

		// MAM: from MMCLibExt: make the viewmode for a report-node overrideable.
		// Defaults to Report-mode (the default for MMCLib).
		/// <summary>
		/// Overrideable method to get the actual type of view for this report-node.
		/// Return one of the ViewMode enum. values.
		/// </summary>
		/// <returns></returns>
		public virtual int GetViewType() {
			return (int)ViewMode.Report;
		}

		////////////////////////////////////////////////////////////////////
		// 
		// Helper methods
		// 
		#region
		/// <summary>
		/// Overridable Helper method to populate the column header
		/// </summary>
		/// <param name="header"></param>
		public virtual void OnShowHeader(IHeaderCtrl2 header) {
			try {
				if (m_table != null)
					// Let's put in the column titles
					foreach (DataColumn col in m_table.Columns) {
						string name = col.Caption;
						int ord = col.Ordinal;
						//int nWidth = col.MaxLength;
						int nWidth = (int)ColumnHeaderWidth.AUTO_WIDTH;

						foreach(DataColumn prkey in m_table.PrimaryKey)
							if (col == prkey)
								nWidth = (int)ColumnHeaderWidth.HIDE_COLUMN;

						header.InsertColumn(ord, name, (int)ColumnHeaderFormat.LEFT, nWidth);
					}
			}
			catch(COMException e) {
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e) {
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}
		
		/// <summary>
		/// Overridable helper method to populate the column data
		/// </summary>
		/// <param name="ResultData"></param>
		public virtual void OnShowData(IResultData ResultData) {
			if (m_table != null) {
				int nRow = 1;
				//Added by Alexander Kachalkov
				Table.AcceptChanges();

				foreach (DataRow row in m_table.Rows)
				{
					// MAM: check if the item should be added based on the filtering.
					if(CheckItem(row)) 
					{
						InsertItem(row, nRow);
					}
					nRow++;
				}
			}
		}

		/// <summary>
		/// Overridable helper method to InsertItem
		/// </summary>
		/// <param name="row"></param>
		/// <param name="nRow"></param>
		public virtual void InsertItem(DataRow row, int nRow) {
			if (row != null) {
				RESULTDATAITEM rdi = new RESULTDATAITEM();

				row.ToString();
				rdi.mask	= (uint)RDI.STR | (uint)RDI.IMAGE | (uint)RDI.PARAM;     

				// TBD: what image?
				rdi.nImage  = -1;   
				rdi.str     = (IntPtr)(-1);  // callback for names
				rdi.nCol    = 0;
				rdi.lParam	= GetItemLParam(nRow);

				ResultData.InsertItem(ref rdi);
			}
		}
		
		/// <summary>
		/// Overridable helper method to SetItem
		/// </summary>
		/// <param name="row"></param>
		/// <param name="nRow"></param>
		public virtual void SetItem(DataRow row, int nRow) {
			if (row != null) {
				int lParam = GetItemLParam(nRow);
				uint pItemID;

				RESULTDATAITEM rdi = new RESULTDATAITEM();

				row.ToString();
				rdi.mask	= (uint)RDI.STR | (uint)RDI.PARAM;

				rdi.str     = (IntPtr)(-1);  // callback for names
				rdi.nCol    = 0;

				//Find ItemID
				ResultData.FindItemByLParam(lParam, out pItemID);
				rdi.lParam = lParam;
				rdi.itemID = (int)pItemID;

				ResultData.SetItem(ref rdi);
			}
		}

		/// <summary>
		/// Overridable helper method to UpdateItem
		/// </summary>
		/// <param name="lParam"></param>
		public virtual void UpdateItem(int lParam) {
			uint pItemID;
			//Find ItemID
			ResultData.FindItemByLParam(lParam, out pItemID);
			ResultData.UpdateItem(pItemID);
		}

		/// <summary>
		/// Overridable helper method to DeleteItem
		/// </summary>
		/// <param name="lParam"></param>
		public virtual void DeleteItem(int lParam) {
			uint pItemID;
			//Find ItemID
			ResultData.FindItemByLParam(lParam, out pItemID);
			ResultData.DeleteItem(pItemID, 0);
		}

		/// <summary>
		/// Overridable helper method to GetItem
		/// </summary>
		/// <param name="ResultDataItem"></param>
		/// <param name="pItemID"></param>
		public virtual void GetItem(out RESULTDATAITEM ResultDataItem, uint pItemID) {
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= (uint)RDI.STR;							// STR is valid
			rdi.itemID	= (int)pItemID;
			rdi.nCol	= 0;

			ResultData.GetItem(ref rdi);
			
			ResultDataItem = rdi;
		}
		
		
		/// <summary>
		/// Overridable helper method to ClearSelection
		/// </summary>
		public virtual void ClearSelection() 
		{
			RESULTDATAITEM rdi = GetSelectedItem();

			while (rdi.nIndex!=-1)
			{
				ResultData.ModifyItemState(rdi.nIndex, (uint)rdi.itemID, 0, (uint)LVIS.SELECTED | (uint)LVIS.FOCUSED);
				rdi = GetSelectedItem();
			}
		}

		/// <summary>
		/// Overridable helper method to SelectItem
		/// </summary>
		/// <param name="nRow"></param>
		public virtual void SelectItem(int nRow) 
		{
			int lParam = GetItemLParam(nRow);
			uint pItemID;

			//Find ItemID
			ResultData.FindItemByLParam(lParam, out pItemID);

			//ResultData.SetItem(ref rdi);
			ResultData.ModifyItemState(0, pItemID, (uint)LVIS.SELECTED | (uint)LVIS.FOCUSED, 0);
		}

		/// <summary>
		/// Overridable helper method to Set
		/// </summary>
		/// <param name="nRow"></param>
		public virtual void SetFocus(int nRow) 
		{
			int lParam = GetItemLParam(nRow);
			uint pItemID;

			//Find ItemID
			ResultData.FindItemByLParam(lParam, out pItemID);

			RESULTDATAITEM rdi = new RESULTDATAITEM();
			rdi.itemID	= (int)pItemID;
			rdi.mask	= (uint)RDI.STATE;
			rdi.nState	= (uint)LVIS.SELECTED;  // select the item
			rdi.nCol    = 0;

			ResultData.SetItem(ref rdi);
		}

		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		public virtual RESULTDATAITEM GetSelectedItem() 
		{
			return GetSelectedItem(-1);	// nIndex == -1 to start at first item
		}

		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		/// <param name="nIndex"></param>
		public virtual RESULTDATAITEM GetSelectedItem(int nIndex) {
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= (uint)RDI.STATE;		// nState is valid 
			rdi.nCol	= 0;
			rdi.nIndex  = nIndex;				// nIndex == -1 to start at first item
			rdi.nState  = (uint)LVIS.SELECTED;	// only interested in selected items

			ResultData.GetNextItem(ref rdi);
			//out
			return rdi;
		}
		
		/// <summary>
		/// helper method to get item row ID by lParam
		/// </summary>
		/// <param name="lParam"></param>
		public virtual int GetTableRowID(int lParam) {
			return (lParam >> 16) - 1;
		}
		
		/// <summary>
		/// helper method to get item lParam by nRow
		/// The low word contains the cookie for the node, while the high word 
		/// contains the row number + 1 we're inserting
		/// </summary>
		/// <param name="nRow"></param>
		public virtual int GetItemLParam(int nRow) {
			return m_iCookie | (nRow << 16);
		}
		#endregion
		
		/// <summary>
		/// BaseNode callback override responsible for provided data.  Lookup the 
		/// row info in the lparam of the resultdataitem since its packed in with  
		/// our own cookie
		/// </summary>
		/// <param name="rdi"></param>
		public override void GetDisplayInfo(ref RESULTDATAITEM rdi) {
			bool bCallbase = true;
			int nRow = 0;

			// the "cell" data
			if (Table != null) {
				//int nRow = (ResultDataItem.lParam >> 16) - 1;
				nRow = GetTableRowID(rdi.lParam);
				int nCol = rdi.nCol;

				if ((rdi.mask & (uint)RDI.STR) > 0) {
					string data = DisplayName;
					if (nRow >= 0 && nRow < Table.Rows.Count && nCol >= 0 && nCol < Table.Columns.Count) {
						data = Table.Rows[nRow].ItemArray[nCol].ToString();
						bCallbase = false;

						//System.Diagnostics.Trace.WriteLine(string.Format("GetDisplayInfo: data={0}, row={1}", data, nRow));
					}

					rdi.str = Marshal.StringToCoTaskMemUni(data);
				}
			}

			// the image - requires 2 images in the image list
			// small - 0
			// large - 1
			//Changed by Alexander Kachalkov
			// small - SmallImageIndex
			// large - LargeImageIndex
			if ((rdi.mask & (uint)RDI.IMAGE) > 0) {
/*				//int offset = 1;
				int offset = LargeImageIndex;
				if (IsUseSmallIcons())
					//offset = 0;
					offset = SmallImageIndex;

				rdi.nImage = (Cookie << 16) + offset;*/

				rdi.nImage = GetResultViewImageIndex(rdi);
			}

			if (bCallbase)
				base.GetDisplayInfo(ref rdi);
		}

		//////////////////////////////////////////////////////////////
		//
		// Filtering implementation
		//

		#region Filtering

		// MAM: override the filter-event and do something (only called when the
		// view-mode is also filtered).
		/// <summary>
		/// When we receive a filter event, we construct a full set of filters per column.
		/// </summary>
		/// <param name="code">The filter-change operator.</param>
		/// <param name="col">The column for which the change is.</param>		
		public override void OnFilterChange(MMC_FILTER_CHANGE_CODE code, int col) 
		{		
			switch(code) 
			{
				case MMC_FILTER_CHANGE_CODE.MFCC_DISABLE:
					this.m_filters = null;
					break;
				
				case MMC_FILTER_CHANGE_CODE.MFCC_ENABLE:
					//ignore
					break;

				case MMC_FILTER_CHANGE_CODE.MFCC_VALUE_CHANGE:
					try 
					{
						int bufLen = 1000;
						string strData = GetColumnFilter(col, bufLen);
											
						// Get the column object and set the string
						if(this.m_filters == null) 
						{
							this.m_filters = new Hashtable();
						}
						
						// Huh? We get a string back: instead of a null-ptr or zero-terminated string
						// we get some strange characters. Assume that if we get a really large string an
						// empty string is meant. Probably some funky interop is needed to get this
						// correct in microsoft parlance.
						// TODO: get this verified/looked at by some COM guru.
						if (!Regex.IsMatch(strData, @"\w") || strData.Length >= (bufLen / 2)) 
						{
							this.m_filters.Remove(col);
						} 
						else 
						{
							this.m_filters[col] = strData;
						}

						// Performance: unset the filter object when no filters are active.
						if(this.m_filters.Count == 0) 
						{
							this.m_filters = null;
						}

						// Do a forced refresh of all items.
						RemoveAllItems();
					}
					catch(Exception e) 
					{
						System.Diagnostics.Debug.WriteLine(e.Message);
						//throw e;
						ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
					}
					break;
			}
			base.OnFilterChange(code, col);							
		}

		// MAM: this is taken from the MMClibExt2 and changed a bit to
		// be a little brute about which item is removed.
		/// <summary>
		///  Remove all items from the view and forces a refresh.
		/// </summary>		
		public virtual void RemoveAllItems() {
			try {				
				IConsole2 console = base.Snapin.ResultViewConsole;
				if(console != null) {
					// add the actual data items 
					IResultData rdata = console as IResultData;
					if(rdata != null) {
						rdata.DeleteAllRsltItems();
						this.OnShowData(rdata);
					} else {
						System.Diagnostics.Debug.WriteLine("Rdata cannot be cast");
					}
				} else {
					System.Diagnostics.Debug.WriteLine("Cannot find console");
				}
			}
			catch(Exception e) {
				System.Diagnostics.Debug.WriteLine("Aaargh: " + e);
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}

		// MAM: filtering code. Generic checker.
		/// <summary>
		/// Overrideable checker method to see if this row is acceptable.		
		/// </summary>
		/// <param name="row"></param>
		/// <returns></returns>
		public virtual bool CheckItem(DataRow row) {
			// Performance: No filters: return early.
			if(m_filters == null) {
				return true;
			}
			
			object[] data = row.ItemArray;
			for(int i=0; i<data.Length; i++) {
				string filterValue = m_filters[i] as string;								
				if(filterValue != null && data[i] != null) {
					if(data[i].ToString().IndexOf(filterValue) == -1) {
						// There is a filter and there is a value but the value does
						// not contain the filter.
						return false;
					}
				}				
			}

			return true;
		}


		//Added by Alexande Kachalkov
		/// <summary>
		/// Overridable helper method to GetColumnFilter
		/// </summary>
		/// <param name="col">Filter column</param>
		/// <param name="bufLen">Buffer length</param>
		/// <returns>string</returns>
		public virtual string GetColumnFilter(int col, int bufLen) 
		{
			int retval = 0;
			MMC_FILTERDATA data = new MMC_FILTERDATA();

			try 
			{
				uint type = 0;
				// Some arbitrary buffer length.
				data.pszText = IntPtr.Zero;

				// COM magic.
				data.pszText = Marshal.AllocCoTaskMem(bufLen);
				data.cchTextMax = bufLen;
				
				retval = HeaderCtrl.GetColumnFilter((uint)col, ref type, ref data);
				string strData = Marshal.PtrToStringAuto(data.pszText); // the size doesn't change?, data.cchTextMax);
				System.Diagnostics.Debug.WriteLine("Got: " + strData + " for column: " + col + " Length of string: " + strData.Length);

				return strData;
			}
			catch(Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			finally 
			{						
				if(! data.pszText.Equals(IntPtr.Zero)) 
				{
					Marshal.FreeCoTaskMem(data.pszText);
				}
			}
			return null;
		}

		//Added by Alexande Kachalkov
		/// <summary>
		/// Overridable helper method to SetColumnFilter
		/// </summary>
		/// <param name="col">Filter column</param>
		/// <param name="type">Filter type</param>
		/// <param name="text">Filter text</param>
		/// <returns></returns>
		public virtual int SetColumnFilter(int col, MMC_FILTER_TYPE type, string text)
		{
			int retval = 0;
			int bufLen = 1000;
			MMC_FILTERDATA data = new MMC_FILTERDATA();

			try 
			{
				// Some arbitrary buffer length.
				data.pszText = IntPtr.Zero;

				// COM magic.
				data.pszText = Marshal.AllocCoTaskMem(bufLen);
				data.cchTextMax = bufLen;
				data.pszText = Marshal.StringToCoTaskMemUni(text);
				
				retval = HeaderCtrl.SetColumnFilter((uint)col, (uint)type, data);
			}
			catch(Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			finally 
			{						
				if(! data.pszText.Equals(IntPtr.Zero)) 
				{
					Marshal.FreeCoTaskMem(data.pszText);
				}
			}
			return retval;
		}
		#endregion
	}
}
