///
/// Use #undef INCLUDEINSTALLER if you don't want/need the functionality
/// 
#define INCLUDEINSTALLER

using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Collections.Specialized;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

using Ironring.Management.MMC;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// The Base class for the snapin object implements the IComponentData interface that is the primary
	/// communication conduit to MMC.  Contains our IComponent implementation as well to handle the other
	/// part of the MMC dialog.  This class also contains the master collection of Nodes and will dispatch
	/// to the appropriate Node when notifications arrive.  All nodes must register back to this class on 
	/// creation for registration in the master collection.  This happen automatically in the BaseNode 
	/// contructor.
	/// </summary>
	public class SnapinBase :
		//Added by Alexander Kachalkov
		//Allow marshaling from the Property Sheet thread
		//using Invoke method
		System.Windows.Forms.Control
		,IComponentData
		,IExtendPropertySheet
		,IExtendContextMenu
		,IPersistStream
	{
		// Added by MAM
		/// <summary>
		/// The scope node that is currently selected for this snapin. This is needed
		/// for component.cs which goes through hoops to get the currently selected
		/// ScopeNode.
		/// </summary>
		protected BaseNode m_currentScopeNode;
		public BaseNode CurrentScopeNode {
			get { return m_currentScopeNode; }
			set { m_currentScopeNode = value; }
		}

		/// <summary>
		/// Contained object that implements IComponent on our behalf
		/// </summary>
		protected Component m_Component = null;

		/// <summary>
		/// A cached reference to the MMC console 
		/// </summary>
		protected IConsole2 m_Console = null;

		/// <summary>
		/// A cached reference to the MMC console namespace 
		/// </summary>
		protected IConsoleNameSpace2 m_ConsoleNameSpace = null; 

		/// <summary>
		/// Master list of all Nodes displayed in the snapin 
		/// this collection "owns" them and assignes cookie values
		/// </summary>
		//protected ArrayList m_Nodes = new ArrayList(16);				
		protected HybridDictionary m_Nodes = new HybridDictionary(8);
		protected int m_NodeId = 0;

		/// <summary>
		/// global images collection 
		/// </summary>
		protected ImageList m_Images = new ImageList();
		public System.Windows.Forms.ContextMenu FilterMenu;

		//Added by Alexander Kachalkov
		/// <summary>
		/// Global images collection
		/// Using it from Component class while creating Toolbar.
		/// </summary>
		public ImageList Images
		{
			get { return m_Images; }
		}

		/// <summary>
		/// Represents the window handle that the control is bound to.
		/// </summary>
		protected IntPtr m_Handle;

		/// <summary>
		/// The window handle property
		/// </summary>
		public IntPtr SnapinHandle
		{
			get {return m_Handle;}
		}

		//////////////////////////////////////////////////////////////////////////
		//
		// SnapinBase Implementation
		//
		#region
		/// <summary>
		/// Default Constructor
		/// </summary>
		public SnapinBase()
		{
			//Added by Alexander Kachalkov
			InitializeComponent();

			// Get the window handle and keep it alive, as long as the MMC is.
			m_Handle = Handle;

			this.SnapinState = CreateSnapinState();
			System.Diagnostics.Debug.WriteLine("Snapin<init>");
		}

		/// <summary>
		/// return the guid for the snapin - dig it out of the Guid attribute in the base class!
		/// Gotta love reflection.
		/// </summary>
		public Guid Guid
		{
			get 
			{
				object[] attrs = GetType().GetCustomAttributes(typeof(GuidAttribute), true);
				if (attrs.Length == 0)
					throw new SnapinException("Failed ot find GuidAttribute on SnapinBase class");

				return new Guid(((GuidAttribute)attrs[0]).Value);
			}
		}

		/// <summary>
		/// get/set the root node - this starts off the population of the entire
		/// namespace.  The RootNode is also known as the static node in MMC.  It 
		/// will be queried for its children.
		/// </summary>
		public BaseNode RootNode
		{
			get { return (BaseNode)m_Nodes[0]; }
			set { m_Nodes[0] = value; }
		}

		/// <summary>
		/// Return the Componenet's console
		/// </summary>
		public IConsole2 ResultViewConsole
		{
			get { return m_Component.Console; }
		}

		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IConsole2 Console
		{
			get { return m_Console; }
		}

		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IConsoleNameSpace2 ConsoleNameSpace
		{
			get { return m_ConsoleNameSpace; }
		}

		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IHeaderCtrl2 HeaderCtrl
		{
			get { return (IHeaderCtrl2)m_Console; }
		}
		//Added by Alexander Kachalkov
		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		/// <param name="ResultData"></param>
		public virtual RESULTDATAITEM GetSelectedItem(IResultData ResultData)
		{
			return GetSelectedItem(ResultData, -1);	// nIndex == -1 to start at first item
		}
		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		/// <param name="ResultData"></param>
		/// <param name="nIndex"></param>
		public virtual RESULTDATAITEM GetSelectedItem(IResultData ResultData, int nIndex)
		{
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= (uint)RDI.STATE;		// nState is valid 
			rdi.nCol	= 0;
			rdi.nIndex  = nIndex;				// nIndex == -1 to start at first item
			rdi.nState  = (uint)LVIS.SELECTED;	// only interested in selected items

			ResultData.GetNextItem(ref rdi);
			//out
			return rdi;
		}

		/// <summary>
		/// This method returns the selected row in the result pane. 
		/// </summary>
		/// <returns></returns>
		public virtual int GetSelectedResultRow()
		{
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= (uint)RDI.STATE;		// nState is valid 
			rdi.nCol	= 0;
			rdi.nIndex  = -1;					// nIndex == -1 to start at first item
			rdi.nState  = (uint)LVIS.SELECTED;	// only interested in selected items

			IResultData ResultData = m_Component.Console as IResultData;
			ResultData.GetNextItem(ref rdi);

			// nIndex is the selected row
			return rdi.nIndex;
		}


		/// <summary>
		/// find a node with the given cookie in our master index
		/// </summary>
		/// <param name="cookie"></param>
		/// <returns></returns>
		public BaseNode FindNode(int cookie)
		{
			// some nodes use the high order word for item id
			// so mask off this word to get the "real" cookie.
			int cleanCookie = cookie & 0xffff;
			BaseNode node = (BaseNode)m_Nodes[cleanCookie];
			if (node == null)
				throw new SnapinException("Failed to find Node with cookie " + cleanCookie.ToString());

			return node;
		}
		
		/// <summary>
		/// find a node by the MMC defined HScopeID 
		/// </summary>
		/// <param name="HScopeID"></param>
		/// <returns></returns>
		public BaseNode FindNodeByHScope(int HScopeID)
		{

			foreach(BaseNode node in m_Nodes.Values)
			{
				if (node.HScopeItem == HScopeID)
					return node;
			}
			
			return null;
		}

		/// <summary>
		/// called be BaseNode constructor to register itself back here for  
		/// centralized lookup when MMC want to talk to a Node
		/// </summary>
		/// <param name="newNode"></param>
		/// <returns></returns>
		public int Register(BaseNode newNode)
		{
			// return the index in the array - that is opaque to the node - it shouldn't 
			// make any assumptions about the value - thats our job in this class.
			// We may change the collection strategy in the future if we need to 
			// support bigger node sets.
			int id = m_NodeId;
			m_Nodes.Add(id, newNode);
			m_NodeId++;
			return id;
		}
		
		//Added by Alexander Kachalkov
		/// <summary>
		/// Unregister node
		/// </summary>
		/// <param name="node"></param>
		public void Unregister(BaseNode node){
			m_Nodes.Remove(node.Cookie);
			//Added by Alexander Ovchinnikov
			node.RemovePropertySheet( );
		}

		/// <summary>
		/// Add a string representing the embedded resource name of an icon to
		/// the snapin's global image coolection
		/// </summary>
		/// <param name="iconResourceName"></param>
		public int AddImage(string iconResourceName)
		{
			return m_Images.Add(iconResourceName);
		}
		
		//Added by Alexander Kachalkov
		/// <summary>
		/// Add a string representing the embedded resource name of an icon to
		/// the snapin's global image coolection
		/// </summary>
		/// <param name="iconResourceName"></param>
		/// <param name="transparentColor">Specifies the TransparentColor of the image to be loaded and appended</param>
		/// <returns></returns>
		public int AddImage(string iconResourceName, Color transparentColor)
		{
			return m_Images.Add(iconResourceName, transparentColor);
		}

		/// <summary>
		/// Add a loaded and initialized icon to the snapin global image collection
		/// </summary>
		/// <param name="icon"></param>
		public int AddImage(Icon icon)
		{
			return m_Images.Add(icon);
		}

		//Added by Arunjeet Singh
		/// <summary>
		/// Programatically selects a scope node in the Snap-in. This method can be
		/// used to refresh the data in the result view.
		/// </summary>
		/// <param name="node">The node to select</param>
		public void SelectScopeNode(BaseNode node)
		{
			Console.SelectScopeItem(node.HScopeItem);
		}

		//Added by Manny Vellon
		/// <summary>
		/// Called when the user selects the Help menu item related to the snap-in as a whole
		/// </summary>
		public virtual void OnHelp()
		{
		}

		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// IComponentData implementation
		//
		#region
		/// <summary>
		/// Called by MMC with the Console interface on startup we do snapin one time init stuff here 
		/// </summary>
		/// <param name="pUnknown">Implements IConsole()2 and IConsoleNameSpace2 interfaces</param>
		public void Initialize(Object pUnknown)
		{
			try
			{
				// cache references to MMC interfaces
				m_Console = (IConsole2)pUnknown;
				m_ConsoleNameSpace = (IConsoleNameSpace2)pUnknown;

				// alow startup init for each node
				foreach(BaseNode node in m_Nodes.Values)
					node.Initialize();

				// Now we'll add the images we need for the snapin
				IImageList il = null;
				m_Console.QueryScopeImageList(out il);
            
				// add the snapin - global images to the scope pane
				m_Images.LoadImageList(il);

				// forward to the root node to see if any nodes care to add their node specific 
				// icons to the image list
				RootNode.OnAddScopePaneImages(il);
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("Initialize failed - " + e.Message);
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}

		// MAM: made virtual.
		/// <summary>
		/// Give MMC our IComponent implementation when asked
		/// </summary>
		/// <param name="ppComponent">a pointer to out Component object</param>
		public virtual void CreateComponent(out IComponent ppComponent)
		{
			//
			// Manny Vellon 8/17/05
			//	I believe we MUST create new Component, anytime we are called. When
			//  multiple views are present, the component is key to distinguishing between
			//  them (as the component holds the back reference to the associated Console).
			//
			// Therefore, I am commenting out the conditional below:
			//  // Make sure we don't already have a component created
			//     if (m_Component == null)
			//
				m_Component = new Component(this);

			ppComponent = m_Component;
		}

		/// <summary>
		/// Set current snapin Component
		/// </summary>
		/// <param name="component"></param>
		public void SetCurrentComponent(Component component)
		{
			m_Component = component;
		}

		/// <summary>
		/// This notify is primarily responsible for inserting data items into 
		/// the console namespace.
		/// </summary>
		/// <param name="lpDataObject">the node to notify</param>
		/// <param name="aevent">notification type</param>
		/// <param name="arg">optional event context</param>
		/// <param name="param">optional event context</param>
		/// <returns></returns>
		public int Notify(IDataObject lpDataObject, uint aevent, IntPtr arg, IntPtr param)
			{
			int hr = HRESULT.S_OK;

			try
			{
				BaseNode node;
				if (lpDataObject is DataObject)
					node = ((DataObject) lpDataObject).Node;
				else
					node = RootNode;

				// TBD: add more events and dispatch to virtual methods instead? or event fire events?

				MMCN_Notify notify = (MMCN_Notify)aevent;

				System.Diagnostics.Debug.WriteLine("IComponentData::Notify - " + notify.ToString());

				switch(notify)
				{
					// If a data item is expanding, we need to tell MMC about it's children.
					// Note, this case doesn't necessarily mean the data item is expanding
					// visually.... MMC is just requesting information about it.
					//
					// (MV 03-14-05) NOTE: if the snapin is an extension that extends a namespace
					// node, then Notify(EXPAND) will be called when the "parent" namespace node is 
					// expanded. In this case, it's not enough to expand the children of the root
					// node; we also have to _insert_ the root node into the parent's scope tree.
					// In order to determine if this is the case, there are two possibilities. First,
					// if lpDataObject is not a DataObject then, by definition, it's not one of our
					// nodes and we'll _assume_ that we are extending it. Second, if lpDataObject IS
					// a DataObject but its Node.Snapin doesn't match _this_, then we assume
					// that we're extending another Ironring primary snapin. 
					// TODO: we can make this test more rigid by querying the GUID of the lpDataObject
					// and checking to see if it's one of the ones listed in the ExtendsNode attributes
					// in our class metadata.
					case MMCN_Notify.EXPAND:
					{
						// see if we're handling the extension snapin case
						if (!(lpDataObject is DataObject) ||
							(((DataObject)lpDataObject).Node.Snapin!=this))
						{
							// yep, it's the extension case
							if ((int)arg!=0)
								node.OnExtensionExpand(lpDataObject, (int) param);
							else
								node.OnExtensionCollapse(lpDataObject);
						}
						else
						{
							// nope, not the extension case - handle regularly
							if ((int)arg != 0)
							{
								if (node.HScopeItem == 0)
									node.HScopeItem = (int)param;
								node.OnExpand();
							}
							else
							{
								node.OnCollapse();
							}
						}
						break;
					}
					//---added by Roman Kiss
					case MMCN_Notify.DELETE:
					{
						node.OnDelete();
						break;
					}
					//Added by Alexander Kachalkov
					case MMCN_Notify.BTN_CLICK:
					{
						// CmdID of the button equal to a value of the MMC_COMMANDS enum type.
						int cmdid = param.ToInt32();
						node.OnBtnClick(cmdid);
						break;
					}
					//Added by Alexander Kachalkov
					case MMCN_Notify.RENAME:
					{
						// can we rename?
						if ((int)arg == 0)
						{
							// can we rename?
							if (!node.OnTryRename())
								hr = HRESULT.S_FALSE;
						}
							// rename editing complete - here is the string
						else // arg == 1
						{
							// is the renamed value ok?
							if (!node.OnRename(Marshal.PtrToStringAuto(param)))
								hr = HRESULT.S_FALSE;
						}

						break;
					}
					default:
					{
						// We didn't handle the message
						hr = HRESULT.S_FALSE;
						break;
					}
				}
			}
            
			catch(SnapinException)
			{
				hr = HRESULT.S_FALSE;
			}
			catch(Exception e)
			{
				//Added by Lesley van Zijl
				//MMC will eat the throw
				System.Diagnostics.Debug.WriteLine(e.Message +"\n"+ e.StackTrace);

				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}

			return hr;
		}

		/// <summary>
		///  Called by MMC to cleanup 
		///  TBD: call Dispose - after that pattern is more pervaisivly used
		/// </summary>
		public virtual void Destroy()
		{
		}

		// Lesley v. Zijl: made virtual.
		/// <summary>
		/// When MMC wants a data object for a specific node 
		/// </summary>
		/// <param name="cookie"></param>
		/// <param name="type"></param>
		/// <param name="ppDataObject"></param>
		public virtual void QueryDataObject(int cookie, uint type, out IDataObject ppDataObject)
		{
		    BaseNode node = FindNode(cookie);
		    ppDataObject = m_Component.createDataObject(node); 
		}

		/// <summary>
		/// provides scope pane info to MMC  
		/// </summary>
		/// <param name="sdi"></param>
		public virtual void GetDisplayInfo(ref SCOPEDATAITEM sdi)
		{
			// First let's find this node we want info on....
			BaseNode NodeWeWant = FindNode(sdi.lParam);
			NodeWeWant.GetDisplayInfo(ref sdi);
		}

   		
		/// <summary>
		/// This method will compare two data objects based on underlying cookie value 
		/// that the nodes contain
		/// </summary>
		/// <param name="lpDataObjectA"></param>
		/// <param name="lpDataObjectB"></param>
		/// <returns></returns>
		public int CompareObjects(IDataObject lpDataObjectA, IDataObject lpDataObjectB)
		{
			// since this is the only type we hand out it should be the only type 
			// provided here and the cast should "always" work.

			DataObject doItem1 = (DataObject)lpDataObjectA;
			DataObject doItem2 = (DataObject)lpDataObjectB;

			// TBD: implement IComparable on Node and compare cookies.

			// These are different objects. We need to return S_FALSE
			if (doItem1.Node.Cookie != doItem2.Node.Cookie)
				return HRESULT.S_FALSE;

			return HRESULT.S_OK;
		}
		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// IExtendPropertySheet implementation
		//
		#region
		// MAM: made virtual
		/// <summary>
		/// MMC wants a property sheet created of the given node 
		/// </summary>
		/// <param name="lpProvider"></param>
		/// <param name="handle"></param>
		/// <param name="lpDataObject"></param>
		/// <returns></returns>
		public virtual int CreatePropertyPages(IPropertySheetCallback lpProvider, IntPtr handle, IDataObject lpDataObject)
		{
			BaseNode node=null;
			if (lpDataObject is DataObject)
				node = ((DataObject)lpDataObject).Node;
			else
				node = RootNode;
	    
			// Let's see if this node has property pages
			if (node.HavePropertyPages)
				node.CreatePropertyPages(lpProvider, handle);

			// don't use the MMC property page facilities - just roll our own.
			return HRESULT.S_FALSE;
		}

		// MAM: made virtual.
		/// <summary>
		/// MMC calls this at initialization time to find out if the node  
		/// supports property pages - this is the one and only time to respond.
		/// </summary>
		/// <param name="lpDataObject">IdataObject for Node in question</param>
		/// <returns></returns>
		public virtual int QueryPagesFor(IDataObject lpDataObject)
		{
			// This snapin does have property pages, so we should return S_OK
			// (which will happen automatically). If the snapin didn't have
			// any property pages, then we should return S_FALSE

			DataObject dataObject = (DataObject)lpDataObject;
	    
			//---added by Roman Kiss
			dataObject.Node.OnQueryProperties();

			// Let's see if this node has property pages
			if (dataObject.Node.HavePropertyPages)
				return HRESULT.S_OK;

			return HRESULT.S_FALSE;
		}
		#endregion
		//////////////////////////////////////////////////////////////////////
		//
		// IExtendContextMenu implementation
		//
		#region
		/// <summary>
		/// Add menu items based on MMC's insertion rules 
		/// </summary>
		/// <param name="piDataObject"></param>
		/// <param name="piCallback"></param>
		/// <param name="pInsertionAllowed"></param>
		public void AddMenuItems(IntPtr piDataObject, IContextMenuCallback piCallback, ref int pInsertionAllowed)
		{
			DataObject item = null;
			try
			{
				//Added by Alexander Kachalkov
				IDataObject iDataObject = (IDataObject)Marshal.GetObjectForIUnknown(piDataObject);
				
				// The piDataObject is really a DataObject is disguise....
				item = (DataObject)iDataObject;


				if (item.Node != null)
					item.Node.AddMenuItems(piCallback, ref pInsertionAllowed);
			}
			catch(SnapinException e)
			{
				System.Diagnostics.Debug.WriteLine("SnapinBase::AddMenuItems SnapinException - " + e.ToString());
				// eat the exception and return s_false indicating that 
				// we didn't handle the notification
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("SnapinBase::AddMenuItems Exception - " + e.ToString());
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}
		/// <summary>
		/// Called when user invokes a menu command
		/// </summary>
		/// <param name="lCommandID"></param>
		/// <param name="piDataObject"></param>
		public void Command(int lCommandID, IntPtr piDataObject)
		{
			DataObject item = null;
			try
			{
				//Added by Alexander Kachalkov
				IDataObject iDataObject = (IDataObject)Marshal.GetObjectForIUnknown(piDataObject);
				
				// The piDataObject is really a DataObject is disguise....
				item = (DataObject)iDataObject;

				// pass the command on to the node
				if (item.Node != null)
					item.Node.Command(lCommandID);
			}
			catch(SnapinException e)
			{
				System.Diagnostics.Debug.WriteLine("SnapinBase::AddMenuItems SnapinException - " + e.ToString());
				// eat the exception and return s_false indicating that 
				// we didn't handle the notification
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine("SnapinBase::AddMenuItems Exception - " + e.ToString());
				//throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}
		#endregion

		/// <summary>
		/// Register the snap-in with MMC. 
		/// 
		/// NOTE: [mv 03-25-05] This works when REGASM is invoked manually
		/// or from DEVSTUDIO. It does NOT work, however, with the installer
		/// programs that we have looked at (InstallShield, Wise). They do
		/// a static analysis (either by running REGASM /regfile or by doing
		/// the equivalent work programmatically) and only set up the CLSIDs
		/// and other "COM" stuff by the DON'T invoke this ComRegisterFunction.
		/// In order for these programs to work, we have to do registration
		/// the "InstallUtil" way, done in the RegisterSnapin elsewhere in this 
		/// module
		/// </summary>
		[ComRegisterFunction()]
		public static void RegisterSnapIn(Type t)
		{
			SnapinRegistrar.Register(t.Assembly);
		}

		/// <summary>
		/// Unregister the snap-in with MMC
		/// </summary>
		[ComUnregisterFunction()]
		public static void UnregisterSnapIn(Type t)
		{
			SnapinRegistrar.UnRegister(t.Assembly);
		}

		private void InitializeComponent()
		{
			this.FilterMenu = new System.Windows.Forms.ContextMenu();

		}

		//////////////////////////////////////////////////////////////////////
		//
		// IPersistStream implementation.
		//		
		#region IPersistStream		
		/// <summary>
		/// The state for this snapin.
		/// </summary>
		protected SnapinStateBase m_SnapinState;
		public virtual SnapinStateBase SnapinState {
			get { return this.m_SnapinState; }
			set { this.m_SnapinState = value; }
		}

		/// <summary>
		/// Normaly IPersistStreamInit would be used. This doesn't
		/// seem to work though. Instead we fake the init. Our
		/// init() will call this method to get a default SnapinState.
		/// Override when needed.
		/// </summary>
		/// <returns></returns>
		public virtual SnapinStateBase CreateSnapinState() {
			SnapinStateBase ssb = new SnapinStateBase();
			return ssb;
		}
		
		/// <summary>
		/// Get the class-id for this snapin.
		/// </summary>
		/// <param name="pClassID"></param>
		public void GetClassID(out Guid pClassID) {			
			System.Diagnostics.Debug.WriteLine("GetClassID");
			pClassID = this.Guid;
		}
		
		/// <summary>
		/// Check if the snapin-state is dirty.
		/// </summary>
		/// <returns></returns>
		public int IsDirty( ) {
			System.Diagnostics.Debug.WriteLine("IsDirty");
			if(this.SnapinState != null) {
				if(this.SnapinState.Dirty) {
					return 0;
				} else {
					return -1;
				}
			} else {
				return -1;
			}
		}

		/// <summary>
		/// Delegate back to the snapin-state for de-serializing.
		/// </summary>
		/// <param name="pStm"></param>
		public void Load(UCOMIStream pStm) {
			try {
				System.Diagnostics.Debug.WriteLine("Load");
				ComStream cs = new ComStream(pStm);
				this.SnapinState.ReadState(cs);
				cs.Close();
				System.Diagnostics.Debug.WriteLine("Loaded state: " + this.SnapinState);				
			}
			catch(Exception e) {
				System.Diagnostics.Debug.WriteLine("Exception in load: " + e);
			}
		}

		/// <summary>
		/// Delegate back to the snapin-state for serialization.
		/// </summary>
		/// <param name="pStm"></param>
		/// <param name="fClearDirty"></param>
		public void Save(UCOMIStream pStm, [In, MarshalAs(UnmanagedType.Bool)] bool fClearDirty) {			
			try {
				System.Diagnostics.Debug.WriteLine("Save");				
				ComStream cs = new ComStream(pStm);
				this.SnapinState.WriteState(cs);
				cs.Close();
			}
			catch(Exception e) {
				System.Diagnostics.Debug.WriteLine("Got an exception" + e + " stack " + e.StackTrace);				
			}
		}
	

		/// <summary>
		/// As mandated by the IPersistStream interface. Not implemented
		/// though: it is not needed yet by MMC and would be the same as calling
		/// save and counting bytes if you are really inclined.
		/// </summary>
		/// <param name="pcbSize"></param>
		public void GetSizeMax(out long pcbSize) {
			System.Diagnostics.Debug.WriteLine("GetSizeMax");
			//throw new Exception("MMC didn't use to call GetSizeMax. Implement this.");
			ceh.OnThreadException(this, new ThreadExceptionEventArgs(new ApplicationException("MMC didn't use to call GetSizeMax. Implement this.")));
			pcbSize = 0;
		}

		/// <summary>
		/// NOT USED! This is part of the IPersistStreamInit() interface.
		/// This would be a more natural fit for our purpose, but I
		/// can't get it to work. Leave it here as documentation or
		/// for someone brave enough to try it.
		/// </summary>
		/// <returns></returns>
		public int InitNew() {
			System.Diagnostics.Debug.WriteLine("InitNew");
			if(this.SnapinState == null) {
				System.Diagnostics.Debug.WriteLine("No previous state");
				this.SnapinState = CreateSnapinState();
				return 0;
			} else {
				System.Diagnostics.Debug.WriteLine("Previous state found. Die!");
				Marshal.ThrowExceptionForHR(unchecked((int)0x8000FFFF));
				return 0;
			}
		}
		#endregion

		//Added by Alexander Ovchinnikov
		internal void GetSelectedResultDataItem( ref RESULTDATAITEM rdi ){
			rdi.mask	= (uint)RDI.STATE;		// nState is valid 
			rdi.nCol	= 0;
			rdi.nIndex  = -1;				// nIndex == -1 to start at first item
			rdi.nState  = (uint)LVIS.SELECTED;	// only interested in selected items

			
			IResultData ResultData = m_Component.Console as IResultData;
			ResultData.GetNextItem(ref rdi);
		}

	    public void AddExtension(int hScopeItem, Guid guid)
	    {
            IntPtr hItem = new IntPtr(hScopeItem);

            byte[] guidArray = guid.ToByteArray();

            CLSID clsid = new CLSID();

            clsid.x = ((uint)guidArray[3] << 24) + ((uint)guidArray[2] << 16) + ((uint)guidArray[1] << 8) + ((uint)guidArray[0]);
            clsid.s1 = (ushort)(((uint)guidArray[5] << 8) + ((uint)guidArray[4]));
            clsid.s2 = (ushort)(((uint)guidArray[7] << 8) + ((uint)guidArray[6]));
            clsid.c1 = guidArray[8];
            clsid.c2 = guidArray[9];
            clsid.c3 = guidArray[10];
            clsid.c4 = guidArray[11];
            clsid.c5 = guidArray[12];
            clsid.c6 = guidArray[13];
            clsid.c7 = guidArray[14];
            clsid.c8 = guidArray[15];

	        ConsoleNameSpace.AddExtension(hItem, ref clsid);
	    }

	}

#if INCLUDEINSTALLER

	/// <summary>
	/// Registers the snap-in with MMC.
	/// 
	/// [mv 03-25-05] This is another way of performing self-installation.
	/// This way works with the InstallUtil tool or with the InstallShield
	/// and Wise installers.
	/// </summary>
	[System.ComponentModel.RunInstaller(true)]
	public class RegisterMMC: System.Configuration.Install.Installer
	{
		public override void Install(IDictionary savedState)
		{
			base.Install(savedState);

			RegistrationServices rs = new RegistrationServices();
			Assembly a = Assembly.GetExecutingAssembly();
			rs.RegisterAssembly(a, AssemblyRegistrationFlags.SetCodeBase);
		}

		public override void Uninstall(IDictionary savedState)
		{
			base.Uninstall(savedState);

			RegistrationServices rs = new RegistrationServices();
			Assembly a = Assembly.GetExecutingAssembly();
			rs.UnregisterAssembly(a);
		}
	}

	/// <summary>
	/// This class is used by snapins that rely on MMCLib.
	/// 
	/// [mv 03-25-05] The snapin must declare it's own class
	/// as a superclass of this one and mark it with the RunInstaller
	/// attribute.
	/// </summary>
	public class RegisterSnapin: System.Configuration.Install.Installer
	{
		public override void Install(IDictionary savedState)
		{
			base.Install(savedState);

			RegistrationServices rs = new RegistrationServices();
			Type t = this.GetType();
			Assembly a = Assembly.GetAssembly(t);
			rs.RegisterAssembly(a, AssemblyRegistrationFlags.SetCodeBase);
		}

		public override void Uninstall(IDictionary savedState)
		{
			base.Uninstall(savedState);

			RegistrationServices rs = new RegistrationServices();
			Type t = this.GetType();
			Assembly a = Assembly.GetAssembly(t);
			rs.UnregisterAssembly(a);
		}
	}

#endif

}
