using System;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// Implements IDataObject interface.  Responsible for mapping IDataObject 
	/// pointers that MMC uses to our managed Node objects.
	/// </summary>
	public class DataObject : IDataObject
	{
        /// <summary>
        /// Clipboard format for Display Name
        /// </summary>
		protected static ushort s_DisplayName; 

        /// <summary>
        /// Clipboard format for NodeType
        /// </summary>
		protected static ushort s_NodeType;    
 
        /// <summary>
        /// Clipboard format for SZNodeType
        /// </summary>
		protected static ushort s_SZNodeType;  

        /// <summary>
        /// Clipboard format for SnapinClsid
        /// </summary>
		protected static ushort s_SnapinClsid; 

		//Added by Alexander Kachalkov
		//---------------------------------------------------------------------------
		//  MultiSelect clipboard formats
		protected static ushort s_Snapins;
		protected static ushort s_Objects;
		protected static ushort s_IsMsObj;
		protected static ushort s_Cookies; // Custom clipformat for getting cookies

        /// <summary>
        /// The node that this Data object is
        /// </summary>
		protected BaseNode m_NodeData;
		// responsible for

        // flags
        public static readonly int NULL = 0;
        public static readonly int CUSTOMOCX = -1;
        public static readonly int CUSTOMWEB = -2;

        /////////////////////////////////////////////////////////////////////
		// 
		// Implementation
        // 
		#region
        /// <summary>
        /// Construct the IDataObject implementation and associate 
        /// it with the node provided
        /// </summary>
        /// <param name="initData"></param>
        public DataObject(BaseNode initData)
        {
            // Set the Node this Data Object will be responsible for
            m_NodeData = initData;
        }

        /// <summary>
        /// Static class constructor used to init our static data
        /// </summary>
        static DataObject()
        {
            // Get the Clipboard Format Numbers for these various items.
            // MMC should have already registered these Clipboard Formats,
            // so this call just gives us the id assigned for each format.

            s_NodeType = RegisterClipboardFormat("CCF_NODETYPE");
            s_SZNodeType = RegisterClipboardFormat("CCF_SZNODETYPE");
            s_SnapinClsid = RegisterClipboardFormat("CCF_SNAPIN_CLASSID");
            s_DisplayName = RegisterClipboardFormat("CCF_DISPLAY_NAME");

			//Added by Alexander Kachalkov
			// MULTISELECT: Register new clipformats
			s_Snapins   = RegisterClipboardFormat("CCF_MULTI_SELECT_SNAPINS");
			s_Objects   = RegisterClipboardFormat("CCF_OBJECT_TYPES_IN_MULTI_SELECT");
			s_IsMsObj   = RegisterClipboardFormat("CCF_MMC_MULTISELECT_DATAOBJECT");

			s_Cookies   = RegisterClipboardFormat("CCF_MULTISELECT_COOKIES");
		}
    
        /// <summary>
        /// The node associated with the IDataObject
        /// </summary>
		public BaseNode Node
		{
			get { return m_NodeData; }
		}

		//-------------------------------------------------
		// GetDataHere
		//
		// This function will send certain data to the MMC. Note
		// this function uses the "unsafe" context... we need to
		// send a pointer to a byte array to IStream:Write, and this
		// is the easiest way to do it.
		//-------------------------------------------------
		/// <summary>
        /// Send data to MMC in a stream depending on what is requested 
        /// </summary>
        /// <param name="pFormatEtc"></param>
        /// <param name="pMedium"></param>
		public void GetDataHere(ref FORMATETC pFormatEtc, ref STGMEDIUM pMedium)
		{
            System.Diagnostics.Debug.WriteLine("DataObject::GetDataHere");

            try
            {
                // We'll send this array if we don't know what to send
                byte[] Nothing = {0x0, 0x0};

                ushort cf = (ushort)pFormatEtc.cfFormat;

                // make a memory stream to handle streaming data to a buffer
                MemoryStream stream = new MemoryStream(128);
                Encoding ue = new UnicodeEncoding();
                BinaryWriter writer = new BinaryWriter(stream, ue);

                // See if we need to send a string....
                if (cf == s_DisplayName || cf == s_SZNodeType)
                {
                    if (cf == s_DisplayName)
                        writer.Write(m_NodeData.DisplayName.ToCharArray());
                    else
                        writer.Write(Nothing);
                }
                    // We need to send a GUID
                else if (cf == s_NodeType || cf == s_SnapinClsid)
                {
                    Guid guid;

                    if (cf == s_NodeType)
                    {
                        guid = m_NodeData.Guid;
                    }
                    else //if (cf == s_SnapinClsid)
                    {
                        guid = m_NodeData.Snapin.Guid;
                    }

                    writer.Write(guid.ToByteArray());
                }
                else
                {
                    const uint DV_E_FORMATETC = 0x80040064;

                    bool handled = m_NodeData.HandleCustomClipboardFormat(cf, writer);
                    if (!handled)
                    {
                        Marshal.ThrowExceptionForHR(Convert.ToInt32(DV_E_FORMATETC));
                    }
                    
                }

                // Write the memstream to the IStream.
                //IStream pStream = null;   
                UCOMIStream pStream = null;
                CreateStreamOnHGlobal(pMedium.hGlobal, 0, out pStream );
    			
                // If we couldn't open a global handle....
                if (pStream == null)
                    throw new SnapinException("Fail to CreateStreamOnHGlobal");

                writer.Flush();
				
				IntPtr dataSentPtr = Marshal.AllocCoTaskMem(4);
				pStream.Write(stream.GetBuffer(), (int)stream.Length, dataSentPtr);

				int dataSent = Marshal.ReadInt32(dataSentPtr);
				Marshal.FreeCoTaskMem(dataSentPtr);

				if (dataSent != stream.Length)
					throw new SnapinException("Cannot write the data provided.");

            }
            catch(SnapinException e)
            {
                System.Diagnostics.Debug.WriteLine("DataObject::GetDataHere " + e.Message);
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
            }
		}

	    //-------------------------------------------------
		// Other IDataObject Methods
		//
		// We don't need to implement any of these other 
		// methods, so if they are called, we'll just return
		// E_NOTIMPL
		//-------------------------------------------------

		/// <summary>
		/// Not Implemented
		/// </summary>
		/// <param name="pFormatetcIn"></param>
		/// <param name="pMedium"></param>
		/// <returns></returns>
		public int GetData(ref FORMATETC pFormatetcIn, ref STGMEDIUM pMedium)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::GetData return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
			//return HRESULT.S_OK;

//			try
//			{
//				ushort cf = (ushort)pFormatetcIn.cfFormat;
//
//				// See if we need to send a string....
//				if (cf == s_Objects)
//				{
//					//Guid guid = m_NodeData.Guid;
//
//					return HRESULT.S_OK;
//				}
//
//			}
//			catch(SnapinException e)
//			{
//				System.Diagnostics.Debug.WriteLine("DataObject::GetDataHere " + e.Message);
//				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
//			}

			//return HRESULT.S_FALSE;
		}


        /// <summary>
        /// Not Implemented
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
		public int QueryGetData(IntPtr a)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::QueryGetData return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}

        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
		public int GetCanonicalFormatEtc(IntPtr a, IntPtr b)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::GetCanonicalFormatEtc return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}

        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <param name="c"></param>
        /// <returns></returns>
		public int SetData(IntPtr a, IntPtr b, int c)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::SetData return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}
    
        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
		public int EnumFormatEtc(uint a, IntPtr b)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::EnumFormatEtc return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}
    
        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <param name="c"></param>
        /// <param name="d"></param>
        /// <returns></returns>
		public int DAdvise(IntPtr a, uint b, IntPtr c, ref uint d)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::DAdvise return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}

        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
		public int DUnadvise(uint a)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::DUnadvise return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}

        /// <summary>
        /// Not impl
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
		public int EnumDAdvise(IntPtr a)
		{
			//System.Diagnostics.Debug.WriteLine("DataObject::EnumDAdvise return HRESULT.E_NOTIMPL");
			return HRESULT.E_NOTIMPL;
		}
		#endregion

        //////////////////////////////////////////////////////////////////////////
        // P/Invoke Helpers

        /// <summary>
        /// Used to create an IStream over global memory
        /// </summary>
		[DllImport("ole32.dll")]
		protected static extern int CreateStreamOnHGlobal(int hGlobal, int fDeleteOnRelease, out UCOMIStream ppstm);

        /// <summary>
        /// Get the clipboard format ids
        /// </summary>
		[DllImport("user32.dll", CharSet=CharSet.Unicode)]
		protected static extern ushort RegisterClipboardFormat(String format);
	}
}
