using System;

namespace Ironring.Management.MMC
{
	public enum ExtensionType
	{
		NameSpace=0,
		ContextMenu=1,
		Toolbar=2,
		PropertySheet=3,
		Task=4
	}

	/// <summary>
	/// Custom attribute applied to snapin classes.  The presence of this 
	/// attribute identifies it to the snapin registration utility and 
	/// associates attribute required during snapin installation to the 
	/// registry.
	/// </summary>
	[AttributeUsage(AttributeTargets.Class)]
	public class SnapinInAttribute : System.Attribute
	{
        /// <summary>
        /// The name
        /// </summary>
        protected string m_name;

        /// <summary>
        /// The company name
        /// </summary>
        protected string m_provider;
        
        /// <summary>
        /// the snapin version
        /// </summary>
        protected string m_version;

        /// <summary>
        /// Standalon boolean
        /// </summary>
        protected bool m_standAlone;

        ///////////////////////////////////////////////////////////////////////
        //
        // Implementaiton
        //

        /// <summary>
        /// Default ctor initialize to benign values
        /// </summary>
		public SnapinInAttribute()
            : this ("unknown", "unknown")
		{
		}

		/// <summary>
		/// Overloaded ctor initializes all fields
		/// </summary>
		/// <param name="namestring"></param>
		/// <param name="provider"></param>
		public SnapinInAttribute(string namestring, string provider)
            : this (namestring, provider, null)
		{
		}

		/// <summary>
		/// Overloaded ctor initializes all fields
		/// </summary>
		/// <param name="namestring"></param>
		/// <param name="provider"></param>
		/// <param name="version"></param>
		public SnapinInAttribute(string namestring, string provider, string version)
            : this(namestring, provider, version, true)
		{
		}

		/// <summary>
		/// Overloaded ctor initializes all fields
		/// </summary>
		/// <param name="namestring"></param>
		/// <param name="provider"></param>
		/// <param name="version"></param>
		/// <param name="standAlone"></param>
		public SnapinInAttribute(string namestring, string provider, string version, bool standAlone)
		{
            m_name = namestring;
            m_provider = provider;
            m_version = version;
            m_standAlone = standAlone;
        }

        /// <summary>
        /// The name
        /// </summary>
		public string Namestring
		{
			get { return m_name;  }
			set { m_name = value; }

		}

        /// <summary>
        /// StandAlone
        /// </summary>
		public bool StandAlone
		{
			get { return m_standAlone; }
			set { m_standAlone = value; }
		}

        /// <summary>
        /// Provider
        /// </summary>
		public string Provider
		{
			get { return m_provider; }
			set { m_provider = value; }
		}

        /// <summary>
        /// Version
        /// </summary>
		public string Version
		{
			get { return m_version; }
		}
    }

    /// <summary>
    /// A Node Type GUID which belongs to this snap-in.
    /// A snap-in may have one or more node type associated with it.
    ///
    /// Each of a snap-in's node type attributes will be placed in the
    /// registry when this assembly is registered, under the registry tree:
    /// HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
    ///
    /// Each node type attribute listed should correspond to the .Guid property
    /// of a node which might be added to the snap-in's tree.
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
    public class NodeTypeGuidAttribute : System.Attribute
    {
        /// <summary>
        /// Declares a node type of this snap-in, which will be entered
        /// into the registry under:
        ///     HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
        /// </summary>
        protected Guid m_nodeType;

        /// <summary>
        /// When true, this node type will not be removed from the
        ///     HKLM\Software\Microsoft\MMC\\NodeTypes
        /// section of the registry when the assembly is uninstalled.
        /// Use this if the node type belongs to an extension snap-in
        /// which is implemeted in a different DLL.
        /// </summary>
        protected bool m_fDontUnregisterNodeType;


        /// <summary>
        /// Used to override the default location of the "NodeTypes" registry
        /// key.  If this is non-null, the node will be registered under
        /// this subkey rather than under the default subkey.
        ///
        /// This key must be present within the HKEY_LOCAL_MACHINE hive.
        /// </summary>
        protected String m_sNodeTypeRegKeyCustom;

        protected static String s_sNodeTypeRegKeyDefault = "SOFTWARE\\Microsoft\\MMC\\NodeTypes";

        /// <summary>
        /// 
        /// </summary>
        public Guid NodeType
        {
            get { return m_nodeType; }
        }

        public bool DontUnregisterNodeType
        {
            get { return m_fDontUnregisterNodeType; }
        }

        /// <summary>
        /// Custom location of the "NodeTypes" registry key in the registry.
        /// If this is null, the DefaultNodeTypeRegKey should be used.
        /// </summary>
        public String CustomNodeTypeRegKey
        {
            get
            {
                return m_sNodeTypeRegKeyCustom;
            }
        }

        /// <summary>
        /// Location of the "NodeTypes" registry key in the registry.
        /// This defaults to
        ///     SOFTWARE\\Microsoft\\MMC\\NodeTypes
        /// </summary>
        public static String DefaultNodeTypeRegKey
        {
            get
            {
                return s_sNodeTypeRegKeyDefault;
            }
        }

        /// <summary>
        /// Declares a node type of this snap-in, which will be entered
        /// into the registry under:
        ///     HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
        /// </summary>
        public NodeTypeGuidAttribute(Guid nodeType)
            : this(nodeType, false, null)
        {
        }

        /// <summary>
        /// Declares a node type of this snap-in, which will be entered
        /// into the registry under:
        ///     HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
        /// and HKLM\Software\Microsoft\MMC\NodeTypes
        /// </summary>
        public NodeTypeGuidAttribute(String nodeTypeGuid)
            : this(nodeTypeGuid, false)
        {
        }

        /// <summary>
        /// Declares a node type of this snap-in, which will be entered
        /// into the registry under:
        ///     HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
        /// and HKLM\Software\Microsoft\MMC\NodeTypes
        ///     
        /// In addition, if fDontUnregisterNodeType is true, the MMC\NodeTypes
        /// subtree will not be removed when the assembly is uninstalled.
        /// Use this if the node type belongs to an extension snap-in
        /// which is implemeted in a different DLL.
        /// </summary>
        public NodeTypeGuidAttribute(Guid nodeTypeGuid,
                                     bool fDontUnregisterNodeType,
                                     String sCustomNodeTypeRegKey)
        {
            m_nodeType = nodeTypeGuid;
            m_fDontUnregisterNodeType = fDontUnregisterNodeType;
            m_sNodeTypeRegKeyCustom = sCustomNodeTypeRegKey;
        }

        /// <summary>
        /// Declares a node type of this snap-in, which will be entered
        /// into the registry under:
        ///     HKLM\Software\Microsoft\MMC\SnapIns\{snapin-GUID}\NodeTypes
        /// and HKLM\Software\Microsoft\MMC\NodeTypes
        ///     
        /// In addition, if fDontUnregisterNodeType is true, the MMC\NodeTypes
        /// subtree will not be removed when the assembly is uninstalled.
        /// Use this if the node type belongs to an extension snap-in
        /// which is implemeted in a different DLL.
        /// </summary>
        public NodeTypeGuidAttribute(String nodeTypeGuid,
                                     bool fDontUnregisterNodeType)
            : this(new Guid(nodeTypeGuid), fDontUnregisterNodeType, null)
        {
        }
    }

	/// <summary>
    /// Custom attribute applied to snapin about classes.  The presence 
    /// of this attribute identifies it to the snapin registration utility and 
    /// associates attribute required during snapin installation to the 
    /// registry.
    /// </summary>
	[AttributeUsage(AttributeTargets.Class)]
	public class AboutSnapinAttribute : System.Attribute
	{
        /// <summary>
        /// The type of the snapin we refer to.  we can pull data from 
        /// the type by reflection
        /// </summary>
        protected Type m_SnapinType;

        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="type"></param>
        public AboutSnapinAttribute(Type type)
		{
			m_SnapinType = type;
		}

        /// <summary>
        /// Get the Type the snapin is based on
        /// </summary>
		public Type SnapinType
		{
			get { return m_SnapinType; }
		}
	}

	/// <summary>
	/// A Node Type that this snap can extend.
	///
	/// The snapin GUID will be listed as an allowed extension under
	/// the node's registry entry:
	/// 
	/// HKLM\Software\Microsoft\MMC\NodeTypes\{Node GUID}\Extensions
	///
	/// The form of the attribute is ExtendsNode(Guid, ExtensionType)
	/// where Guid is the guid of the node to be extended and ExtensionType identifies
	/// the kind of extension (namespace, context, etc.)
	/// </summary>
	[AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
	public class ExtendsNodeAttribute : System.Attribute
	{
		protected Guid m_guidExtendedNode;
		protected ExtensionType m_extType;

        /// <summary>
        /// Used to override the default location of the "NodeTypes" registry
        /// key.  If this is non-null, the node will be registered under
        /// this subkey rather than under the default subkey.
        ///
        /// This key must be present within the HKEY_LOCAL_MACHINE hive.
        /// </summary>
        protected String m_sNodeTypeRegKeyCustom;

        protected static String s_sNodeTypeRegKeyDefault = "SOFTWARE\\Microsoft\\MMC\\NodeTypes";

        public ExtendsNodeAttribute(Guid guidExtendedNode, ExtensionType extType, String sNodeTypeRegKeyCustom)
        {
            m_guidExtendedNode = guidExtendedNode;
            m_extType = extType;
            m_sNodeTypeRegKeyCustom = sNodeTypeRegKeyCustom;
        }

        public ExtendsNodeAttribute(string sguidExtendedNode, ExtensionType extType, String sNodeTypeRegKeyCustom)
            : this(new Guid(sguidExtendedNode), extType, sNodeTypeRegKeyCustom)
        {
        }

		public ExtendsNodeAttribute(Guid guidExtendedNode, ExtensionType extType)
            : this(guidExtendedNode, extType, null)
		{
		}

		public ExtendsNodeAttribute(string sguidExtendedNode, ExtensionType extType)
            : this(new Guid(sguidExtendedNode), extType)
		{
		}

		public Guid GuidOfExtendedNode
		{
			get
			{
				return m_guidExtendedNode;
			}
		}

		public ExtensionType ExtType
		{
			get 
			{
				return m_extType;
			}
		}

        /// <summary>
        /// Custom location of the "NodeTypes" registry key in the registry.
        /// If this is null, the DefaultNodeTypeRegKey should be used.
        /// </summary>
        public String CustomNodeTypeRegKey
        {
            get
            {
                return m_sNodeTypeRegKeyCustom;
            }
        }

        /// <summary>
        /// Location of the "NodeTypes" registry key in the registry.
        /// This defaults to
        ///     SOFTWARE\\Microsoft\\MMC\\NodeTypes
        /// </summary>
        public static String DefaultNodeTypeRegKey
        {
            get
            {
                return s_sNodeTypeRegKeyDefault;
            }
        }

        // this property returns the registry string that's associated with
		// extensions of this particular type
		public string ExtensionTypeSubkey
		{
			get
			{
				// TODO: deal with MF_TOOLBAR, MF_WIZARD and others?
				switch (m_extType)
				{
					case ExtensionType.ContextMenu:
						return "ContextMenu";

					case ExtensionType.NameSpace:
						return "NameSpace";

					case ExtensionType.PropertySheet:
						return "PropertySheet";

					case ExtensionType.Task:
						return "Task";

					case ExtensionType.Toolbar:
						return "Toolbar";

					default:
						throw new Exception("Unexpected extension type");
				}
			}
		}
	}
}
