using System;
using System.Collections;
using System.Runtime.InteropServices;

using System.Threading;
using ceh = Ironring.Management.MMC.CustomExceptionHandler;

namespace Ironring.Management.MMC
{
	/// <summary>
	/// Configures the resultview as a Listview control.  Supports binding 
	/// to a DataTable to provide the schema and data
	/// 
	/// TBD: overridable data binding methods to augment GetDisplayInfo
	/// </summary>
	public class SortListNode : BaseNode
	{
		//Added by Alexander Kachalkov
		/// <summary>
		/// A cached reference to the MMC console 
		/// </summary>
		protected IConsole2 m_Console = null;          

		//Added by Alexander Kachalkov
		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IHeaderCtrl2 HeaderCtrl
		{
			get { return (IHeaderCtrl2)m_Console; }
		}

		//Added by Alexander Kachalkov
		/// <summary>
		/// Return the cached member variable to get at MMC
		/// </summary>
		public IResultData ResultData
		{
			get { return (IResultData)m_Console; }
		}

		/// <summary>
        /// Underlying template for the listview control
        /// </summary>
		protected SortedList m_List = null;
		protected ArrayList  m_HeaderInfo = null;

        /// <summary>
        /// ctor takes the snapin to register with
        /// </summary>
        /// <param name="snapin"></param>
		public SortListNode(SnapinBase snapin) : base(snapin)
		{
		}

		/// <summary>
		/// ctor takes the snapin to register with
		/// </summary>
		/// <param name="snapin"></param>
		/// <param name="parentNode"></param>
		public SortListNode(SnapinBase snapin, BaseNode parentNode) : base(snapin, parentNode)
		{
		}
		
		/// <summary>
        /// Access the underlying DataTable 
        /// </summary>
		public SortedList DataList{
			get{ return m_List; }
			set{ m_List = value; }
		}

		public ArrayList HeaderInfo{
			get{ return m_HeaderInfo; }
			set{ m_HeaderInfo = value; }
		}

        /// <summary>
        /// BaseNode override to populate the treeview control
        /// with column information and data
        /// </summary>
        public override void OnShow()
		{
            m_Console = m_Snapin.ResultViewConsole;

			// setup the header for the columns we want to show
			OnShowHeader(HeaderCtrl);
			
			// add the actual data items 
			OnShowData(ResultData);

            ResultData.SetViewMode((int)ViewMode.Report);

			base.OnShow();
		}
		////////////////////////////////////////////////////////////////////
		// Helper methods
		///////////////////////////////////////////////////////////////////// 
		#region
		/// <summary>
        /// Overridable Helper method to populate the column header
        /// </summary>
        /// <param name="header"></param>
		public virtual void OnShowHeader( IHeaderCtrl2 header ){
			try{
				if( m_HeaderInfo != null)
					// Let's put in the column titles
					for( int iIndex = 0; iIndex < m_HeaderInfo.Count; iIndex++ ){
						string name = m_HeaderInfo[ iIndex ] as string;
						int nWidth = (int)ColumnHeaderWidth.AUTO_WIDTH;
						header.InsertColumn( iIndex, name, (int)ColumnHeaderFormat.LEFT, nWidth );
					}
			}
			catch(COMException e)
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
                //throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
			catch(Exception e)
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
                //throw e;
				ceh.OnThreadException(this, new ThreadExceptionEventArgs(e));
			}
		}

		/// <summary>
		/// Overridable helper method to populate the column data
		/// </summary>
		/// <param name="ResultData"></param>
		public virtual void OnShowData( IResultData ResultData ){
			if( m_List != null ){
				for( int iIndex = 0; iIndex < m_List.Count; iIndex++ ){
					InsertItem(iIndex + 1 );
				}
			}			
		}

		/// <summary>
		/// Overridable helper method to InsertItem
		/// </summary>
		/// <param name="nRow"></param>
		public virtual void InsertItem( int nRow )
        {
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= ( uint )RDI.STR | ( uint )RDI.IMAGE | ( uint )RDI.PARAM;     

			// TBD: what image?
			rdi.nImage  = -1;   
			rdi.str     = ( IntPtr )( -1 );  // callback for names
			rdi.nCol    = 0;

			rdi.lParam = GetItemLParam( nRow );
			ResultData.InsertItem( ref rdi );
		}
		/// <summary>
		/// Overridable helper method to SetItem
		/// </summary>
		/// <param name="row"></param>
		/// <param name="nRow"></param>
		public virtual void SetItem( object row, int nRow ){
			if( row != null ){
				int lParam = GetItemLParam( nRow );
				uint pItemID;

				RESULTDATAITEM rdi = new RESULTDATAITEM( );

				row.ToString();
				rdi.mask	= ( uint )RDI.STR | ( uint )RDI.PARAM;

				rdi.str     = ( IntPtr )( -1 );  // callback for names
				rdi.nCol    = 0;

				//Find ItemID
				ResultData.FindItemByLParam( lParam, out pItemID );
				rdi.lParam = lParam;
				rdi.itemID = ( int )pItemID;

				ResultData.SetItem( ref rdi );
			}
		}
		/// <summary>
		/// Overridable helper method to UpdateItem
		/// </summary>
		/// <param name="lParam"></param>
		public virtual void UpdateItem( int lParam )
		{
			uint pItemID;
			//Find ItemID
			ResultData.FindItemByLParam( lParam, out pItemID );
			ResultData.UpdateItem( pItemID );
		}
		/// <summary>
		/// Overridable helper method to DeleteItem
		/// </summary>
		/// <param name="lParam"></param>
		public virtual void DeleteItem( int lParam )
		{
			uint pItemID;
			//Find ItemID
			ResultData.FindItemByLParam( lParam, out pItemID );
			ResultData.DeleteItem( pItemID, 0 );
		}
		/// <summary>
		/// Overridable helper method to GetItem
		/// </summary>
		/// <param name="ResultDataItem"></param>
		/// <param name="pItemID"></param>
		public virtual void GetItem( out RESULTDATAITEM ResultDataItem, uint pItemID ){
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= ( uint )RDI.STR;							// STR is valid
			rdi.itemID	= ( int )pItemID;
			rdi.nCol	= 0;

			ResultData.GetItem( ref rdi );
			
			ResultDataItem = rdi;
		}
		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		public virtual RESULTDATAITEM GetSelectedItem( ){
			return GetSelectedItem( -1 );	// nIndex == -1 to start at first item
		}
		/// <summary>
		/// Overridable helper method to GetSelectedItem
		/// </summary>
		/// <param name="nIndex"></param>
		public virtual RESULTDATAITEM GetSelectedItem( int nIndex ){
			RESULTDATAITEM rdi = new RESULTDATAITEM();

			rdi.mask	= ( uint )RDI.STATE;		// nState is valid 
			rdi.nCol	= 0;
			rdi.nIndex  = nIndex;				// nIndex == -1 to start at first item
			rdi.nState  = ( uint )LVIS.SELECTED;	// only interested in selected items

			ResultData.GetNextItem( ref rdi );
			//out
			return rdi;
		}
		/// <summary>
		/// helper method to get item row ID by lParam
		/// </summary>
		/// <param name="lParam"></param>
		public virtual int GetTableRowID( int lParam ){
			return( lParam >> 16 ) - 1;
		}
		/// <summary>
		/// helper method to get item lParam by nRow
		/// The low word contains the cookie for the node, while the high word 
		/// contains the row number + 1 we're inserting
		/// </summary>
		/// <param name="nRow"></param>
		public virtual int GetItemLParam( int nRow ){
			return m_iCookie | ( nRow << 16 );
		}
		#endregion
		
		/*************************************************************
		 * 
		 *  this method I have to reload in any derived class
		 *  Code for example AS IS
		 * 
		 * ***********************************************************/


		/*
		public override void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem)
		{
			bool bCallbase = true;
			int nRow = 0;

            // the "cell" data
			if (Table != null)
			{
                //int nRow = (ResultDataItem.lParam >> 16) - 1;
				nRow = GetTableRowID(ResultDataItem.lParam);
				int nCol = ResultDataItem.nCol;

				if ((ResultDataItem.mask & (uint)RDI.STR) > 0)
				{
					string data = DisplayName;
					if (nRow >= 0 && nRow < Table.Rows.Count && nCol >= 0 && nCol < Table.Columns.Count)
					{
						data = Table.Rows[nRow].ItemArray[nCol].ToString();
						bCallbase = false;

						//System.Diagnostics.Trace.WriteLine(string.Format("GetDisplayInfo: data={0}, row={1}", data, nRow));
					}

					ResultDataItem.str = Marshal.StringToCoTaskMemUni(data);
				}
			}

            // the image - requires 2 images in the image list
			// small - 0
			// large - 1
			//Changed by Alexander Kachalkov
			// small - SmallImageIndex
            // large - LargeImageIndex
            if ((ResultDataItem.mask & (uint)RDI.IMAGE) > 0)
            {
                //int offset = 1;
				int offset = LargeImageIndex;
				if (IsUseSmallIcons())
					//offset = 0;
					offset = SmallImageIndex;

				if  (nRow == 44)
					offset = 2;
                
                ResultDataItem.nImage = (Cookie << 16) + offset;
            }

			if (bCallbase)
				base.GetDisplayInfo(ref ResultDataItem);
		}
		*/

	}
}
