using System;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Collections;
using System.Threading;

namespace Ironring.Management.MMC
{
	public class WizardBase : Form
	{
		///Submitted by Johnson Chu
		/// <summary>
		/// Allow go back to the first page
		/// </summary>
		public int SelectedPage
		{
			get
			{
				return this.tabWizardPages.SelectedIndex + 1;
			}
			set
			{
				this.tabWizardPages.SelectedIndex = value - 1;
			}
		}

		/**
		 * Required designer variable.
		 */
		private Container components = null;
		private TabControl tabWizardPages;
		private Button btnCancel;
		private Button btnNext;
		private Button btnBack;
		protected Bitmap m_Logo;

		protected ArrayList pages = new ArrayList();

		protected DialogResult m_CachedDialogResult = DialogResult.None;	
	
		/// <summary>
		/// Delegate
		/// </summary>
		public delegate void RepaintDelegate();	

		/// <summary>
		/// Delegate 
		/// </summary>
		public delegate void WizardAction(WizardPage[] pages);

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="title">WizardBases Title</param>
		public WizardBase(String title)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			this.Text = title;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
		}

		public virtual Bitmap Logo
		{
			set{m_Logo = value;}
			get{return m_Logo;}
		}

		/// <summary>
		/// Show the wizard. A new thread is created for this. When the 'finish'
		/// button is pressed, the wizardAction is Invoked().
		/// </summary>
		/// <param name="title">The forms title</param>
		/// <param name="logo">custom top right logo, null for none</param>
		/// <param name="pages">Array of WizardPages for this WizardsPage</param>
		/// <param name="wizardAction">The Method that will be executed after 'finish'</param>
		public static void ShowWizard(String title, Bitmap logo, WizardPage[] pages, WizardAction wizardAction) 
		{
			System.Diagnostics.Debug.WriteLine("WizardsBase::showWizard");
			Thread t = new Thread(new ThreadStart(new WizardThread(pages, wizardAction, title, logo).Run));
			t.Start();	
		}

		/// <summary>
		/// Dialog Result Cache
		/// </summary>
		public DialogResult CachedDialogResult
		{
			set{m_CachedDialogResult = value; } 
			get{return m_CachedDialogResult; }
		}

		/// <summary>
		/// Show a modal dialog with an exception.
		/// </summary>
		/// <param name="msg">Error Messsage</param>
		/// <param name="e">Exception that is caught</param>
		public static void ShowException(String msg, Exception e) 
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append(msg + e + "\n");
			sb.Append(e.StackTrace);
			System.Windows.Forms.MessageBox.Show(sb.ToString());
		}

		/// <summary>
		/// Adds a WizardPage to this WizardBase
		/// </summary>
		/// <param name="wp">The WizardPage</param>
		public void AddWizardPage(WizardPage wp)  
		{
			wp.WizardBase = this;
			wp.Add_WizardPageStateChange(new WizardPage.WizardPageChanged(this.CheckWizardStatus));
			this.pages.Add(wp);
		}	

		/// <summary>
		/// Adds a WizardPage to this WizardBase
		/// </summary>
		/// <param name="wp">The WizardPage</param>
		/// <param name="rebuild">reInit the WizardPages</param>
		public void AddWizardPage(WizardPage wp, bool rebuild)
		{
			this.AddWizardPage(wp);
			if(rebuild)
				InitPages();
		}	

		/// <summary>
		/// Removes a WizardPage to this WizardBase
		/// </summary>
		/// <param name="wp">The WizardPage</param>
		public void RemoveWizardPage(WizardPage wp) 
		{
			wp.Remove_WizardPageStateChange(new WizardPage.WizardPageChanged(this.CheckWizardStatus));
			wp.WizardBase = null;		
			this.pages.Remove(wp);
		}

		/// <summary>
		/// Removes a WizardPage to this WizardBase
		/// </summary>
		/// <param name="wp">The WizardPage</param>
		/// <param name="rebuild">reInit the WizardPages</param>
		public void RemoveWizardPage(WizardPage wp, bool rebuild) 
		{
			RemoveWizardPage(wp);
			if(rebuild)
				InitPages();
		}

		/// <summary>
		/// Method to check if a WizardPage is valid, if so the next button will be enabled
		/// </summary>
		/// <param name="wp">The WizardPage</param>
		public void CheckWizardStatus(WizardPage wp)
		{				
			try 
			{
				if(wp.WizardPageValid) 
				{
					btnNext.Enabled = true;
				} 
				else 
				{				
					btnNext.Enabled = false;
				}
			}
			catch(Exception e) 
			{
				WizardBase.ShowException("Failed to check the wizard", e);
			}
		}

		/// <summary>
		/// Get the tabpane to show our pages.
		/// </summary>
		public void InitPages() 
		{
			this.tabWizardPages.SuspendLayout();
			this.tabWizardPages.Controls.Clear();
			int count = 0;
			IEnumerator i=this.pages.GetEnumerator();
			while(i.MoveNext()) 
			{
				count++;
				WizardPage wp = (WizardPage)i.Current;
				TabPage tb = new TabPage("Page: " + count);
				tb.Controls.Add(wp);
				
				//custom logo
				if(m_Logo != null)
				{
					wp.pictWizLogo.Image = m_Logo.GetThumbnailImage(48,48,null,IntPtr.Zero);
				}

				this.tabWizardPages.Controls.Add(tb);
			}
			this.tabWizardPages.ResumeLayout(true);
			InitButtons();
		}

		/// <summary>
		/// Set the buttons to the correct value: (next/finish).
		/// </summary>
		protected void InitButtons()
		{
			try 
			{
				if( tabWizardPages.SelectedIndex == (tabWizardPages.TabCount - 1) ) 
				{
					btnNext.Text = "&Finish";
				} 
				else 
				{
					btnNext.Text = "&Next >";
				}

				if( tabWizardPages.SelectedIndex == 0 ) 
				{
					btnBack.Enabled = false;
				} 
				else 
				{
					btnBack.Enabled = true;
				}

				// Get the current page and check it
				WizardPage wp = (WizardPage)tabWizardPages.SelectedTab.Controls[0];
				CheckWizardStatus(wp);
			}
			catch(Exception e) 
			{
				WizardBase.ShowException("Cannot initButton", e);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		/// <param name="disposing">Are we disposing?</param>
		protected override void Dispose(bool disposing)
		{
			// Wake up the UI thread.
			lock(this) 
			{
				Monitor.PulseAll(this); 
			}
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code
		/**
		 * Required method for Designer support - do not modify 
		 * the contents of this method with the code editor.
		 */
		private void InitializeComponent()
		{
			this.tabWizardPages = new TabControl();
			this.btnCancel = new Button();
			this.btnNext = new Button();
			this.btnBack = new Button();
			this.SuspendLayout();
			// 
			// tabWizardPages
			// 
			this.tabWizardPages.Location =new System.Drawing.Point(-1, -23);
			this.tabWizardPages.Name = "tabWizardPages";
			this.tabWizardPages.SelectedIndex = 0;
			this.tabWizardPages.Size = new System.Drawing.Size(499, 367);
			this.tabWizardPages.TabIndex = 0;
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Location = new System.Drawing.Point(408, 352);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(80, 24);
			this.btnCancel.TabIndex = 1;
			this.btnCancel.Text = "&Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// btnNext
			// 
			this.btnNext.Location = new System.Drawing.Point(320, 352);
			this.btnNext.Name = "btnNext";
			this.btnNext.Size = new System.Drawing.Size(80, 24);
			this.btnNext.TabIndex = 2;
			this.btnNext.Text = "&Next >";
			this.btnNext.Click += new System.EventHandler(this.btnNext_Click) ;
			// 
			// btnBack
			// 
			this.btnBack.Location = new System.Drawing.Point(240, 352);
			this.btnBack.Name = "btnBack";
			this.btnBack.Size = new System.Drawing.Size(80, 24);
			this.btnBack.TabIndex = 3;
			this.btnBack.Text = "< &Back";
			this.btnBack.Click += new System.EventHandler(this.btnBack_Click);
			// 
			// WizardBase
			// 
			this.AcceptButton = this.btnNext;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(496, 381);
			this.Controls.Add(this.btnBack);
			this.Controls.Add(this.btnNext);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.tabWizardPages);
			this.FormBorderStyle = FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "WizardBase";
			this.TopMost = true;
			this.Click += new System.EventHandler(this.WizardBase_Click);
			this.VisibleChanged += new System.EventHandler(this.WizardBase_VisibleChanged);
			this.ResumeLayout(false);

		}
		#endregion

		private void btnBack_Click (object sender, System.EventArgs e)
		{
			int current = tabWizardPages.SelectedIndex;		
			if(current > 0) 
			{
				tabWizardPages.SelectedIndex = current - 1;
			}
			InitButtons();
		}

		private void btnNext_Click (Object sender, System.EventArgs e)
		{
			int current = tabWizardPages.SelectedIndex;
			int max = tabWizardPages.TabCount;
			if(current < (max-1)) 
			{
				tabWizardPages.SelectedIndex = current + 1;
				InitButtons();
			} 
			else 
			{
				// Done (clicked finish)
				this.CachedDialogResult = DialogResult.OK;
				this.Close();			
			}
		}

		private void btnCancel_Click (Object sender, System.EventArgs e)
		{
			this.CachedDialogResult = DialogResult.Cancel;
			this.Close();
		}

		private void WizardBase_Click (Object sender, System.EventArgs e)
		{
			this.Invoke(new RepaintDelegate(this.RepaintMe));
		}

		private void RepaintMe() 
		{
			this.Invalidate();
			this.Update();
		}

		private void WizardBase_VisibleChanged (Object sender, System.EventArgs e)
		{
			this.Invoke(new RepaintDelegate(this.RepaintMe));
		}
	}

	/// <summary>
	/// The form to show.
	/// </summary>
	class WizardThread
	{
		protected WizardBase m_Form;
		protected DialogResult formResult = DialogResult.None;
		protected WizardPage[] m_Pages;
		protected WizardBase.WizardAction m_Action;
		protected String m_Title;
		protected Bitmap m_Logo = null;

		public WizardThread(WizardPage[] pages, WizardBase.WizardAction act, String title, Bitmap logo)
		{
			m_Pages = pages;
			this.m_Action = act;
			this.m_Title = title;
			this.m_Logo = logo;
		}

		/// <summary>
		/// Start the thread and show the Window.
		/// </summary>
		public void Run() 
		{
			try 
			{
				m_Form = new WizardBase(m_Title);
				for(int i=0; i<m_Pages.Length; i++) 
				{
					m_Form.AddWizardPage(m_Pages[i]);
				}
				m_Form.Logo = m_Logo;
				m_Form.InitPages();
				System.Diagnostics.Debug.WriteLine("Showing wizard...");
				formResult = m_Form.ShowDialog();
				System.Diagnostics.Debug.WriteLine("Return from ShowDialog. Current result: " + m_Form.CachedDialogResult);
				try 
				{
					if(m_Form.CachedDialogResult == DialogResult.OK) 
					{
						this.m_Action(m_Pages);
					}
				}
				finally 
				{
					this.m_Form.Dispose();
				}
			}
			catch(Exception e)
			{
				WizardBase.ShowException("Caught an exception", e);
			}
		}

		/// <summary>
		/// Dialog Result Cache
		/// </summary>
		public DialogResult CachedDialogResult 
		{
			get {return m_Form.CachedDialogResult;}
		}

	}
}
