/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "StdAfx.h"

#include "spoofreg.h"       // for typedefs of registry API funtions
#include ".\regliemanager.h"
#include "regwalk.h"


// these strings should be lower-case
//
const HKEY RegLieManager::s_hkeyRootReal = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszReal = L"software\\microsoft\\mmc\\nodetypes\\";
const UINT RegLieManager::s_cchReal = (UINT)wcslen(RegLieManager::s_pwszReal);

const HKEY RegLieManager::s_hkeyRootCenteris = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszCenteris = L"software\\microsoft\\mmc\\nodetypes.centeris\\";

const HKEY RegLieManager::s_hkeyFalseNodeRoot = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszCrossPolinationNodeTypeGuid = L"{fe3e3ffd-3a56-420a-b93e-60f216a809ab}";
LPCWSTR RegLieManager::s_pwszComputerManagementSnapinGuid = L"SOFTWARE\\Microsoft\\MMC\\SnapIns\\{58221C67-EA27-11CF-ADCF-00AA00A80033}\\NodeTypes";


RegLieManager::RegLieManager(void)
{
    m_fShouldLie = FALSE;
    m_LieMap.SetAt(s_hkeyRootReal, 0);
}

RegLieManager::~RegLieManager(void)
{
}


void RegLieManager::SetShouldLie(BOOL fShouldLie)
{
    m_fShouldLie = fShouldLie;

    // TODO: currently half-baked... should clear out map when set to false?
}

// returns TRUE to accept the returned lie values
// returns FALSE to just call the real API
//
BOOL RegLieManager::HandleRegOpenExW(HKEY       hKey,
                                     LPCWSTR    lpSubKey,
                                     DWORD      ulOptions,
                                     REGSAM     samDesired,
                                     REG_OPEN_KEY_EXW_API_PTR lpRealRegOpenKeyExW,
                                     PHKEY      phkLieResult,
                                     PLONG      plLieReturnValue)
{
    ATLASSERT(lpRealRegOpenKeyExW);

    // TODO: do we need to do other cleanup on lpSubKey?
    // TODO: enable/disable lie feature

    if (!m_fShouldLie)
    {
        return FALSE;
    }

    UINT cchPreviousLength = 0;

    if (!lpSubKey || (L'\0' == lpSubKey[0]))
    {
        // no subkey, just call the real funtion (which will return
        // a copy of hKey)
        //
        return FALSE;
    }

    bool fFound = m_LieMap.Lookup(hKey, cchPreviousLength);
    if (!fFound)
    {
        // the base key they're starting with isn't in our
        // table of lies, so it must relate to a different
        // part of the registry than what we're concerned with
        //
        return FALSE;
    }

    // compare the new lpSubKey against the key of interest
    // (s_pwszReal).
    //
    ATLASSERT(cchPreviousLength < s_cchReal);

    UINT cchSubKey = (UINT)wcslen(lpSubKey);

    // the length of lpSubKey from the root of the registry hive,
    // as totalled across several calls
    //
    UINT cchTotalSubkey = cchSubKey + cchPreviousLength;

    // the number of characters different 
    //
    UINT cchCharsToCompare = min(s_cchReal - cchPreviousLength, cchSubKey);

    LPCWSTR pszRootRemaining = s_pwszReal + cchPreviousLength;
    int result = _wcsnicmp(pszRootRemaining, lpSubKey, cchCharsToCompare);
    if (0 != result)
    {
        // the strings are different, so we don't need to lie about them.
        //
        return FALSE;
    }

    // the strings do match up to the point we care about them, so
    // build a lie...
    //
    if (cchTotalSubkey == s_cchReal)
    {
        // we've exactly opened the key to lie about.
        // return a handle to the fake key.
        //
        *plLieReturnValue = lpRealRegOpenKeyExW(s_hkeyRootCenteris,
                                                s_pwszCenteris,
                                                ulOptions,
                                                samDesired,
                                                phkLieResult);
    }
    else
    {
        if (!m_fShouldLie)
        {
            return FALSE;
        }

        if (cchTotalSubkey < s_cchReal)
        {
            // they don't want the whole thing, just part leading
            // up to the lie... so return them something honest,
            // but keep the answer in our queue
            //
            LONG lResult = lpRealRegOpenKeyExW(s_hkeyRootCenteris,
                                            s_pwszCenteris,
                                            ulOptions,
                                            samDesired,
                                            phkLieResult);

            if (ERROR_SUCCESS == lResult)
            {
                m_LieMap.SetAt(*phkLieResult, cchCharsToCompare);
            }

            // return what they asked for, on success or failure
            //
            *plLieReturnValue = lResult;

        }
        else // (cchTotalSubkey > s_cchReal)
        {
            // they want a key beyond our fake-o-key
            // so, just use our fake-o-key as a base, and call
            // regopen...
            //
            HKEY hkResult = 0;

            HKEY hkCenteris = 0;
            LONG lResult = lpRealRegOpenKeyExW(s_hkeyRootCenteris,
                                            s_pwszCenteris,
                                            ulOptions,
                                            samDesired,
                                            &hkCenteris);

            if (ERROR_SUCCESS == lResult)
            {
                LPCWSTR pszSubKey2 = lpSubKey + cchCharsToCompare;

                lResult = lpRealRegOpenKeyExW(hkCenteris,
                                            pszSubKey2,
                                            ulOptions,
                                            samDesired,
                                            &hkResult);

                ::RegCloseKey(hkCenteris);
                hkCenteris = 0;
            }

            *phkLieResult = hkResult;
            *plLieReturnValue = lResult;
        }
    }

    return TRUE;
}


void RegLieManager::HandleRegClose(HKEY hKey)
{
    // it doesn't matter if the key was removed or not
    //
    m_LieMap.RemoveKey(hKey);
}



// We need to cross-polinate the node types from the Windows extensions
// section into the centeris section, and vice-versa.
//

/*
const HKEY RegLieManager::s_hkeyRootReal = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszReal = L"software\\microsoft\\mmc\\nodetypes\\";
const UINT RegLieManager::s_cchReal = (UINT)wcslen(RegLieManager::s_pwszReal);

const HKEY RegLieManager::s_hkeyRootCenteris = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszCenteris = L"software\\microsoft\\mmc\\nodetypes.centeris\\";


const HKEY RegLieManager::s_hkeyFalseNodeRoot = HKEY_LOCAL_MACHINE;
LPCWSTR RegLieManager::s_pwszCrossPolinationNodeTypeGuid = L"{fe3e3ffd-3a56-420a-b93e-60f216a809ab}";
LPCWSTR RegLieManager::s_pwszComputerManagementSnapinGuid = L"SOFTWARE\\Microsoft\\MMC\\SnapIns\\{58221C67-EA27-11CF-ADCF-00AA00A80033}\\NodeTypes";
*/


HRESULT OpenCrossPolinationRegKey(HKEY hkeyRoot,              // registry hive
                                  LPCWSTR pwszSubkey,         // subkey of "MMC\NodeTypes"
                                  LPCWSTR pszNodeTypeGuid,    // fake node type for "cross poliniation" guid
                                  REGSAM samDesiredCross,     // permissions to use when opening phKeyCross
                                  PHKEY phKeyNodeTypes,       // registry hkey to "node types" list, opened with read access
                                  PHKEY phKeyCross)           // registry hkey to "crossover" node type, opened with write access
{
    HKEY hKeyNodeTypes = 0;
    HKEY hKeyCross = 0;

    LONG lResult = ::RegOpenKeyExW(hkeyRoot,
                                   pwszSubkey,
                                   0,
                                   KEY_READ,
                                   &hKeyNodeTypes);
    if (ERROR_SUCCESS != lResult)
    {
        return HRESULT_FROM_LAST_WIN32_ERROR();
    }

    lResult = ::RegOpenKeyExW(hKeyNodeTypes,
                              pszNodeTypeGuid,
                              0,
                              samDesiredCross,
                              &hKeyCross);
    if (ERROR_SUCCESS != lResult)
    {
        // free the reg key we've already opened, then fail
        //
        ::RegCloseKey(hKeyNodeTypes);
        return HRESULT_FROM_LAST_WIN32_ERROR();
    }


    *phKeyNodeTypes = hKeyNodeTypes;
    *phKeyCross = hKeyCross;

    return S_OK;
}

HRESULT RegLieManager::PopulateFakeNodeType()
{
    HRESULT hr = S_OK;

    // build a list of the nodetypes from the computer management snapin.
    // for each nodetype, open its list of extension snap-ins, and copy that
    // tree to the s_pwszCrossPolinationNodeTypeGuid under s_pwszCenteris.
    // be sure to exclude the fake s_pwszCrossPolinationNodeTypeGuid when doing this.
    //
    // then, do the dual...
    // walk the nodetypes from the computer management snapin in s_pwszCenteris,
    // and copy them all to the s_pwszCrossPolinationNodeTypeGuid under s_cchReal
    //
    HKEY hKeyNodeTypes = 0;
    HKEY hKeyRealNodeTypes = 0;
    HKEY hKeyCenterisNodeTypes = 0;
    HKEY hKeyRealCross = 0;
    HKEY hKeyCenterisCross = 0;
    LONG lResult = ::RegOpenKeyExW(s_hkeyFalseNodeRoot,                    // HKLM
                                   s_pwszComputerManagementSnapinGuid,     // list of "NodeTypes" for the Comp Mgmt snap-in
                                   0,                                      // options, must be zero
                                   KEY_READ,                               // desired access rights
                                   &hKeyNodeTypes);
    if (ERROR_SUCCESS != lResult)
    {
        hKeyNodeTypes = 0;
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }


    hr = OpenCrossPolinationRegKey(s_hkeyRootReal,              // registry hive
                                   s_pwszReal,                  // subkey of "MMC\NodeTypes"
                                   s_pwszCrossPolinationNodeTypeGuid,   // fake node type for "cross poliniation" guid
                                   KEY_WRITE,                   // permissions to use when opening phKeyCross
                                   &hKeyRealNodeTypes,          // node types hkey, opened with read access
                                   &hKeyRealCross);             // crossover hkey, opened with write access
    if (FAILED(hr))
    {
        hKeyRealNodeTypes = 0;
        hKeyRealCross = 0;
        goto error;
    }

    hr = OpenCrossPolinationRegKey(s_hkeyRootCenteris,          // registry hive
                                   s_pwszCenteris,              // subkey of "MMC\NodeTypes"
                                   s_pwszCrossPolinationNodeTypeGuid,   // fake node type for "cross poliniation" guid
                                   KEY_WRITE,                   // permissions to use when opening phKeyCross
                                   &hKeyCenterisNodeTypes,      // node types key, opened with read access
                                   &hKeyCenterisCross);         // crossover hkey, opened with write access
    if (FAILED(hr))
    {
        hKeyCenterisNodeTypes = 0;
        hKeyCenterisCross = 0;
        goto error;
    }


    // now hKeyNodeTypes is the list of all the nodes of the computer management
    // snap-in.
    //
    // walk both the "real" and "centeris" keys, building a list of extensions,
    // and copying any we find into the "crossover" extension list in the opposite key
    //
    lResult = ERROR_SUCCESS;
    for (LONG lKeyIndex = 0; ERROR_SUCCESS == lResult; ++lKeyIndex)
    {
        WCHAR wszNodeTypeName[MAX_KEY_LEN] ;
        DWORD cchSubkeyName = sizeof(wszNodeTypeName)/sizeof(wszNodeTypeName[0]);

        FILETIME ftLastWriteTime;                    // last write time
        lResult = ::RegEnumKeyExW(hKeyNodeTypes,
                                  lKeyIndex,         // index of subkey
                                  wszNodeTypeName,   // & to subkey name
                                  &cchSubkeyName,    // & to subkey len, in chars
                                  NULL,              // reserved
                                  NULL,              // & to class name str
                                  NULL,              // & to len of class name
                                  &ftLastWriteTime ) ;       // & for time last written
        if (ERROR_NO_MORE_ITEMS != lResult)
        {
            // skip over the cross-polination key
            //
            int result = _wcsicmp(wszNodeTypeName, s_pwszCrossPolinationNodeTypeGuid);
            if (0 == result)
            {
                // don't copy our cross-polination key into the other dest,
                // as it's unnecessary
                //
                continue;
            }
            if (ERROR_SUCCESS != lResult)
            {
                hr = HRESULT_FROM_LAST_WIN32_ERROR();
                goto error;
            }

            CopyRegKeyNames (hKeyCenterisNodeTypes, wszNodeTypeName, hKeyRealCross    );
            CopyRegKeyNames (hKeyRealNodeTypes,     wszNodeTypeName, hKeyCenterisCross);
        }
    }


error:
    if (hKeyNodeTypes)
    {
        ::RegCloseKey(hKeyNodeTypes);
    }
    if (hKeyRealNodeTypes)
    {
        ::RegCloseKey(hKeyRealNodeTypes);
    }
    if (hKeyCenterisNodeTypes)
    {
        ::RegCloseKey(hKeyCenterisNodeTypes);
    }
    if (hKeyRealCross)
    {
        ::RegCloseKey(hKeyRealCross);
    }
    if (hKeyCenterisCross)
    {
        ::RegCloseKey(hKeyCenterisCross);
    }
    return hr;
}

// enable all users on the machine to write to the s_pwszCrossPolinationNodeTypeGuid,
// under both the "real" and "centeris" keys.
//
// This is called at setup, when the installing user has permissions for all of HKLM.
//
HRESULT RegLieManager::EnablePermissiveRegistryKeys()
{
    HRESULT hr = S_OK;

    HKEY hKeyRealNodeTypes = 0;
    HKEY hKeyCenterisNodeTypes = 0;
    HKEY hKeyRealCross = 0;
    HKEY hKeyCenterisCross = 0;

    PSID pSid = NULL;
    PACL pDacl = NULL;
    SECURITY_DESCRIPTOR SecurityDescriptor;

    hr = OpenCrossPolinationRegKey(s_hkeyRootReal,              // registry hive
                                   s_pwszReal,                  // subkey of "MMC\NodeTypes"
                                   s_pwszCrossPolinationNodeTypeGuid,   // fake node type for "cross poliniation" guid
                                   WRITE_DAC,                   // permissions to use when opening phKeyCross
                                   &hKeyRealNodeTypes,          // node types hkey, opened with read access
                                   &hKeyRealCross);             // crossover hkey, opened with write access
    if (FAILED(hr))
    {
        hKeyRealNodeTypes = 0;
        hKeyRealCross = 0;
        goto error;
    }

    hr = OpenCrossPolinationRegKey(s_hkeyRootCenteris,          // registry hive
                                   s_pwszCenteris,              // subkey of "MMC\NodeTypes"
                                   s_pwszCrossPolinationNodeTypeGuid,   // fake node type for "cross poliniation" guid
                                   WRITE_DAC,                   // permissions to use when opening phKeyCross
                                   &hKeyCenterisNodeTypes,      // node types key, opened with read access
                                   &hKeyCenterisCross);         // crossover hkey, opened with write access
    if (FAILED(hr))
    {
        hKeyCenterisNodeTypes = 0;
        hKeyCenterisCross = 0;
        goto error;
    }

    {
        // initialize painfully complex SecurityDescritior
        // to enable full access for all users...
        //

        // GEE WILLICKERS!  THIS IS WORST DARN STUFF
        // I'VE SEEN IN A WEEK!  double check and document this stuff...
        //

        SID_IDENTIFIER_AUTHORITY sia = SECURITY_NT_AUTHORITY;
        BOOL fSucceeded = ::AllocateAndInitializeSid(&sia,
                                                     1,
                                                     SECURITY_INTERACTIVE_RID,
                                                     0,
                                                     0,
                                                     0,
                                                     0,
                                                     0,
                                                     0,
                                                     0,
                                                     &pSid);
        if (!fSucceeded)
        {
            goto error;
        }

        DWORD dwAclSize = sizeof(ACL) + sizeof(ACCESS_ALLOWED_ACE) + ::GetLengthSid(pSid) - sizeof(DWORD);
        pDacl = (PACL) new BYTE[dwAclSize];

        fSucceeded = ::InitializeAcl(pDacl, dwAclSize, ACL_REVISION);
        if (!fSucceeded)
        {
            goto error;
        }

        fSucceeded = ::AddAccessAllowedAceEx(pDacl,
                                             ACL_REVISION,
                                             CONTAINER_INHERIT_ACE,
                                             KEY_ALL_ACCESS,
                                             pSid);
        if (!fSucceeded)
        {
            goto error;
        }

        fSucceeded = ::InitializeSecurityDescriptor(&SecurityDescriptor,
                                                    SECURITY_DESCRIPTOR_REVISION);
        if (!fSucceeded)
        {
            goto error;
        }

        fSucceeded = ::SetSecurityDescriptorDacl(&SecurityDescriptor,
                                                 TRUE,
                                                 pDacl,
                                                 FALSE);
        if (!fSucceeded)
        {
            goto error;
        }
    }

    LONG lResult;
    lResult = ::RegSetKeySecurity(hKeyRealCross,
                                  DACL_SECURITY_INFORMATION,
                                  &SecurityDescriptor);
    if (ERROR_SUCCESS != lResult)
    {
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }

    lResult = ::RegSetKeySecurity(hKeyCenterisCross,
                                  DACL_SECURITY_INFORMATION,
                                  &SecurityDescriptor);
    if (ERROR_SUCCESS != lResult)
    {
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }


error:
    if (hKeyRealNodeTypes)
    {
        ::RegCloseKey(hKeyRealNodeTypes);
    }
    if (hKeyCenterisNodeTypes)
    {
        ::RegCloseKey(hKeyCenterisNodeTypes);
    }
    if (hKeyRealCross)
    {
        ::RegCloseKey(hKeyRealCross);
    }
    if (hKeyCenterisCross)
    {
        ::RegCloseKey(hKeyCenterisCross);
    }
    if (pSid)
    {
        ::FreeSid(pSid);
    }
    if (pDacl)
    {
        delete [] (BYTE*)pDacl;
    }
    return hr;
}

HRESULT RecurseDeleteKey(HKEY hkeyRoot, LPCWSTR pwszSubkey, LPCWSTR pszSubSubkey)
{
    HKEY hKeyParent = 0;
    LONG lResult = ::RegOpenKeyExW(hkeyRoot,
                                   pwszSubkey,
                                   0,
                                   KEY_READ,
                                   &hKeyParent);
    if (ERROR_SUCCESS != lResult)
    {
        return HRESULT_FROM_LAST_WIN32_ERROR();
    }

    // the CRegKey destructor will free hKeyParent
    //
    CRegKey key(hKeyParent);

    // bugbug: I thought we were using the new version of ATL?
    USES_CONVERSION;
    lResult = key.RecurseDeleteKey(W2T(pszSubSubkey));

    // that we've already been cleaned
    //
    if (ERROR_FILE_NOT_FOUND == lResult)
    {
        lResult = ERROR_SUCCESS;
    }

    return HRESULT_FROM_WIN32(lResult);
}

HRESULT RegLieManager::CleanupCenterisNodeTypes()
{
    HRESULT hr;

    // remove our "cross polination" node type deom the real MMC NodeTypes registry section
    //
    hr = RecurseDeleteKey(s_hkeyRootReal, s_pwszReal, s_pwszCrossPolinationNodeTypeGuid);

    // remove the entire NodeTypes.Centeris subtree
    //
    if (SUCCEEDED(hr))
    {
        hr = RecurseDeleteKey(s_hkeyRootCenteris, NULL, s_pwszCenteris);
    }
    return hr;
}

