/*
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at 
 * http://www.opensource.org/licenses/cddl1.php
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the original License file at src/license.txt.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[ ]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 *
 * Copyright (c) 2005 by Centeris, Inc.  All rights reserved.
 * Use is subject to license terms.
 * Initial Developer updates, news, and information may be found at 
 * http://sourceforge.net/projects/likewiseopenagt
 */

#include "stdafx.h"

#include <atlstr.h>

#include "ComUtil.h"

//*****************************************************************
//** Utility functions
//**

typedef int (__stdcall *PDLLGETCLSOBJ)(REFCLSID rclsid, REFIID riid, LPVOID* ppv);

// this will store the path to the dll implemenation

HRESULT GetPathToInprocServer32(REFCLSID refClsid,
                                LPTSTR ppszModuleName,
                                int cchModuleName)
{
    HRESULT hr = S_OK;

    // we want to build a string of the form:
    //
    //     CLSID\{c200e360-38c5-11ce-ae62-08002b2b79ef}\InprocServer32
    //
    // where "{c200...}" is the string representation of "rclsid".
    //
    // This represents the registry subkey which COM ordains to
    // store the DLL in which this CLSID is implemented.
    //
    // If this CLSID is not registered on the machine, or if the
    // COM object is not implemented as an In-Proc DLL, then this
    // subkey will not exist.
    //

    // first build the string.
    //
    WCHAR wstrGuid[120];
    const int cchGuid = sizeof(wstrGuid)/sizeof(wstrGuid[0]);

    int result = ::StringFromGUID2(refClsid,
                                    wstrGuid,
                                    cchGuid);
    // we should always have enough space to store the guid in our
    // fixed-sized buffer
    ATLASSERT(0 != result);

    CString cstrSubkey;
    cstrSubkey.Format(_T("CLSID\\%s\\InprocServer32"),
                        CW2T(wstrGuid));

    // now, strSubKey should be a string of the form
    //
    //     CLSID\{c200e360-38c5-11ce-ae62-08002b2b79ef}\InprocServer32
    //
    // query the registry and try to open this key.
    //

    HKEY hKey = 0;
    result = ::RegOpenKeyEx(HKEY_CLASSES_ROOT,
                            cstrSubkey,
                            0,
                            KEY_READ,
                            &hKey);
    if (ERROR_SUCCESS != result)
    {
        // the key doesn't exist, so the object must not be registered,
        // at least not with this particular type of implementation.
        //
        ATLTRACE("Failed to open registry key for dll");
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }

    DWORD cbPath = cchModuleName * sizeof(ppszModuleName[0]);
    DWORD dwType = REG_NONE;

    // open default value, and get path to library
    //
    result = ::RegQueryValueEx(hKey,
                                _T(""),
                                NULL,
                                &dwType,
                                (LPBYTE)ppszModuleName,
                                &cbPath);
    if (ERROR_SUCCESS != result)
    {
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }
    if (REG_SZ != dwType)
    {
        ATLTRACE("Unexpected registry type for dll InprocServer");
        hr = E_UNEXPECTED;
        goto error;
    }

    // ensure that the returned string is null-terminated.
    //
    // If the data in the registry was exactly MAX_PATH characters
    // long, or greater, than the string won't be.
    //
    int cchPath = cbPath/sizeof(ppszModuleName[0]);
    if (cchPath >= cchModuleName)
    {
        cchPath = cchModuleName - 1;
    }
    ppszModuleName[cchPath] = _T('\0');


error:
    return hr;
}

HRESULT CoCreateInstanceIgnoringTreatAs(
                                    REFCLSID refClsid,
                                    IUnknown * punkOuter,
                                    REFIID riid,
                                    LPVOID * ppvObject,
                                    HMODULE * phModuleLoaded)
{
    IClassFactory * pClassFactory = NULL;
    boolean fLibraryLoaded = false;

    TCHAR pszModuleName[MAX_PATH];
    HRESULT hr = GetPathToInprocServer32(refClsid,
                                            pszModuleName,
                                            sizeof(pszModuleName)/sizeof(TCHAR));
    if (FAILED(hr))
    {
        // this HR is what the regular CoCreateInstance
        //
        hr = REGDB_E_CLASSNOTREG;
    }

    // Load the DLL into our address space, but only if it hasn't
    // been loaded already.
    //
    HMODULE hDll = ::GetModuleHandle(pszModuleName);
    if (!hDll)
    {
        hDll = ::LoadLibrary(pszModuleName);
        if (!hDll)
        {
            hr = HRESULT_FROM_LAST_WIN32_ERROR();
            goto error;
        }
        else
        {
            fLibraryLoaded = true;
        }
    }

    const LPCTSTR pszProcName = _T("DllGetClassObject");
    PDLLGETCLSOBJ pDllGetCLSObj = (PDLLGETCLSOBJ)::GetProcAddress(hDll, pszProcName);
    if (NULL == pDllGetCLSObj)
    {
        hr = HRESULT_FROM_LAST_WIN32_ERROR();
        goto error;
    }

    hr = (*pDllGetCLSObj)(refClsid, IID_IClassFactory, (LPVOID*)&pClassFactory);
    if (FAILED(hr))
    {
        pClassFactory = NULL;
        goto error;
    }
    ATLASSERT(pClassFactory);

    hr = pClassFactory->CreateInstance(punkOuter, riid, ppvObject);

error:
    if (pClassFactory)
    {
        pClassFactory->Release();
        pClassFactory = NULL;
    }

    // TODO: how to do this?
    //
    // release the library that we loaded.  If we managed to create an object
    // from the library, then this won't actually unload it.
    //
    // We only call FreeLibrary if we had previously called LoadLibrary,
    // since GetModuleHandle doesn't affect the library's refcount.
    //
    if (fLibraryLoaded)
    {
        ATLASSERT(hDll);
        ATLASSERT(*phModuleLoaded == NULL);
        *phModuleLoaded = hDll;
    }
    else
    {
        *phModuleLoaded = NULL;
    }
    return hr;
}

// Utility functions.  Some were taken from the Platform SDK sample
//
HRESULT ExtractData( IDataObject* piDataObject,
                     CLIPFORMAT   cfClipFormat,
                     BYTE*        pbData,
                     DWORD        cbData )
{
    HRESULT hr = S_OK;
    
    FORMATETC formatetc = {cfClipFormat, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
    STGMEDIUM stgmedium = {TYMED_HGLOBAL, NULL};
    
    stgmedium.hGlobal = ::GlobalAlloc(GPTR, cbData);

    if (NULL == stgmedium.hGlobal)
    {
        hr = E_OUTOFMEMORY;
        goto cleanup;
    }
    hr = piDataObject->GetDataHere( &formatetc, &stgmedium );
    if ( FAILED(hr) )
    {
        goto cleanup;
    }
    
    BYTE* pbNewData = reinterpret_cast<BYTE*>(stgmedium.hGlobal);
    if (NULL == pbNewData)
    {
        hr = E_UNEXPECTED;
        goto cleanup;
    }

    ::memcpy( pbData, pbNewData, cbData );

cleanup:
    if (NULL != stgmedium.hGlobal)
    {
        ::GlobalFree(stgmedium.hGlobal);
    }
    return hr;
} // ExtractData()
